@@ -20,13 +20,14 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: htmlchars.c 4861 2010-04-25 17:43:09Z eht16 $
+ * $Id: htmlchars.c 5354 2010-11-02 13:05:12Z ntrel $
  */
 
 /* HTML Characters plugin (Inserts HTML character entities like '&amp;') */
 
 #include "geanyplugin.h"
 #include <string.h>
+#include "SciLexer.h"
 
 
 GeanyData		*geany_data;
@@ -376,7 +377,7 @@ static void set_status(gboolean new_status)
 		g_free(config_dir);
 		g_key_file_free(config);
 	}
-}	
+}
 
 
 static void toggle_status(G_GNUC_UNUSED GtkMenuItem * menuitem)
@@ -422,11 +423,17 @@ static const gchar *get_entity(gchar *letter)
 static gboolean ht_editor_notify_cb(GObject *object, GeanyEditor *editor,
 									SCNotification *nt, gpointer data)
 {
+	gint lexer;
+
 	g_return_val_if_fail(editor != NULL, FALSE);
 
-	if (plugin_active != TRUE)
+	if (!plugin_active)
 		return FALSE;
 
+	lexer = sci_get_lexer(editor->sci);
+	if (lexer != SCLEX_HTML && lexer != SCLEX_XML)
+			return FALSE;
+
 	if (nt->nmhdr.code == SCN_CHARADDED)
 	{
 		gchar buf[7];
@@ -743,14 +750,14 @@ void plugin_init(GeanyData *data)
 	ui_add_document_sensitive(menu_item);
 
 	/* Add menuitem for html replacement functions*/
-	main_menu = gtk_menu_item_new_with_mnemonic(_("HTML Replacement"));
+	main_menu = gtk_menu_item_new_with_mnemonic(_("_HTML Replacement"));
 	gtk_widget_show_all(main_menu);
 	gtk_container_add(GTK_CONTAINER(geany->main_widgets->tools_menu), main_menu);
 
 	main_menu_submenu = gtk_menu_new();
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(main_menu), main_menu_submenu);
 
-	menu_htmltoggle = gtk_check_menu_item_new_with_mnemonic(_("_HTMLToggle"));
+	menu_htmltoggle = gtk_check_menu_item_new_with_mnemonic(_("_Auto-replace Special Characters"));
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menu_htmltoggle),
 		plugin_active);
 	gtk_container_add(GTK_CONTAINER(main_menu_submenu), menu_htmltoggle);
@@ -759,7 +766,7 @@ void plugin_init(GeanyData *data)
 			 G_CALLBACK(toggle_status), NULL);
 
 	menu_bulk_replace = gtk_menu_item_new_with_mnemonic(
-						_("Bulk replacement of special chars"));
+						_("_Replace Characters in Selection"));
 	g_signal_connect((gpointer) menu_bulk_replace, "activate",
 			 G_CALLBACK(replace_special_character_activated), NULL);
 
