@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: saveactions.c 4861 2010-04-25 17:43:09Z eht16 $
+ * $Id: saveactions.c 5323 2010-10-25 17:24:47Z ntrel $
  */
 
 
@@ -261,7 +261,7 @@ static void instantsave_document_new_cb(GObject *obj, GeanyDocument *doc, gpoint
 
 		doc->file_name = new_filename;
 
-		if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE)
+		if (doc->file_type->id == GEANY_FILETYPES_NONE)
 			document_set_filetype(doc, filetypes_lookup_by_name(instantsave_default_ft));
 
 		/* force saving the file to enable all the related actions(tab name, filetype, etc.) */
