@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: splitwindow.c 4518 2010-01-01 22:55:18Z eht16 $
+ * $Id: splitwindow.c 5351 2010-11-02 12:49:05Z ntrel $
  */
 
 /* Split Window plugin. */
@@ -27,10 +27,6 @@
 #include "geanyplugin.h"
 #include <string.h>
 
-#include "Scintilla.h"
-#include "ScintillaWidget.h"
-#include "SciLexer.h"
-
 
 PLUGIN_VERSION_CHECK(GEANY_API_VERSION)
 PLUGIN_SET_INFO(_("Split Window"), _("Splits the editor view into two windows."),
