@@ -1,8 +1,8 @@
 /*
  *      about.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: about.c 4046 2009-07-30 11:29:53Z ntrel $
+ * $Id: about.c 5321 2010-10-25 16:24:55Z ntrel $
  */
 
 /*
@@ -33,7 +33,6 @@
 #include "geanywraplabel.h"
 #include "main.h"
 
-static GtkWidget *gb_window = NULL;
 #include "gb.c"
 
 
@@ -41,52 +40,66 @@ static GtkWidget *gb_window = NULL;
 #define INFO "<span size=\"larger\" weight=\"bold\">%s</span>"
 #define CODENAME "<span weight=\"bold\">\"" GEANY_CODENAME "\"</span>"
 #define BUILDDATE "<span size=\"smaller\">%s</span>"
-#define COPYRIGHT "Copyright (c)  2005-2009\nEnrico Tröger\nNick Treleaven\nFrank Lanitz\nAll rights reserved."
+#define COPYRIGHT "Copyright (c)  2005-2010\nNick Treleaven\nEnrico Tröger\nFrank Lanitz\nAll rights reserved."
 
 const gchar *translators[][2] = {
-	{ "be_BY", "Yura Semashko &lt;yurand2@gmail.com&gt;" },
+	{ "ast", "Marcos Costales &lt;marcoscostales@gmail.com&gt;"},
+	{ "be_BY", "Yura Siamashka &lt;yurand2@gmail.com&gt;" },
 	{ "bg", "Dilyan Rusev &lt;dilyanrusev@gmail.com&gt;" },
 	{ "ca_ES", "Toni Garcia-Navarro &lt;topi@elpiset.net&gt;" },
 	{ "cs_CZ", "Petr Messner &lt;messa@messa.cz&gt;\nAnna Talianova &lt;anickat1@gmail.com&gt;" },
 	{ "de_DE", "Frank Lanitz &lt;frank@frank.uvena.de&gt;\nDominic Hopf &lt;dmaphy@googlemail.com&gt;" },
 	{ "el", "Stavros Temertzidis &lt;bullgr@gmail.com&gt;" },
 	{ "en_GB", "Jeff Bailes &lt;thepizzaking@gmail.com&gt;" },
-	{ "es", "Damián Viano &lt;debian@damianv.com.ar&gt;\nNacho Cabanes &lt;ncabanes@gmail.com&gt;" },
+	{ "es", "Antonio Jiménez González &lt;tonificante@hotmail.com&gt;\nLucas Vieites &lt;lucasvieites@gmail.com&gt;" },
 	{ "fi", "Harri Koskinen &lt;harri@fastmonkey.org&gt;\nJari Rahkonen &lt;jari.rahkonen@pp1.inet.fi&gt;" },
 	{ "fr", "Jean-Philippe Moal &lt;skateinmars@skateinmars.net&gt;" },
+	{ "gl", "José Manuel Castroagudín Silva &lt;chavescesures@gmail.com&gt;"},
 	{ "hu", "Gabor Kmetyko &lt;kg_kilo@freemail.hu&gt;" },
 	{ "it", "Max Baldinelli &lt;m.baldinelli@agora.it&gt;,\nDario Santomarco &lt;dariello@yahoo.it&gt;" },
 	{ "ja", "Tarot Osuji &lt;tarot@sdf.lonestar.org&gt;\nChikahiro Masami &lt;cmasa.z321@gmail.com&gt;" },
 	{ "ko", "Park Jang-heon &lt;dotkabi@gmail.com&gt;" },
+	{ "kk", "Baurzhan Muftakhidinov &lt;baurthefirst@gmail.com&gt;"},
 	{ "lb", "Laurent Hoeltgen &lt;hoeltgman@gmail.com&gt;" },
-	{ "nl", "Kurt De Bree &lt;kdebree@telenet.be&gt;" },
+	{ "nl", "Peter Scholtens &lt;peter.scholtens@xs4all.nl&gt;\nAyke van Laethem &lt;aykevanlaethem@gmail.com&gt;" },
 	{ "pl_PL", "Jarosław Foksa &lt;jfoksa@gmail.com&gt;" },
 	{ "pt_BR", "Alexandra Moreire &lt;alexandream@gmail.com&gt;\nAdrovane Marques Kade &lt;adrovane@gmail.com&gt;" },
 	{ "ro", "Alex Eftimie &lt;alex@rosedu.org&gt;" },
 	{ "ru_RU", "brahmann_ &lt;brahmann@pisem.net&gt;,\nNikita E. Shalaev &lt;nshalaev@eu.spb.ru&gt;" },
-	{ "sl_SI", "Jože Klepec &lt;joze.klepec@siol.net&gt;"},
+	{ "sl", "Jože Klepec &lt;joze.klepec@siol.net&gt;"},
 	{ "sv", "Tony Mattsson &lt;superxorn@gmail.com&gt;" },
 	{ "tr", "Gürkan Gür &lt;seqizz@gmail.com&gt;"},
 	{ "uk", "Boris Dibrov &lt;dibrov.bor@gmail.com&gt;" },
 	{ "vi_VN", "Clytie Siddall &lt;clytie@riverland.net.au&gt;" },
-	{ "zh_CN", "Dormouse Young &lt;mouselinux@163.com&gt;" },
+	{ "zh_CN", "Dormouse Young &lt;mouselinux@163.com&gt;,\nXhacker Liu &lt;liu.dongyuan@gmail.com&gt;" },
 	{ "zh_TW", "KoViCH &lt;kovich.ian@gmail.com&gt;" }
 };
 static const gint translators_len = G_N_ELEMENTS(translators);
 
 const gchar *prev_translators[][2] = {
+	{ "es", "Damián Viano &lt;debian@damianv.com.ar&gt;\nNacho Cabanes &lt;ncabanes@gmail.com&gt;" },
 	{ "pl_PL", "Jacek Wolszczak &lt;shutdownrunner@o2.pl&gt;" },
+	{ "nl", "Kurt De Bree &lt;kdebree@telenet.be&gt;" }
 };
 static const gint prev_translators_len = G_N_ELEMENTS(prev_translators);
 
 static const gchar *contributors =
-"Alexander Rodin, Andrew Rowland, Anh Phạm, blackdog, Bo Lorentsen, Bob Doan, Bronisław Białek, Catalin Marinas, "
-"Chris Macksey, Christoph Berg, Colomban Wendling, Conrad Steenberg, Daniel Richard G., Dave Moore, Dirk Weber, "
-"Elias Pschernig, Eugene Arshinov, Felipe Pena, François Cami, "
-"Giuseppe Torelli, Guillaume de Rorthais, Guillaume Hoffmann, Herbert Voss, Jason Oster, Jean-François Wauthy, Jeff Pohlmeyer, "
-"Jesse Mayes, John Gabriele, Jon Strait, Josef Whiter, Kevin Ellwood, Kristoffer A. Tjernås, Marko Peric, Matti Mårds, Moritz Barsnick, "
-"Peter Strand, Philipp Gildein, Pierre Joye, Rob van der Linde, Robert McGinley, Roland Baudin, S Jagannathan, Saleem Abdulrasool, "
-"Sebastian Kraft, Shiv, Slava Semushin, Stefan Oltmanns, Tamim, Thomas Martitz, Tomás Vírseda, "
+"Adam Ples, "
+"Alexander Rodin, Alexey Antipov, Andrew Rowland, Anh Phạm, blackdog, Bo Lorentsen, Bob Doan, "
+"Bronisław Białek, Can Koy, Catalin Marinas, "
+"Chris Macksey, Christoph Berg, Colomban Wendling, Conrad Steenberg, Daniel Richard G., "
+"Daniel Marjamaki, Dave Moore, "
+"Dimitar Zhekov, Dirk Weber, Elias Pschernig, Eric Forgeot, "
+"Erik de Castro Lopo, Eugene Arshinov, Felipe Pena, François Cami, "
+"Giuseppe Torelli, Guillaume de Rorthais, Guillaume Hoffmann, Herbert Voss, Jason Oster, "
+"Jean-François Wauthy, Jeff Pohlmeyer, Jesse Mayes, Jiří Techet, "
+"John Gabriele, Jon Senior, Jon Strait, Josef Whiter, "
+"Jörn Reder, Kelvin Gardiner, Kevin Ellwood, Kristoffer A. Tjernås, Lex Trotman, "
+"Manuel Bua, Mário Silva, Marko Peric, Matti Mårds, "
+"Moritz Barsnick, Ondrej Donek, Peter Strand, Philipp Gildein, Pierre Joye, Rob van der Linde, "
+"Robert McGinley, Roland Baudin, Ross McKay, S Jagannathan, Saleem Abdulrasool, "
+"Sebastian Kraft, Shiv, Slava Semushin, Stefan Oltmanns, Tamim, Taylor Venable, "
+"Thomas Huth, Thomas Martitz, Tomás Vírseda, "
 "Tyler Mulligan, Walery Studennikov, Yura Siamashka";
 
 
@@ -151,14 +164,14 @@ static GtkWidget *create_dialog(void)
 	gtk_widget_show(header_hbox);
 	gtk_container_add(GTK_CONTAINER(header_eventbox), header_hbox);
 	header_image = gtk_image_new();
-	gtk_box_pack_start(GTK_BOX(header_hbox), header_image, FALSE,FALSE,0);
+	gtk_box_pack_start(GTK_BOX(header_hbox), header_image, FALSE, FALSE, 0);
 	header_label = gtk_label_new(NULL);
 	gtk_label_set_use_markup(GTK_LABEL(header_label), TRUE);
 	/* print the subversion revision generated by ./configure if it is available */
 	g_snprintf(buffer, sizeof(buffer), HEADER, main_get_version_string());
 	gtk_label_set_markup(GTK_LABEL(header_label), buffer);
 	gtk_widget_show(header_label);
-	gtk_box_pack_start(GTK_BOX(header_hbox), header_label, FALSE,FALSE,0);
+	gtk_box_pack_start(GTK_BOX(header_hbox), header_label, FALSE, FALSE, 0);
 	header_eventbox_style_set(header_eventbox);
 	header_label_style_set(header_label);
 	g_signal_connect_after(header_eventbox, "style-set", G_CALLBACK(header_eventbox_style_set), NULL);
@@ -263,18 +276,18 @@ static GtkWidget *create_dialog(void)
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 	row++;
 
-	g_snprintf(buffer, sizeof(buffer), "Enrico Tröger - %s", _("maintainer"));
+	g_snprintf(buffer, sizeof(buffer), "Nick Treleaven - %s", _("maintainer"));
 	ROW(buffer, row, 0, 0, 0, 1);
 	row++;
-	ROW("&lt;enrico.troeger@uvena.de&gt;", row, 0, 0, 0, 1);
+	ROW("&lt;nick.treleaven@btinternet.com&gt;", row, 0, 0, 0, 1);
 	row++;
 	ROW("", row, 0, 0, 0, 0);
 	row++;
 
-	g_snprintf(buffer, sizeof(buffer), "Nick Treleaven - %s", _("developer"));
+	g_snprintf(buffer, sizeof(buffer), "Enrico Tröger - %s", _("developer"));
 	ROW(buffer, row, 0, 0, 0, 1);
 	row++;
-	ROW("&lt;nick.treleaven@btinternet.com&gt;", row, 0, 0, 0, 1);
+	ROW("&lt;enrico.troeger@uvena.de&gt;", row, 0, 0, 0, 1);
 	row++;
 	ROW("", row, 0, 0, 0, 0);
 	row++;
