@@ -19,12 +19,16 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: build.c 5176 2010-08-17 15:02:49Z ntrel $
+ * $Id: build.c 5481 2010-12-09 15:34:28Z ntrel $
  */
 
 /*
  * Build commands and menu items.
  */
+/* TODO: tidy code:
+ * Use intermediate pointers for common subexpressions.
+ * Replace defines with enums.
+ * Other TODOs in code. */
 
 #include "geany.h"
 #include "build.h"
@@ -152,25 +156,36 @@ static void add_menu_accel(GeanyKeyGroup *group, guint kb_id,
 /* convenience routines to access parts of GeanyBuildCommand */
 static gchar *id_to_str(GeanyBuildCommand *bc, gint id)
 {
-	return bc->entries[id];
+	switch (id)
+	{
+		case GEANY_BC_LABEL:
+			return bc->label;
+		case GEANY_BC_COMMAND:
+			return bc->command;
+		case GEANY_BC_WORKING_DIR:
+			return bc->working_dir;
+	}
+	g_assert(0);
+	return NULL;
 }
 
 
-static gchar *buildcmd_label(GeanyBuildCommand *bc)
+static void set_command(GeanyBuildCommand *bc, gint id, gchar *str)
 {
-	return id_to_str(bc, GEANY_BC_LABEL);
-}
-
-
-static gchar *buildcmd_cmd(GeanyBuildCommand *bc)
-{
-	return id_to_str(bc, GEANY_BC_COMMAND);
-}
-
-
-static gchar *buildcmd_working_dir(GeanyBuildCommand *bc)
-{
-	return id_to_str(bc, GEANY_BC_WORKING_DIR);
+	switch (id)
+	{
+		case GEANY_BC_LABEL:
+			setptr(bc->label, str);
+			break;
+		case GEANY_BC_COMMAND:
+			setptr(bc->command, str);
+			break;
+		case GEANY_BC_WORKING_DIR:
+			setptr(bc->working_dir, str);
+			break;
+		default:
+			g_assert(0);
+	}
 }
 
 
@@ -209,6 +224,7 @@ static gboolean printbuildcmds = PRINTBUILDCMDS;
 /* for debug only, print the commands structures in priority order */
 static void printfcmds(void)
 {
+#if 0
 	GeanyBuildCommand **cl[GEANY_GBG_COUNT][GEANY_BCS_COUNT] = {
 		/* GEANY_BCS_DEF, GEANY_BCS_FT, GEANY_BCS_HOME_FT, GEANY_BCS_PREF,
 		 * GEANY_BCS_FT_PROJ, GEANY_BCS_PROJ */
@@ -285,6 +301,7 @@ static void printfcmds(void)
 		}
 		printf("\n");
 	}
+#endif
 }
 
 
@@ -455,7 +472,7 @@ static GeanyBuildCommand *get_build_group(GeanyBuildSource src, GeanyBuildGroup
 }
 
 
-/** Remove the specified Build menu item.
+/* * Remove the specified Build menu item.
  *
  * Makes the specified menu item configuration no longer exist. This
  * is different to setting fields to blank because the menu item
@@ -490,7 +507,7 @@ void build_remove_menu_item(GeanyBuildSource src, GeanyBuildGroup grp, gint cmd)
 }
 
 
-/** Get the @a GeanyBuildCommand structure for the specified Build menu item.
+/* * Get the @a GeanyBuildCommand structure for the specified Build menu item.
  *
  * Get the command for any menu item specified by @a src, @a grp and @a cmd even if it is
  * hidden by higher priority commands.
@@ -517,7 +534,7 @@ GeanyBuildCommand *build_get_menu_item(GeanyBuildSource src, GeanyBuildGroup grp
 }
 
 
-/** Get the @a GeanyBuildCommand structure for the menu item.
+/* * Get the @a GeanyBuildCommand structure for the menu item.
  *
  * Get the current highest priority command specified by @a grp and @a cmd.  This is the one
  * that the menu item will use if activated.
@@ -706,7 +723,7 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 	/* set the build info for the message window */
 	g_free(build_info.dir);
 	build_info.dir = g_strdup(working_dir);
-	build_info.file_type_id = (doc == NULL) ? GEANY_FILETYPES_NONE : FILETYPE_ID(doc->file_type);
+	build_info.file_type_id = (doc == NULL) ? GEANY_FILETYPES_NONE : doc->file_type->id;
 	build_info.message_count = 0;
 
 #ifdef G_OS_WIN32
@@ -759,8 +776,6 @@ static GPid build_spawn_cmd(GeanyDocument *doc, const gchar *cmd, const gchar *d
 static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript, gint cmdindex)
 {
 	gchar *locale_filename = NULL;
-	gboolean have_project;
-	GeanyProject *project = app->project;
 	GeanyBuildCommand *cmd = NULL;
 	gchar *executable = NULL;
 	gchar *working_dir = NULL;
@@ -775,11 +790,10 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript,
 
 	locale_filename = utils_get_locale_from_utf8(doc->file_name);
 
-	have_project = project != NULL;
 	cmd = get_build_cmd(doc, GEANY_GBG_EXEC, cmdindex, NULL);
 
-	cmd_string = build_replace_placeholder(doc, buildcmd_cmd(cmd));
-	cmd_working_dir =  buildcmd_working_dir(cmd);
+	cmd_string = build_replace_placeholder(doc, cmd->command);
+	cmd_working_dir =  cmd->working_dir;
 	if (! NZV(cmd_working_dir))
 		cmd_working_dir = "%d";
 	working_dir = build_replace_placeholder(doc, cmd_working_dir); /* in utf-8 */
@@ -797,7 +811,7 @@ static gchar *prepare_run_script(GeanyDocument *doc, gchar **vte_cmd_nonscript,
 	}
 
 #ifdef HAVE_VTE
-	if (vte_info.load_vte && vc != NULL && vc->run_in_vte)
+	if (vte_info.have_vte && vc->run_in_vte)
 	{
 		if (vc->skip_run_script)
 		{
@@ -846,10 +860,10 @@ static GPid build_run_cmd(GeanyDocument *doc, gint cmdindex)
 	if (working_dir == NULL)
 		return (GPid) 0;
 
-	run_info[cmdindex].file_type_id = FILETYPE_ID(doc->file_type);
+	run_info[cmdindex].file_type_id = doc->file_type->id;
 
 #ifdef HAVE_VTE
-	if (vte_info.load_vte && vc != NULL && vc->run_in_vte)
+	if (vte_info.have_vte && vc->run_in_vte)
 	{
 		gchar *vte_cmd;
 
@@ -979,7 +993,10 @@ static void process_build_output_line(const gchar *str, gint color)
 	g_strchomp(msg);
 
 	if (! NZV(msg))
+	{
+		g_free(msg);
 		return;
+	}
 
 	if (build_parse_make_dir(msg, &tmp))
 	{
@@ -1138,7 +1155,7 @@ static void build_exit_cb(GPid child_pid, gint status, gpointer user_data)
 
 static void run_exit_cb(GPid child_pid, gint status, gpointer user_data)
 {
-	RunInfo *run_info_data = (RunInfo*)user_data;
+	RunInfo *run_info_data = user_data;
 
 	g_spawn_close_pid(child_pid);
 
@@ -1196,7 +1213,7 @@ static void build_command(GeanyDocument *doc, GeanyBuildGroup grp, gint cmd, gch
 	if (buildcmd == NULL)
 		return;
 
-	cmdstr = buildcmd_cmd(buildcmd);
+	cmdstr = buildcmd->command;
 
 	if (cmd_cat != NULL)
 	{
@@ -1208,7 +1225,7 @@ static void build_command(GeanyDocument *doc, GeanyBuildGroup grp, gint cmd, gch
 	else
 		full_command = cmdstr;
 
-	dir = build_replace_placeholder(doc, buildcmd_working_dir(buildcmd));
+	dir = build_replace_placeholder(doc, buildcmd->working_dir);
 	subs_command = build_replace_placeholder(doc, full_command);
 	build_info.grp = grp;
 	build_info.cmd = cmd;
@@ -1244,6 +1261,8 @@ static void on_build_menu_item(GtkWidget *w, gpointer user_data)
 	gint grp = GPOINTER_TO_GRP(user_data);
 	gint cmd = GPOINTER_TO_CMD(user_data);
 
+	g_signal_emit_by_name(geany_object, "build-start");
+
 	if (doc && doc->changed)
 		document_save_file(doc, FALSE);
 	if (grp == GEANY_GBG_NON_FT && cmd == GBO_TO_CMD(GEANY_GBO_CUSTOM))
@@ -1252,7 +1271,7 @@ static void on_build_menu_item(GtkWidget *w, gpointer user_data)
 
 		if (! dialog)
 		{
-			dialog = dialogs_show_input_persistent(_("Custom Text"),
+			dialog = dialogs_show_input_persistent(_("Custom Text"), GTK_WINDOW(main_widgets.window),
 				_("Enter custom text here, all entered text is appended to the command."),
 				build_info.custom_target, &on_make_custom_input_response);
 		}
@@ -1270,7 +1289,7 @@ static void on_build_menu_item(GtkWidget *w, gpointer user_data)
 			return;
 		}
 		bc = get_build_cmd(doc, grp, cmd, NULL);
-		if (bc != NULL && strcmp(buildcmd_cmd(bc), "builtin") == 0)
+		if (bc != NULL && strcmp(bc->command, "builtin") == 0)
 		{
 			gchar *uri;
 			if (doc == NULL)
@@ -1330,9 +1349,9 @@ static struct BuildMenuItemSpec {
 		GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT) + 1, NULL, on_build_menu_item},
 	{NULL, -1, MENU_SEPARATOR,
 		GBF_SEP_2, NULL, NULL},
-	{NULL, GEANY_KEYS_BUILD_NEXTERROR, MENU_NEXT_ERROR,
+	{GTK_STOCK_GO_DOWN, GEANY_KEYS_BUILD_NEXTERROR, MENU_NEXT_ERROR,
 		GBF_NEXT_ERROR, N_("_Next Error"), on_build_next_error},
-	{NULL, GEANY_KEYS_BUILD_PREVIOUSERROR, MENU_PREV_ERROR,
+	{GTK_STOCK_GO_UP, GEANY_KEYS_BUILD_PREVIOUSERROR, MENU_PREV_ERROR,
 		GBF_PREV_ERROR, N_("_Previous Error"), on_build_previous_error},
 	{NULL, -1, MENU_SEPARATOR,
 		GBF_SEP_3, NULL, NULL},
@@ -1405,14 +1424,14 @@ static void create_build_menu(BuildMenuItems *build_menu_items)
 			for (j = bs->build_cmd; j < build_groups_count[grp]; ++j)
 			{
 				GeanyBuildCommand *bc = get_build_cmd(NULL, grp, j, NULL);
-				const gchar *lbl = (bc == NULL) ? "" : buildcmd_label(bc);
+				const gchar *lbl = (bc == NULL) ? "" : bc->label;
 				create_build_menu_item(menu, keygroup, accel_group, bs, lbl, grp, j);
 			}
 		}
 		else
 		{
 			GeanyBuildCommand *bc = get_build_cmd(NULL, bs->build_grp, bs->build_cmd, NULL);
-			const gchar *lbl = (bc == NULL) ? "" : buildcmd_label(bc);
+			const gchar *lbl = (bc == NULL) ? "" : bc->label;
 			create_build_menu_item(menu, keygroup, accel_group, bs, lbl, bs->build_grp, bs->build_cmd);
 		}
 	}
@@ -1433,7 +1452,7 @@ static void geany_menu_item_set_label(GtkWidget *w, const gchar *label)
 }
 
 
-/** Update the build menu to reflect changes in configuration or status.
+/* * Update the build menu to reflect changes in configuration or status.
  *
  * Sets the labels and number of visible items to match the highest
  * priority configured commands.  Also sets sensitivity if build commands are
@@ -1501,7 +1520,7 @@ void build_menu_update(GeanyDocument *doc)
 					const gchar *label;
 					bc = get_build_cmd(doc, grp, cmd, NULL);
 					if (bc)
-						label = buildcmd_label(bc);
+						label = bc->label;
 					else
 						label = NULL;
 
@@ -1732,8 +1751,19 @@ typedef struct RowWidgets
 	gint grp;
 	gint cmd;
 	gboolean cleared;
+	gboolean used_dst;
 } RowWidgets;
 
+static GdkColor *insensitive_color;
+
+static void set_row_color(RowWidgets *r, GdkColor *color )
+{
+	enum GeanyBuildCmdEntries i;
+
+	for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
+		gtk_widget_modify_text(r->entries[i], GTK_STATE_NORMAL, color);
+};
+
 
 static void set_build_command_entry_text(GtkWidget *wid, const gchar *text)
 {
@@ -1746,7 +1776,7 @@ static void set_build_command_entry_text(GtkWidget *wid, const gchar *text)
 
 static void on_clear_dialog_row(GtkWidget *unused, gpointer user_data)
 {
-	RowWidgets *r = (RowWidgets*)user_data;
+	RowWidgets *r = user_data;
 	gint src;
 	enum GeanyBuildCmdEntries i;
 	GeanyBuildCommand *bc = get_next_build_cmd(NULL, r->grp, r->cmd, r->dst, &src);
@@ -1769,6 +1799,8 @@ static void on_clear_dialog_row(GtkWidget *unused, gpointer user_data)
 			set_build_command_entry_text(r->entries[i], "");
 		}
 	}
+	r->used_dst = FALSE;
+	set_row_color(r, insensitive_color);
 	r->cleared = TRUE;
 }
 
@@ -1779,20 +1811,37 @@ static void on_clear_dialog_regex_row(GtkEntry *regex, gpointer unused)
 }
 
 
-static void on_label_button_clicked(GtkWidget *wid)
+static void on_label_button_clicked(GtkWidget *wid, gpointer user_data)
 {
+	RowWidgets *r = user_data;
 	const gchar *old = gtk_button_get_label(GTK_BUTTON(wid));
-	gchar *str = dialogs_show_input(_("Set menu item label"), NULL, old);
+	/* FIXME: we should pass either build dialog or project dialog instead of NULL for parent */
+	gchar *str = dialogs_show_input(_("Set menu item label"),
+		NULL, NULL, old);
+
+	if (!str)
+		return;
 
 	gtk_button_set_label(GTK_BUTTON(wid), str);
 	g_free(str);
+	r->used_dst = TRUE;
+	set_row_color(r, NULL);
+}
+
+
+static void on_entry_focus(GtkWidget *wid, GdkEventFocus *unused, gpointer user_data)
+{
+	RowWidgets *r = user_data;
+
+	r->used_dst = TRUE;
+	set_row_color(r, NULL);
 }
 
 
 /* Column headings, array NULL-terminated */
 static const gchar *colheads[] =
 {
-	N_("Item"),
+	"#",
 	N_("Label"),
 	N_("Command"),
 	N_("Working directory"),
@@ -1817,8 +1866,12 @@ static RowWidgets *build_add_dialog_row(GeanyDocument *doc, GtkTable *table, gui
 	gint src;
 	enum GeanyBuildCmdEntries i;
 	guint column = 0;
+	gchar *text;
 
-	label = gtk_label_new(g_strdup_printf("%d:", cmd + 1));
+	text = g_strdup_printf("%d.", cmd + 1);
+	label = gtk_label_new(text);
+	g_free(text);
+	insensitive_color = &(gtk_widget_get_style(label)->text[GTK_STATE_INSENSITIVE]);
 	gtk_table_attach(table, label, column, column + 1, row, row + 1, GTK_FILL,
 		GTK_FILL | GTK_EXPAND, entry_x_padding, entry_y_padding);
 	roww = g_new0(RowWidgets, 1);
@@ -1836,10 +1889,13 @@ static RowWidgets *build_add_dialog_row(GeanyDocument *doc, GtkTable *table, gui
 			GtkWidget *wid = roww->entries[i] = gtk_button_new();
 			gtk_button_set_use_underline(GTK_BUTTON(wid), TRUE);
 			ui_widget_set_tooltip_text(wid, _("Click to set menu item label"));
-			g_signal_connect(wid, "clicked", G_CALLBACK(on_label_button_clicked), NULL);
+			g_signal_connect(wid, "clicked", G_CALLBACK(on_label_button_clicked), roww);
 		}
 		else
+		{
 			roww->entries[i] = gtk_entry_new();
+			g_signal_connect(roww->entries[i], "focus-in-event", G_CALLBACK(on_entry_focus), roww);
+		}
 		gtk_table_attach(table, roww->entries[i], column, column + 1, row, row + 1, xflags,
 			GTK_FILL | GTK_EXPAND, entry_x_padding, entry_y_padding);
 	}
@@ -1857,10 +1913,18 @@ static RowWidgets *build_add_dialog_row(GeanyDocument *doc, GtkTable *table, gui
 	for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
 	{
 		const gchar *str = "";
-		if (bc != NULL && (str = bc->entries[i]) == NULL)
-			str = "";
+
+		if (bc != NULL )
+		{
+			if ((str = id_to_str(bc, i)) == NULL)
+				str = "";
+			else if ((gint)dst == src)
+				roww->used_dst = TRUE;
+		}
 		set_build_command_entry_text(roww->entries[i], str);
 	}
+	if (bc != NULL && ((gint)dst > src))
+		set_row_color(roww, insensitive_color);
 	if (bc != NULL && (src > (gint)dst || (grp == GEANY_GBG_FT && (doc == NULL || doc->file_type == NULL))))
 	{
 		for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
@@ -1946,7 +2010,7 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_table_attach(table, sep, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, sep_padding);
 	++row;
-	label = ui_label_new_bold(_("Non-Filetype Commands"));
+	label = ui_label_new_bold(_("Independent Commands"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach(table, label, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, entry_y_padding);
@@ -1976,7 +2040,9 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_widget_set_sensitive(fields->nonfileregex, sensitivity);
 	gtk_widget_set_sensitive(clear, sensitivity);
 	++row;
-	label = gtk_label_new(_("Note: Item 2 opens a dialog and appends the response to the command."));
+	label = gtk_label_new(NULL);
+	ui_label_set_markup(GTK_LABEL(label), "<i>%s</i>",
+		_("Note: Item 2 opens a dialog and appends the response to the command."));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach(table, label, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, entry_y_padding);
@@ -1995,7 +2061,8 @@ GtkWidget *build_commands_table(GeanyDocument *doc, GeanyBuildSource dst, BuildT
 	gtk_table_attach(table, sep, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
 		entry_x_padding, sep_padding);
 	++row;
-	label = gtk_label_new(
+	label = gtk_label_new(NULL);
+	ui_label_set_markup(GTK_LABEL(label), "<i>%s</i>",
 		_("%d, %e, %f, %p are substituted in command and directory fields, see manual for details."));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach(table, label, 0, DC_N_COL, row, row + 1, GTK_FILL, GTK_FILL | GTK_EXPAND,
@@ -2019,6 +2086,7 @@ void build_free_fields(BuildTableData table_data)
 
 
 /* string compare where null pointers match null or 0 length strings */
+#if 0
 static gint stcmp(const gchar *a, const gchar *b)
 {
 	if (a == NULL && b == NULL)
@@ -2029,6 +2097,7 @@ static gint stcmp(const gchar *a, const gchar *b)
 		return strlen(a);
 	return strcmp(a, b);
 }
+#endif
 
 
 static const gchar *get_build_command_entry_text(GtkWidget *wid)
@@ -2064,35 +2133,14 @@ static gboolean read_row(BuildDestination *dst, BuildTableData table_data, gint
 			changed = TRUE;
 		}
 	}
-	if (
-			(
-				table_data->rows[drow]->cmdsrc == NULL		/* originally there was no content */
-				&&
-				(
-					NZV(entries[GEANY_BC_LABEL])			/* but now one field has some */
-					|| NZV(entries[GEANY_BC_COMMAND])
-					|| NZV(entries[GEANY_BC_WORKING_DIR])
-				)
-			)
-			||
-			(
-				table_data->rows[drow]->cmdsrc != NULL		/* originally there was content */
-				&&
-				(											/* and some of it was changed */
-					stcmp(entries[GEANY_BC_LABEL], table_data->rows[drow]->cmdsrc->entries[GEANY_BC_LABEL]) != 0
-					|| stcmp(entries[GEANY_BC_COMMAND], table_data->rows[drow]->cmdsrc->entries[GEANY_BC_COMMAND]) != 0
-					|| stcmp(entries[GEANY_BC_WORKING_DIR],
-							table_data->rows[drow]->cmdsrc->entries[GEANY_BC_WORKING_DIR]) != 0
-				)
-			)
-		)
+	if (table_data->rows[drow]->used_dst == TRUE)
 	{
 		if (dst->dst[grp] != NULL)
 		{
 			if (*(dst->dst[grp]) == NULL)
 				*(dst->dst[grp]) = g_new0(GeanyBuildCommand, build_groups_count[grp]);
 			for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
-				setptr((*(dst->dst[grp]))[cmd].entries[i], entries[i]);
+				set_command(&(*(dst->dst[grp]))[cmd], i, entries[i]);
 			(*(dst->dst[grp]))[cmd].exists = TRUE;
 			(*(dst->dst[grp]))[cmd].changed = TRUE;
 			changed = TRUE;
@@ -2110,23 +2158,17 @@ static gboolean read_row(BuildDestination *dst, BuildTableData table_data, gint
 static gboolean read_regex(GtkWidget *regexentry, gchar **src, gchar **dst)
 {
 	gboolean changed = FALSE;
-	gchar *reg = g_strdup(gtk_entry_get_text(GTK_ENTRY(regexentry)));
+	const gchar *reg = gtk_entry_get_text(GTK_ENTRY(regexentry));
 
-	if (
-			(
-				(src == NULL			/* originally there was no regex */
-				 || *src == NULL		/* or it was NULL*/
-				)
-				&& NZV(reg) > 0			/* and something was typed */
-			)
-			||(src != NULL				/* originally there was a regex*/
-			&& strcmp(*src, reg) != 0 	/* and it has been changed */
-			)
-		)
+	if (((src == NULL			/* originally there was no regex */
+		|| *src == NULL)		/* or it was NULL*/
+		&& NZV(reg))			/* and something was typed */
+		|| (src != NULL				/* originally there was a regex*/
+		&& strcmp(*src, reg) != 0))	/* and it has been changed */
 	{
 		if (dst != NULL)
 		{
-			setptr(*dst, reg);
+			setptr(*dst, g_strdup(reg));
 			changed = TRUE;
 		}
 	}
@@ -2234,18 +2276,6 @@ BuildMenuItems *build_get_menu_items(gint filetype_idx)
 }
 
 
-/* set non_ft working directory entries to %p for project */
-void build_set_non_ft_wd_to_proj(BuildTableData table_data)
-{
-	gint i, start, end;
-
-	start = build_groups_count[GEANY_GBG_FT];
-	end = start + build_groups_count[GEANY_GBG_NON_FT];
-	for (i = start; i < end; ++i)
-		gtk_entry_set_text(GTK_ENTRY(table_data->rows[i]->entries[GEANY_BC_WORKING_DIR]), "%p");
-}
-
-
 /*----------------------------------------------------------
  *
  * Load and store configuration
@@ -2293,12 +2323,12 @@ static void build_load_menu_grp(GKeyFile *config, GeanyBuildCommand **dst, gint
 		if (label != NULL)
 		{
 			dstcmd[cmd].exists = TRUE;
-			setptr(dstcmd[cmd].entries[GEANY_BC_LABEL], label);
+			setptr(dstcmd[cmd].label, label);
 			set_key_fld(key,"CM");
-			setptr(dstcmd[cmd].entries[GEANY_BC_COMMAND],
+			setptr(dstcmd[cmd].command,
 					g_key_file_get_string(config, build_grp_name, key, NULL));
 			set_key_fld(key,"WD");
-			setptr(dstcmd[cmd].entries[GEANY_BC_WORKING_DIR],
+			setptr(dstcmd[cmd].working_dir,
 					g_key_file_get_string(config, build_grp_name, key, NULL));
 		}
 		else dstcmd[cmd].exists = FALSE;
@@ -2384,12 +2414,13 @@ void build_load_menu(GKeyFile *config, GeanyBuildSource src, gpointer p)
 	/* load old [build_settings] values if there is no value defined by [build-menu] */
 
 	/* set GeanyBuildCommand if it doesn't already exist and there is a command */
+/* TODO: rewrite as function */
 #define ASSIGNIF(type, id, string, value) \
 	if (NZV(value) && ! type[GBO_TO_CMD(id)].exists) { \
 		type[GBO_TO_CMD(id)].exists = TRUE; \
-		setptr(type[GBO_TO_CMD(id)].entries[GEANY_BC_LABEL], g_strdup(string)); \
-		setptr(type[GBO_TO_CMD(id)].entries[GEANY_BC_COMMAND], (value)); \
-		setptr(type[GBO_TO_CMD(id)].entries[GEANY_BC_WORKING_DIR], NULL); \
+		setptr(type[GBO_TO_CMD(id)].label, g_strdup(string)); \
+		setptr(type[GBO_TO_CMD(id)].command, (value)); \
+		setptr(type[GBO_TO_CMD(id)].working_dir, NULL); \
 		type[GBO_TO_CMD(id)].old = TRUE; \
 	} else \
 		g_free(value);
@@ -2434,11 +2465,11 @@ void build_load_menu(GKeyFile *config, GeanyBuildSource src, gpointer p)
 			else
 				makebasedir = g_strdup("%d");
 			if (non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_ALL)].old)
-				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_ALL)].entries[GEANY_BC_WORKING_DIR], g_strdup(makebasedir));
+				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_ALL)].working_dir, g_strdup(makebasedir));
 			if (non_ft_pref[GBO_TO_CMD(GEANY_GBO_CUSTOM)].old)
-				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_CUSTOM)].entries[GEANY_BC_WORKING_DIR], g_strdup(makebasedir));
+				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_CUSTOM)].working_dir, g_strdup(makebasedir));
 			if (non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT)].old)
-				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT)].entries[GEANY_BC_WORKING_DIR], g_strdup("%d"));
+				setptr(non_ft_pref[GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT)].working_dir, g_strdup("%d"));
 			value = g_key_file_get_string(config, "project", "run_cmd", NULL);
 			if (NZV(value))
 			{
@@ -2447,9 +2478,9 @@ void build_load_menu(GKeyFile *config, GeanyBuildSource src, gpointer p)
 				if (! exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].exists)
 				{
 					exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].exists = TRUE;
-					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].entries[GEANY_BC_LABEL], g_strdup(_("_Execute")));
-					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].entries[GEANY_BC_COMMAND], value);
-					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].entries[GEANY_BC_WORKING_DIR], g_strdup(basedir));
+					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].label, g_strdup(_("_Execute")));
+					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].command, value);
+					setptr(exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].working_dir, g_strdup(basedir));
 					exec_proj[GBO_TO_CMD(GEANY_GBO_EXEC)].old = TRUE;
 				}
 			}
@@ -2502,7 +2533,7 @@ static gint build_save_menu_grp(GKeyFile *config, GeanyBuildCommand *src, gint g
 				for (i = 0; i < GEANY_BC_CMDENTRIES_COUNT; i++)
 				{
 					set_key_fld(key, config_keys[i]);
-					g_key_file_set_string(config, build_grp_name, key, src[cmd].entries[i]);
+					g_key_file_set_string(config, build_grp_name, key, id_to_str(&src[cmd], i));
 				}
 			}
 			else
@@ -2529,8 +2560,8 @@ typedef struct ForEachData
 
 static void foreach_project_filetype(gpointer data, gpointer user_data)
 {
-	GeanyFiletype *ft = (GeanyFiletype*) data;
-	ForEachData *d = (ForEachData*) user_data;
+	GeanyFiletype *ft = data;
+	ForEachData *d = user_data;
 	gint i = 0;
 	gchar *regkey = g_strdup_printf("%serror_regex", ft->name);
 
@@ -2631,15 +2662,17 @@ static void on_project_close(void)
 
 static struct
 {
-	const gchar *entries[GEANY_BC_CMDENTRIES_COUNT];
+	const gchar *label;
+	const gchar *command;
+	const gchar *working_dir;
 	GeanyBuildCommand **ptr;
 	gint index;
 } default_cmds[] = {
-	{ {N_("_Make"), "make", NULL}, &non_ft_def, GBO_TO_CMD(GEANY_GBO_MAKE_ALL)},
-	{ {N_("Make Custom _Target"), "make ", NULL}, &non_ft_def, GBO_TO_CMD(GEANY_GBO_CUSTOM)},
-	{ {N_("Make _Object"), "make %e.o", NULL }, &non_ft_def, GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT)},
-	{ {N_("_Execute"), "./%e", NULL }, &exec_def, GBO_TO_CMD(GEANY_GBO_EXEC)},
-	{ {NULL, NULL, NULL}, NULL, 0 }
+	{ N_("_Make"), "make", NULL, &non_ft_def, GBO_TO_CMD(GEANY_GBO_MAKE_ALL)},
+	{ N_("Make Custom _Target"), "make ", NULL, &non_ft_def, GBO_TO_CMD(GEANY_GBO_CUSTOM)},
+	{ N_("Make _Object"), "make %e.o", NULL, &non_ft_def, GBO_TO_CMD(GEANY_GBO_MAKE_OBJECT)},
+	{ N_("_Execute"), "./%e", NULL, &exec_def, GBO_TO_CMD(GEANY_GBO_EXEC)},
+	{ NULL, NULL, NULL, NULL, 0 }
 };
 
 
@@ -2655,18 +2688,14 @@ void build_init(void)
 	non_ft_def = g_new0(GeanyBuildCommand, build_groups_count[GEANY_GBG_NON_FT]);
 	exec_def = g_new0(GeanyBuildCommand, build_groups_count[GEANY_GBG_EXEC]);
 	run_info = g_new0(RunInfo, build_groups_count[GEANY_GBG_EXEC]);
-	for (cmdindex = 0; default_cmds[cmdindex].entries[GEANY_BC_COMMAND] != NULL; ++cmdindex)
+
+	for (cmdindex = 0; default_cmds[cmdindex].command != NULL; ++cmdindex)
 	{
-		enum GeanyBuildCmdEntries k;
 		GeanyBuildCommand *cmd = &((*(default_cmds[cmdindex].ptr))[ default_cmds[cmdindex].index ]);
 		cmd->exists = TRUE;
-		for (k = 0; k < GEANY_BC_CMDENTRIES_COUNT; k++)
-		{
-			if (k == GEANY_BC_LABEL)
-				cmd->entries[k] = g_strdup(_(default_cmds[cmdindex].entries[k]));
-			else
-				cmd->entries[k] = g_strdup(default_cmds[cmdindex].entries[k]);
-		}
+		cmd->label = g_strdup(_(default_cmds[cmdindex].label));
+		cmd->command = g_strdup(default_cmds[cmdindex].command);
+		cmd->working_dir = g_strdup(default_cmds[cmdindex].working_dir);
 	}
 
 	/* create the toolbar Build item sub menu */
