@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.c 4894 2010-05-09 15:48:55Z eht16 $
+ * $Id: callbacks.c 5470 2010-12-07 13:02:46Z ntrel $
  */
 
 /*
@@ -485,7 +485,9 @@ on_reload_as_activate                  (GtkMenuItem     *menuitem,
 		charset = doc->encoding;
 
 	base_name = g_path_get_basename(doc->file_name);
-	if (dialogs_show_question_full(NULL, _("_Reload"), GTK_STOCK_CANCEL,
+	/* don't prompt if file hasn't been edited at all */
+	if ((!doc->changed && !document_can_undo(doc) && !document_can_redo(doc)) ||
+		dialogs_show_question_full(NULL, _("_Reload"), GTK_STOCK_CANCEL,
 		_("Any unsaved changes will be lost."),
 		_("Are you sure you want to reload '%s'?"), base_name))
 	{
@@ -1015,6 +1017,8 @@ static void find_usage(gboolean in_session)
 	}
 	else
 	{
+		editor_find_current_word(doc->editor, -1,
+			editor_info.current_word, GEANY_MAX_WORD_LENGTH, NULL);
 		search_text = g_strdup(editor_info.current_word);
 		flags = SCFIND_MATCHCASE | SCFIND_WHOLEWORD;
 	}
@@ -1040,12 +1044,8 @@ on_find_usage1_activate                (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_goto_tag_activate                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+static void goto_tag(gboolean definition)
 {
-	gboolean definition = (menuitem ==
-		GTK_MENU_ITEM(ui_lookup_widget(main_widgets.editor_menu, "goto_tag_definition1")));
 	GeanyDocument *doc = document_get_current();
 
 	g_return_if_fail(doc != NULL);
@@ -1062,6 +1062,22 @@ on_goto_tag_activate                   (GtkMenuItem     *menuitem,
 }
 
 
+void
+on_goto_tag_definition1                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	goto_tag(TRUE);
+}
+
+
+void
+on_goto_tag_declaration1               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	goto_tag(FALSE);
+}
+
+
 void
 on_count_words1_activate               (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -1102,33 +1118,11 @@ on_find1_activate                      (GtkMenuItem     *menuitem,
 }
 
 
-static void find_again(gboolean change_direction)
-{
-	GeanyDocument *doc = document_get_current();
-
-	g_return_if_fail(doc != NULL);
-
-	if (search_data.text)
-	{
-		gboolean forward = ! search_data.backwards;
-		gint result = document_find_text(doc, search_data.text, search_data.flags,
-			change_direction ? forward : !forward, FALSE, NULL);
-
-		if (result > -1)
-			editor_display_current_line(doc->editor, 0.3F);
-
-		if (search_data.search_bar)
-			ui_set_search_entry_background(
-				toolbar_get_widget_child_by_name("SearchEntry"), (result > -1));
-	}
-}
-
-
 void
 on_find_next1_activate                 (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
-	find_again(FALSE);
+	search_find_again(FALSE);
 }
 
 
@@ -1140,7 +1134,7 @@ on_find_previous1_activate             (GtkMenuItem     *menuitem,
 		/* Can't reverse search order for a regex (find next ignores search backwards) */
 		utils_beep();
 	else
-		find_again(TRUE);
+		search_find_again(TRUE);
 }
 
 
@@ -1199,7 +1193,8 @@ on_go_to_line_activate                 (GtkMenuItem     *menuitem,
 	gchar *result;
 
 	result = dialogs_show_input_goto_line(
-		_("Go to Line"), _("Enter the line you want to go to:"), value);
+		_("Go to Line"), GTK_WINDOW(main_widgets.window),
+		_("Enter the line you want to go to:"), value);
 	if (result != NULL)
 	{
 		GeanyDocument *doc = document_get_current();
@@ -1442,11 +1437,12 @@ on_insert_date_activate                (GtkMenuItem     *menuitem,
 		format = ui_prefs.custom_date_format;
 	else
 	{
-		setptr(ui_prefs.custom_date_format,
-			dialogs_show_input(_("Custom Date Format"),
+		gchar *str = dialogs_show_input(_("Custom Date Format"), GTK_WINDOW(main_widgets.window),
 				_("Enter here a custom date and time format. "
 				"You can use any conversion specifiers which can be used with the ANSI C strftime function."),
-				ui_prefs.custom_date_format));
+				ui_prefs.custom_date_format);
+		if (str)
+			setptr(ui_prefs.custom_date_format, str);
 		return;
 	}
 
@@ -1581,7 +1577,10 @@ on_menu_show_sidebar1_toggled          (GtkCheckMenuItem *checkmenuitem,
 
 	ui_prefs.sidebar_visible = ! ui_prefs.sidebar_visible;
 
-	if (! interface_prefs.sidebar_openfiles_visible && ! interface_prefs.sidebar_symbol_visible)
+	/* show built-in tabs if no tabs visible */
+	if (ui_prefs.sidebar_visible &&
+		! interface_prefs.sidebar_openfiles_visible && ! interface_prefs.sidebar_symbol_visible &&
+		gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.sidebar_notebook)) <= 2)
 	{
 		interface_prefs.sidebar_openfiles_visible = TRUE;
 		interface_prefs.sidebar_symbol_visible = TRUE;
@@ -2010,7 +2009,7 @@ static void set_indent_type(GtkCheckMenuItem *menuitem, GeanyIndentType type)
 	doc = document_get_current();
 	g_return_if_fail(doc != NULL);
 
-	editor_set_indent_type(doc->editor, type);
+	editor_set_indent(doc->editor, type, doc->editor->indent_width);
 	ui_update_statusbar(doc, -1);
 }
 
@@ -2169,7 +2168,7 @@ on_send_selection_to_vte1_activate     (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
 {
 #ifdef HAVE_VTE
-	if (vte_info.load_vte)
+	if (vte_info.have_vte)
 		vte_send_selection_to_vte();
 #endif
 }
@@ -2331,3 +2330,52 @@ on_plugin_preferences1_activate        (GtkMenuItem     *menuitem,
 #endif
 }
 
+
+static void set_indent_width(guint width)
+{
+	GeanyDocument *doc;
+
+	if (ignore_callback)
+		return;
+
+	doc = document_get_current();
+	g_return_if_fail(doc != NULL);
+
+	editor_set_indent(doc->editor, doc->editor->indent_type, width);
+}
+
+
+void
+on_indent_width_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	gchar *label = ui_menu_item_get_text(menuitem);
+	gint width = atoi(label);
+
+	g_free(label);
+	if (width)
+		set_indent_width(width);
+}
+
+
+void
+on_reset_indentation1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	guint i;
+
+	foreach_document(i)
+		document_apply_indent_settings(documents[i]);
+
+	ui_update_statusbar(NULL, -1);
+	ui_document_show_hide(NULL);
+}
+
+
+void
+on_mark_all1_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+	keybindings_send_command(GEANY_KEY_GROUP_SEARCH, GEANY_KEYS_SEARCH_MARKALL);
+}
+
