@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: callbacks.h 4818 2010-04-11 21:56:23Z eht16 $
+ * $Id: callbacks.h 5427 2010-11-23 12:50:24Z ntrel $
  */
 
 
@@ -174,10 +174,6 @@ void
 on_show_messages_window1_toggled       (GtkCheckMenuItem *checkmenuitem,
                                         gpointer         user_data);
 
-void
-on_goto_tag_activate                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data);
-
 void
 on_construct_completion1_activate      (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
@@ -640,3 +636,24 @@ on_smart_line_indent1_activate         (GtkMenuItem     *menuitem,
 void
 on_plugin_preferences1_activate        (GtkMenuItem     *menuitem,
                                         gpointer         user_data);
+
+void
+on_goto_tag_definition1                (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_goto_tag_declaration1               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_indent_width_activate               (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+void
+on_reset_indentation1_activate         (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
+
+
+void
+on_mark_all1_activate                  (GtkMenuItem     *menuitem,
+                                        gpointer         user_data);
