@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.c 4929 2010-05-16 18:22:24Z eht16 $
+ * $Id: dialogs.c 5419 2010-11-18 18:53:12Z ntrel $
  */
 
 /*
@@ -85,8 +85,9 @@ on_file_open_dialog_response           (GtkDialog *dialog,
 		GSList *filelist;
 		gint filetype_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
 						ui_lookup_widget(GTK_WIDGET(dialog), "filetype_combo")));
-		gint encoding_idx = gtk_combo_box_get_active(GTK_COMBO_BOX(
-						ui_lookup_widget(GTK_WIDGET(dialog), "encoding_combo")));
+		gint encoding_idx;
+		GtkTreeModel *encoding_model;
+		GtkTreeIter encoding_iter;
 		GeanyFiletype *ft = NULL;
 		const gchar *charset = NULL;
 		gboolean ro = (response == GEANY_RESPONSE_VIEW);	/* View clicked */
@@ -94,6 +95,12 @@ on_file_open_dialog_response           (GtkDialog *dialog,
 		/* ignore detect from file item */
 		if (filetype_idx > 0)
 			ft = g_slist_nth_data(filetypes_by_title, filetype_idx);
+
+		encoding_model = gtk_combo_box_get_model(GTK_COMBO_BOX(
+			ui_lookup_widget(GTK_WIDGET(dialog), "encoding_combo")));
+		gtk_combo_box_get_active_iter(GTK_COMBO_BOX(
+			ui_lookup_widget(GTK_WIDGET(dialog), "encoding_combo")), &encoding_iter);
+		gtk_tree_model_get(encoding_model, &encoding_iter, 0, &encoding_idx, -1);
 		if (encoding_idx >= 0 && encoding_idx < GEANY_ENCODINGS_MAX)
 			charset = encodings[encoding_idx].charset;
 
@@ -133,12 +140,100 @@ on_file_open_check_hidden_toggled(GtkToggleButton *togglebutton, gpointer user_d
 }
 
 
+static gint encoding_combo_store_sort_func(GtkTreeModel *model,
+										   GtkTreeIter *a,
+										   GtkTreeIter *b,
+										   gpointer data)
+{
+	gboolean a_has_child = gtk_tree_model_iter_has_child(model, a);
+	gboolean b_has_child = gtk_tree_model_iter_has_child(model, b);
+	gchar *a_string;
+	gchar *b_string;
+	gint cmp_res;
+
+	if (a_has_child != b_has_child)
+		return a_has_child ? -1 : 1;
+
+	gtk_tree_model_get(model, a, 1, &a_string, -1);
+	gtk_tree_model_get(model, b, 1, &b_string, -1);
+	cmp_res = strcmp(a_string, b_string);
+	g_free(a_string);
+	g_free(b_string);
+	return cmp_res;
+}
+
+
+static GtkTreeStore *create_encoding_combo_store(GtkTreeIter *iter_detect)
+{
+	GtkTreeStore *store;
+	GtkTreeIter iter_current, iter_westeuro, iter_easteuro, iter_eastasian, iter_asian,
+				iter_utf8, iter_middleeast;
+	GtkTreeIter *iter_parent;
+	gchar *encoding_string;
+	guint i;
+
+	store = gtk_tree_store_new(2, G_TYPE_INT, G_TYPE_STRING);
+
+	gtk_tree_store_append(store, iter_detect, NULL);
+	gtk_tree_store_set(store, iter_detect, 0, GEANY_ENCODINGS_MAX, 1, _("Detect from file"), -1);
+
+	gtk_tree_store_append(store, &iter_westeuro, NULL);
+	gtk_tree_store_set(store, &iter_westeuro, 0, -1, 1, _("West European"), -1);
+	gtk_tree_store_append(store, &iter_easteuro, NULL);
+	gtk_tree_store_set(store, &iter_easteuro, 0, -1, 1, _("East European"), -1);
+	gtk_tree_store_append(store, &iter_eastasian, NULL);
+	gtk_tree_store_set(store, &iter_eastasian, 0, -1, 1, _("East Asian"), -1);
+	gtk_tree_store_append(store, &iter_asian, NULL);
+	gtk_tree_store_set(store, &iter_asian, 0, -1, 1, _("SE & SW Asian"), -1);
+	gtk_tree_store_append(store, &iter_middleeast, NULL);
+	gtk_tree_store_set(store, &iter_middleeast, 0, -1, 1, _("Middle Eastern"), -1);
+	gtk_tree_store_append(store, &iter_utf8, NULL);
+	gtk_tree_store_set(store, &iter_utf8, 0, -1, 1, _("Unicode"), -1);
+
+	for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
+	{
+		switch (encodings[i].group)
+		{
+			case WESTEUROPEAN: iter_parent = &iter_westeuro; break;
+			case EASTEUROPEAN: iter_parent = &iter_easteuro; break;
+			case EASTASIAN: iter_parent = &iter_eastasian; break;
+			case ASIAN: iter_parent = &iter_asian; break;
+			case MIDDLEEASTERN: iter_parent = &iter_middleeast; break;
+			case UNICODE: iter_parent = &iter_utf8; break;
+			case NONE:
+			default: iter_parent = NULL;
+		}
+		gtk_tree_store_append(store, &iter_current, iter_parent);
+		encoding_string = encodings_to_string(&encodings[i]);
+		gtk_tree_store_set(store, &iter_current, 0, i, 1, encoding_string, -1);
+		g_free(encoding_string);
+	}
+
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(store), 1, GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), 1, encoding_combo_store_sort_func, NULL, NULL);
+
+	return store;
+}
+
+
+static void encoding_combo_cell_data_func(GtkCellLayout *cell_layout,
+										  GtkCellRenderer *cell,
+										  GtkTreeModel *tree_model,
+										  GtkTreeIter *iter,
+										  gpointer data)
+{
+	gboolean sensitive = !gtk_tree_model_iter_has_child(tree_model, iter);
+
+	g_object_set(cell, "sensitive", sensitive, NULL);
+}
+
+
 static void create_open_file_dialog(void)
 {
 	GtkWidget *filetype_combo, *encoding_combo;
 	GtkWidget *viewbtn;
-	guint i;
-	gchar *encoding_string;
+	GtkCellRenderer *encoding_renderer;
+	GtkTreeIter encoding_iter;
 	GSList *node;
 
 	ui_widgets.open_filesel = gtk_file_chooser_dialog_new(_("Open File"), GTK_WINDOW(main_widgets.window),
@@ -191,14 +286,14 @@ static void create_open_file_dialog(void)
 
 	/* fill encoding combo box */
 	encoding_combo = ui_lookup_widget(ui_widgets.open_filesel, "encoding_combo");
-	for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
-	{
-		encoding_string = encodings_to_string(&encodings[i]);
-		gtk_combo_box_append_text(GTK_COMBO_BOX(encoding_combo), encoding_string);
-		g_free(encoding_string);
-	}
-	gtk_combo_box_append_text(GTK_COMBO_BOX(encoding_combo), _("Detect from file"));
-	gtk_combo_box_set_active(GTK_COMBO_BOX(encoding_combo), GEANY_ENCODINGS_MAX);
+	gtk_combo_box_set_model(GTK_COMBO_BOX(encoding_combo), GTK_TREE_MODEL(
+		create_encoding_combo_store(&encoding_iter)));
+	gtk_combo_box_set_active_iter(GTK_COMBO_BOX(encoding_combo), &encoding_iter);
+	encoding_renderer = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(encoding_combo), encoding_renderer, TRUE);
+	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(encoding_combo), encoding_renderer, "text", 1);
+	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(encoding_combo), encoding_renderer,
+		encoding_combo_cell_data_func, NULL, NULL);
 
 	g_signal_connect(ui_widgets.open_filesel, "notify::show-hidden",
 				G_CALLBACK(on_file_open_notify), NULL);
@@ -284,8 +379,7 @@ static GtkWidget *add_file_open_extra_widget()
 					(GtkAttachOptions) (0), 4, 5);
 	/* the ebox is for the tooltip, because gtk_combo_box can't show tooltips */
 	encoding_ebox = gtk_event_box_new();
-	encoding_combo = gtk_combo_box_new_text();
-	gtk_combo_box_set_wrap_width(GTK_COMBO_BOX(encoding_combo), 3);
+	encoding_combo = gtk_combo_box_new();
 	ui_widget_set_tooltip_text(encoding_ebox,
 		_("Explicitly defines an encoding for the file, if it would not be detected. This is useful when you know that the encoding of a file cannot be detected correctly by Geany.\nNote if you choose multiple files, they will all be opened with the chosen encoding."));
 	gtk_container_add(GTK_CONTAINER(encoding_ebox), encoding_combo);
@@ -316,12 +410,9 @@ static GtkWidget *add_file_open_extra_widget()
 	g_signal_connect(check_hidden, "toggled",
 				G_CALLBACK(on_file_open_check_hidden_toggled), NULL);
 
-	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "check_hidden",
-				g_object_ref(check_hidden), (GDestroyNotify)g_object_unref);
-	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "filetype_combo",
-				g_object_ref(filetype_combo), (GDestroyNotify)g_object_unref);
-	g_object_set_data_full(G_OBJECT(ui_widgets.open_filesel), "encoding_combo",
-				g_object_ref(encoding_combo), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(ui_widgets.open_filesel, check_hidden, "check_hidden");
+	ui_hookup_widget(ui_widgets.open_filesel, filetype_combo, "filetype_combo");
+	ui_hookup_widget(ui_widgets.open_filesel, encoding_combo, "encoding_combo");
 
 	return expander;
 }
@@ -460,8 +551,7 @@ static void create_save_file_dialog(void)
 	g_signal_connect(check_open_new_tab, "toggled",
 				G_CALLBACK(on_save_as_new_tab_toggled), rename_btn);
 
-	g_object_set_data_full(G_OBJECT(ui_widgets.save_filesel), "check_open_new_tab",
-				g_object_ref(check_open_new_tab), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(ui_widgets.save_filesel, check_open_new_tab, "check_open_new_tab");
 
 	g_signal_connect(ui_widgets.save_filesel, "delete-event",
 		G_CALLBACK(gtk_widget_hide_on_delete), NULL);
@@ -831,7 +921,7 @@ on_input_dialog_response(GtkDialog *dialog,
 		if (persistent)
 		{
 			GtkWidget *combo = (GtkWidget *) g_object_get_data(G_OBJECT(dialog), "combo");
-			ui_combo_box_add_to_history(GTK_COMBO_BOX(combo), str);
+			ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(combo), str, 0);
 		}
 		input_cb(str);
 	}
@@ -890,12 +980,13 @@ static void add_input_widgets(GtkWidget *dialog, GtkWidget *vbox,
  * 	and can be reshown.
  * Returns: the dialog widget. */
 static GtkWidget *
-dialogs_show_input_full(const gchar *title, const gchar *label_text, const gchar *default_text,
+dialogs_show_input_full(const gchar *title, GtkWindow *parent,
+						const gchar *label_text, const gchar *default_text,
 						gboolean persistent, GeanyInputCallback input_cb, GCallback insert_text_cb)
 {
 	GtkWidget *dialog, *vbox;
 
-	dialog = gtk_dialog_new_with_buttons(title, GTK_WINDOW(main_widgets.window),
+	dialog = gtk_dialog_new_with_buttons(title, parent,
 		GTK_DIALOG_DESTROY_WITH_PARENT, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
 	vbox = ui_dialog_vbox_new(GTK_DIALOG(dialog));
@@ -924,10 +1015,11 @@ dialogs_show_input_full(const gchar *title, const gchar *label_text, const gchar
 /* Remember previous entry text in a combo box.
  * Returns: the dialog widget. */
 GtkWidget *
-dialogs_show_input_persistent(const gchar *title, const gchar *label_text, const gchar *default_text,
+dialogs_show_input_persistent(const gchar *title, GtkWindow *parent,
+		const gchar *label_text, const gchar *default_text,
 		GeanyInputCallback input_cb)
 {
-	return dialogs_show_input_full(title, label_text, default_text, TRUE, input_cb, NULL);
+	return dialogs_show_input_full(title, parent, label_text, default_text, TRUE, input_cb, NULL);
 }
 
 
@@ -940,24 +1032,32 @@ static void on_dialog_input(const gchar *str)
 }
 
 
-/* Returns: newly allocated string - a copy of either the entry text or default_text. */
-gchar *dialogs_show_input(const gchar *title, const gchar *label_text,
+/** Asks the user for text input.
+ * @param title Dialog title.
+ * @param parent The currently focused window, usually @c geany->main_widgets->window.
+ * 	@c NULL can be used but is discouraged due to window manager effects.
+ * @param label_text Label text, or @c NULL.
+ * @param default_text Text to display in the input field, or @c NULL.
+ * @return New copy of user input or @c NULL if cancelled.
+ * @since 0.20. */
+gchar *dialogs_show_input(const gchar *title, GtkWindow *parent, const gchar *label_text,
 	const gchar *default_text)
 {
 	dialog_input = NULL;
-	dialogs_show_input_full(title, label_text, default_text, FALSE, on_dialog_input, NULL);
-	return NVL(dialog_input, g_strdup(default_text));
+	dialogs_show_input_full(title, parent, label_text, default_text, FALSE, on_dialog_input, NULL);
+	return dialog_input;
 }
 
 
+/* Note: could be changed to dialogs_show_validated_input with argument for callback. */
 /* Returns: newly allocated copy of the entry text or NULL on cancel.
  * Specialised variant for Goto Line dialog. */
-gchar *dialogs_show_input_goto_line(const gchar *title, const gchar *label_text,
+gchar *dialogs_show_input_goto_line(const gchar *title, GtkWindow *parent, const gchar *label_text,
 	const gchar *default_text)
 {
 	dialog_input = NULL;
 	dialogs_show_input_full(
-		title, label_text, default_text, FALSE, on_dialog_input,
+		title, parent, label_text, default_text, FALSE, on_dialog_input,
 		G_CALLBACK(ui_editable_insert_text_callback));
 	return dialog_input;
 }
