@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: dialogs.h 4893 2010-05-09 15:48:42Z eht16 $
+ * $Id: dialogs.h 5414 2010-11-18 16:15:10Z ntrel $
  */
 
 /**
@@ -45,17 +45,17 @@ void dialogs_show_word_count(void);
 
 void dialogs_show_color(gchar *colour);
 
-gchar *dialogs_show_input(const gchar *title, const gchar *label_text,
-	const gchar *default_text);
+gchar *dialogs_show_input(const gchar *title, GtkWindow *parent,
+	const gchar *label_text, const gchar *default_text);
 
-gchar *dialogs_show_input_goto_line(const gchar *title, const gchar *label_text,
-	const gchar *default_text);
+gchar *dialogs_show_input_goto_line(const gchar *title, GtkWindow *parent,
+	const gchar *label_text, const gchar *default_text);
 
-GtkWidget *dialogs_show_input_persistent(const gchar *title, const gchar *label_text,
-	const gchar *default_text, GeanyInputCallback input_cb);
+GtkWidget *dialogs_show_input_persistent(const gchar *title, GtkWindow *parent,
+	const gchar *label_text, const gchar *default_text, GeanyInputCallback input_cb);
 
 gboolean dialogs_show_input_numeric(const gchar *title, const gchar *label_text,
-									gdouble *value, gdouble min, gdouble max, gdouble step);
+	gdouble *value, gdouble min, gdouble max, gdouble step);
 
 void dialogs_show_file_properties(GeanyDocument *doc);
 
