@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: document.c 4940 2010-05-22 19:38:49Z eht16 $
+ * $Id: document.c 5466 2010-12-06 18:04:42Z ntrel $
  */
 
 /*
@@ -48,10 +48,10 @@
 
 /* uncomment to use GIO based file monitoring, though it is not completely stable yet */
 /*#define USE_GIO_FILEMON 1*/
-#if USE_GIO_FILEMON
-# ifdef HAVE_GIO
-#  include <gio/gio.h>
-# else
+#ifdef HAVE_GIO
+# include <gio/gio.h>
+#else
+# if USE_GIO_FILEMON
 #  undef USE_GIO_FILEMON
 # endif
 #endif
@@ -78,6 +78,7 @@
 #include "navqueue.h"
 #include "win32.h"
 #include "search.h"
+#include "filetypesprivate.h"
 
 
 GeanyFilePrefs file_prefs;
@@ -540,6 +541,8 @@ static gboolean on_idle_focus(gpointer doc)
 }
 
 
+static gboolean remove_page(guint page_num);
+
 /* Creates a new document and editor, adding a tab in the notebook.
  * @return The created document */
 static GeanyDocument *document_create(const gchar *utf8_filename)
@@ -551,9 +554,11 @@ static GeanyDocument *document_create(const gchar *utf8_filename)
 	if (cur_pages == 1)
 	{
 		GeanyDocument *cur = document_get_current();
-		/* remove the empty document and open a new one */
+		/* remove the empty document first */
 		if (cur != NULL && cur->file_name == NULL && ! cur->changed)
-			document_remove_page(0);
+			/* prevent immediately opening another new doc with
+			 * new_document_after_close pref */
+			remove_page(0);
 	}
 
 	new_idx = document_get_new_idx();
@@ -608,15 +613,8 @@ gboolean document_close(GeanyDocument *doc)
 }
 
 
-/**
- *  Removes the given notebook tab at @a page_num and clears all related information
- *  in the document list.
- *
- *  @param page_num The notebook page number to remove.
- *
- *  @return @c TRUE if the document was actually removed or @c FALSE otherwise.
- **/
-gboolean document_remove_page(guint page_num)
+/* Call document_remove_page() instead, this is only needed for document_create(). */
+static gboolean remove_page(guint page_num)
 {
 	GeanyDocument *doc = document_get_from_page(page_num);
 
@@ -681,6 +679,25 @@ gboolean document_remove_page(guint page_num)
 }
 
 
+/**
+ *  Removes the given notebook tab at @a page_num and clears all related information
+ *  in the document list.
+ *
+ *  @param page_num The notebook page number to remove.
+ *
+ *  @return @c TRUE if the document was actually removed or @c FALSE otherwise.
+ **/
+gboolean document_remove_page(guint page_num)
+{
+	gboolean done = remove_page(page_num);
+
+	if (done && ui_prefs.new_document_after_close)
+		document_new_file_if_non_open();
+
+	return done;
+}
+
+
 /* used to keep a record of the unchanged document state encoding */
 static void store_saved_encoding(GeanyDocument *doc)
 {
@@ -702,6 +719,7 @@ GeanyDocument *document_new_file_if_non_open(void)
 
 /**
  *  Creates a new document.
+ *  Line endings in @a text will be converted to the default setting.
  *  Afterwards, the @c "document-new" signal is emitted for plugins.
  *
  *  @param utf8_filename The file name in UTF-8 encoding, or @c NULL to open a file as "untitled".
@@ -710,8 +728,7 @@ GeanyDocument *document_new_file_if_non_open(void)
  *
  *  @return The new document.
  **/
-GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
-		const gchar *text)
+GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft, const gchar *text)
 {
 	GeanyDocument *doc;
 
@@ -728,15 +745,17 @@ GeanyDocument *document_new_file(const gchar *utf8_filename, GeanyFiletype *ft,
 
 	sci_set_undo_collection(doc->editor->sci, FALSE); /* avoid creation of an undo action */
 	if (text)
-		sci_set_text(doc->editor->sci, text);
+	{
+		GString *template = g_string_new(text);
+		utils_ensure_same_eol_characters(template, file_prefs.default_eol_character);
+
+		sci_set_text(doc->editor->sci, template->str);
+		g_string_free(template, TRUE);
+	}
 	else
 		sci_clear_all(doc->editor->sci);
 
 	sci_set_eol_mode(doc->editor->sci, file_prefs.default_eol_character);
-	/* convert the eol chars in the template text in case they are different from
-	 * from file_prefs.default_eol */
-	if (text != NULL)
-		sci_convert_eols(doc->editor->sci, file_prefs.default_eol_character);
 
 	sci_set_undo_collection(doc->editor->sci, TRUE);
 	sci_empty_undo_buffer(doc->editor->sci);
@@ -1138,16 +1157,18 @@ void document_apply_indent_settings(GeanyDocument *doc)
 	const GeanyIndentPrefs *iprefs = editor_get_indent_prefs(NULL);
 	GeanyIndentType type = iprefs->type;
 
-	switch (FILETYPE_ID(doc->file_type))
+	switch (doc->file_type->id)
 	{
 		case GEANY_FILETYPES_MAKE:
 			/* force using tabs for indentation for Makefiles */
-			editor_set_indent_type(doc->editor, GEANY_INDENT_TYPE_TABS);
+			editor_set_indent(doc->editor, GEANY_INDENT_TYPE_TABS, iprefs->width);
 			return;
 		case GEANY_FILETYPES_F77:
 			/* force using spaces for indentation for Fortran 77 */
-			editor_set_indent_type(doc->editor, GEANY_INDENT_TYPE_SPACES);
+			editor_set_indent(doc->editor, GEANY_INDENT_TYPE_SPACES, iprefs->width);
 			return;
+		default:
+			break;
 	}
 	if (iprefs->detect_type)
 	{
@@ -1175,7 +1196,7 @@ void document_apply_indent_settings(GeanyDocument *doc)
 				DOC_FILENAME(doc));
 		}
 	}
-	editor_set_indent_type(doc->editor, type);
+	editor_set_indent(doc->editor, type, iprefs->width);
 }
 
 
@@ -1324,7 +1345,7 @@ GeanyDocument *document_open_file_full(GeanyDocument *doc, const gchar *filename
 
 		/* set indentation settings after setting the filetype */
 		if (reload)
-			editor_set_indent_type(doc->editor, doc->editor->indent_type); /* resetup sci */
+			editor_set_indent(doc->editor, doc->editor->indent_type, doc->editor->indent_width); /* resetup sci */
 		else
 			document_apply_indent_settings(doc);
 
@@ -1562,13 +1583,15 @@ void document_rename_file(GeanyDocument *doc, const gchar *new_filename)
 }
 
 
-/* Return TRUE if the document hasn't been saved before, i.e. either the filename or
- * the real_path is not set. */
+/* Return TRUE if the document doesn't have a full filename set.
+ * This makes filenames without a path show the save as dialog, e.g. for file templates.
+ * Otherwise just use the set filename instead of asking the user - e.g. for command-line
+ * new files. */
 gboolean document_need_save_as(GeanyDocument *doc)
 {
 	g_return_val_if_fail(doc != NULL, FALSE);
 
-	return (doc->file_name == NULL || doc->real_path == NULL);
+	return (doc->file_name == NULL || !g_path_is_absolute(doc->file_name));
 }
 
 
@@ -1596,7 +1619,7 @@ gboolean document_save_file_as(GeanyDocument *doc, const gchar *utf8_fname)
 	setptr(doc->real_path, NULL);
 
 	/* detect filetype */
-	if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_NONE)
+	if (doc->file_type->id == GEANY_FILETYPES_NONE)
 	{
 		GeanyFiletype *ft = filetypes_detect_from_document(doc);
 
@@ -1682,48 +1705,111 @@ _("An error occurred while converting the file from UTF-8 in \"%s\". The file re
 		*data = conv_file_contents;
 		*len = conv_len;
 	}
-
 	return TRUE;
 }
 
 
-static gchar *write_data_to_disk(GeanyDocument *doc, const gchar *locale_filename,
+static gchar *write_data_to_disk(const gchar *locale_filename,
 								 const gchar *data, gint len)
 {
-	FILE *fp;
-	gint bytes_written;
-	gint err = 0;
 	GError *error = NULL;
 
+	if (! file_prefs.use_safe_file_saving)
+	{
+#ifdef HAVE_GIO
+		GFile *fp;
+
+		/* Use GIO API to save file (GVFS-safe) */
+		fp = g_file_new_for_path(locale_filename);
+		g_file_replace_contents(fp, data, len, NULL, file_prefs.gio_unsafe_save_backup,
+			G_FILE_CREATE_NONE, NULL, NULL, &error);
+		g_object_unref(fp);
+#else
+		FILE *fp;
+		int save_errno;
+		gchar *display_name = g_filename_display_name(locale_filename);
+
+		/* Use POSIX API for unsafe saving (GVFS-unsafe) */
+		/* The error handling is taken from glib-2.26.0 gfileutils.c */
+		errno = 0;
+		fp = g_fopen(locale_filename, "wb");
+		if (fp == NULL)
+		{
+			save_errno = errno;
+
+			g_set_error(&error,
+				G_FILE_ERROR,
+				g_file_error_from_errno(save_errno),
+				_("Failed to open file '%s' for writing: fopen() failed: %s"),
+				display_name,
+				g_strerror(save_errno));
+		}
+		else
+		{
+			gint bytes_written;
+
+			errno = 0;
+			bytes_written = fwrite(data, sizeof(gchar), len, fp);
+
+			if (len != bytes_written)
+			{
+				save_errno = errno;
+
+				g_set_error(&error,
+					G_FILE_ERROR,
+					g_file_error_from_errno(save_errno),
+					_("Failed to write file '%s': fwrite() failed: %s"),
+					display_name,
+					g_strerror(save_errno));
+			}
+
+			errno = 0;
+			/* preserve the fwrite() error if any */
+			if (fclose(fp) != 0 && error == NULL)
+			{
+				save_errno = errno;
+
+				g_set_error(&error,
+					G_FILE_ERROR,
+					g_file_error_from_errno(save_errno),
+					_("Failed to close file '%s': fclose() failed: %s"),
+					display_name,
+					g_strerror(save_errno));
+			}
+		}
+
+		g_free(display_name);
+#endif
+	}
+	else
+	{
+		/* Use old GLib API for safe saving (GVFS-safe, but alters ownership and permissons).
+		 * This is the only option that handles disk space exhaustion. */
+		if (g_file_set_contents(locale_filename, data, len, &error))
+			geany_debug("Wrote %s with g_file_set_contents().", locale_filename);
+	}
+	if (error != NULL)
+	{
+		gchar *msg = g_strdup(error->message);
+		g_error_free(error);
+		/* geany will warn about file truncation for unsafe saving below */
+		return msg;
+	}
+	return NULL;
+}
+
+
+static gchar *save_doc(GeanyDocument *doc, const gchar *locale_filename,
+								 const gchar *data, gint len)
+{
+	gchar *err;
+
 	g_return_val_if_fail(doc != NULL, g_strdup(g_strerror(EINVAL)));
 	g_return_val_if_fail(data != NULL, g_strdup(g_strerror(EINVAL)));
 
-	if (! file_prefs.use_safe_file_saving)
-	{
-		fp = g_fopen(locale_filename, "wb");
-		if (G_UNLIKELY(fp == NULL))
-			return g_strdup(g_strerror(errno));
-
-		bytes_written = fwrite(data, sizeof(gchar), len, fp);
-
-		if (G_UNLIKELY(len != bytes_written))
-			err = errno;
-
-		fclose(fp);
-
-		if (err != 0)
-			return g_strdup(g_strerror(err));
-	}
-	else
-	{
-		g_file_set_contents(locale_filename, data, len, &error);
-		if (error != NULL)
-		{
-			gchar *msg = g_strdup(error->message);
-			g_error_free(error);
-			return msg;
-		}
-	}
+	err = write_data_to_disk(locale_filename, data, len);
+	if (err)
+		return err;
 
 	/* now the file is on disk, set real_path */
 	if (doc->real_path == NULL)
@@ -1732,21 +1818,23 @@ static gchar *write_data_to_disk(GeanyDocument *doc, const gchar *locale_filenam
 		doc->priv->is_remote = utils_is_remote_path(locale_filename);
 		monitor_file_setup(doc);
 	}
-
 	return NULL;
 }
 
 
 /**
- *  Saves the document. Saving includes replacing tabs by spaces,
- *  stripping trailing spaces and adding a final new line at the end of the file (all only if
- *  user enabled these features). Then the @c "document-before-save" signal is emitted,
+ *  Saves the document. Saving may include replacing tabs by spaces,
+ *  stripping trailing spaces and adding a final new line at the end of the file, depending
+ *  on user preferences. Then the @c "document-before-save" signal is emitted,
  *  allowing plugins to modify the document before it is saved, and data is
  *  actually written to disk. The filetype is set again or auto-detected if it wasn't set yet.
  *  Afterwards, the @c "document-save" signal is emitted for plugins.
  *
  *  If the file is not modified, this functions does nothing unless force is set to @c TRUE.
  *
+ *  @note You should ensure @c doc->file_name is not @c NULL before calling this; otherwise
+ *  call dialogs_show_save_as().
+ *
  *  @param doc The document to save.
  *  @param force Whether to save the file even if it is not modified (e.g. for Save As).
  *
@@ -1767,13 +1855,13 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 
 	if (G_UNLIKELY(doc->file_name == NULL))
 	{
-		ui_set_statusbar(TRUE, _("Error saving file."));
+		ui_set_statusbar(TRUE, _("Error saving file (%s)."), _("Invalid filename"));
 		utils_beep();
 		return FALSE;
 	}
 
 	/* replaces tabs by spaces but only if the current file is not a Makefile */
-	if (file_prefs.replace_tabs && FILETYPE_ID(doc->file_type) != GEANY_FILETYPES_MAKE)
+	if (file_prefs.replace_tabs && doc->file_type->id != GEANY_FILETYPES_MAKE)
 		editor_replace_tabs(doc->editor);
 	/* strip trailing spaces */
 	if (file_prefs.strip_trailing_spaces)
@@ -1781,6 +1869,9 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 	/* ensure the file has a newline at the end */
 	if (file_prefs.final_new_line)
 		editor_ensure_final_newline(doc->editor);
+	/* ensure newlines are consistent */
+	if (file_prefs.ensure_convert_new_lines)
+		sci_convert_eols(doc->editor->sci, sci_get_eol_mode(doc->editor->sci));
 
 	/* notify plugins which may wish to modify the document before it's saved */
 	g_signal_emit_by_name(geany_object, "document-before-save", doc);
@@ -1824,12 +1915,18 @@ gboolean document_save_file(GeanyDocument *doc, gboolean force)
 	doc->priv->file_disk_status = FILE_IGNORE;
 
 	/* actually write the content of data to the file on disk */
-	errmsg = write_data_to_disk(doc, locale_filename, data, len);
+	errmsg = save_doc(doc, locale_filename, data, len);
 	g_free(data);
 
 	if (errmsg != NULL)
 	{
 		ui_set_statusbar(TRUE, _("Error saving file (%s)."), errmsg);
+
+		if (!file_prefs.use_safe_file_saving)
+		{
+			setptr(errmsg,
+				g_strdup_printf(_("%s\n\nThe file on disk may now be truncated!"), errmsg));
+		}
 		dialogs_show_msgbox_with_secondary(GTK_MESSAGE_ERROR, _("Error saving file."), errmsg);
 		doc->priv->file_disk_status = FILE_OK;
 		utils_beep();
@@ -2381,7 +2478,7 @@ static gboolean update_type_keywords(GeanyDocument *doc, gint lang)
 	g_return_val_if_fail(doc != NULL, FALSE);
 	sci = doc->editor->sci;
 
-	switch (FILETYPE_ID(doc->file_type))
+	switch (doc->file_type->id)
 	{	/* continue working with the following languages, skip on all others */
 		case GEANY_FILETYPES_C:
 		case GEANY_FILETYPES_CPP:
@@ -2455,6 +2552,7 @@ static void document_load_config(GeanyDocument *doc, GeanyFiletype *type,
 		editor_set_indentation_guides(doc->editor);
 		build_menu_update(doc);
 		queue_colourise(doc);
+		doc->priv->symbol_list_sort_mode = type->priv->symbol_list_sort_mode;
 	}
 
 	document_update_tag_list(doc, TRUE);
@@ -2479,14 +2577,17 @@ void document_set_filetype(GeanyDocument *doc, GeanyFiletype *type)
 	old_ft = doc->file_type;
 	geany_debug("%s : %s (%s)",
 		(doc->file_name != NULL) ? doc->file_name : "unknown",
-		(type->name != NULL) ? type->name : "unknown",
+		type->name,
 		(doc->encoding != NULL) ? doc->encoding : "unknown");
 
 	ft_changed = (doc->file_type != type); /* filetype has changed */
 	document_load_config(doc, type, ft_changed);
 
 	if (ft_changed)
+	{
+		sidebar_openfiles_update(doc); /* to update the icon */
 		g_signal_emit_by_name(geany_object, "document-filetype-set", doc, old_ft);
+	}
 }
 
 
@@ -2887,21 +2988,24 @@ gboolean document_close_all(void)
 }
 
 
-static gboolean monitor_reload_file(GeanyDocument *doc)
+static void monitor_reload_file(GeanyDocument *doc)
 {
 	gchar *base_name = g_path_get_basename(doc->file_name);
-	gboolean want_reload;
+	gint ret;
 
-	want_reload = dialogs_show_question_full(NULL, _("_Reload"), GTK_STOCK_CANCEL,
+	ret = dialogs_show_prompt(NULL,
+		GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		_("_Reload"), GTK_RESPONSE_ACCEPT,
 		_("Do you want to reload it?"),
 		_("The file '%s' on the disk is more recent than\nthe current buffer."),
 		base_name);
-
-	if (want_reload)
-		document_reload_file(doc, doc->encoding);
-
 	g_free(base_name);
-	return want_reload;
+
+	if (ret == GTK_RESPONSE_ACCEPT)
+		document_reload_file(doc, doc->encoding);
+	else if (ret == GTK_RESPONSE_CLOSE)
+		document_close(doc);
 }
 
 
@@ -2915,8 +3019,8 @@ static gboolean monitor_resave_missing_file(GeanyDocument *doc)
 		_("Close _without saving"), GTK_RESPONSE_CLOSE,
 		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-		NULL,
-		_("File \"%s\" was not found on disk! Try to resave the file?"),
+		_("Try to resave the file?"),
+		_("File \"%s\" was not found on disk!"),
 		doc->file_name);
 	if (ret == GTK_RESPONSE_ACCEPT)
 	{
@@ -2977,6 +3081,7 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 	if (g_stat(locale_filename, &st) != 0)
 	{
 		monitor_resave_missing_file(doc);
+		/* doc may be closed now */
 		ret = TRUE;
 	}
 	else if (! use_gio_filemon && /* ignore these checks when using GIO */
@@ -2986,14 +3091,15 @@ gboolean document_check_disk_status(GeanyDocument *doc, gboolean force)
 	}
 	else if (doc->priv->mtime < st.st_mtime)
 	{
-		monitor_reload_file(doc);
 		doc->priv->mtime = st.st_mtime;
+		monitor_reload_file(doc);
+		/* doc may be closed now */
 		ret = TRUE;
 	}
 	g_free(locale_filename);
 
 	if (DOC_VALID(doc))
-	{	/* doc can get invalid when a document was closed by monitor_resave_missing_file() */
+	{	/* doc can get invalid when a document was closed */
 		old_status = doc->priv->file_disk_status;
 		doc->priv->file_disk_status = FILE_OK;
 		if (old_status != doc->priv->file_disk_status)
