@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: filetypes.c 5161 2010-08-13 17:00:36Z ntrel $
+ * $Id: filetypes.c 5393 2010-11-10 18:04:15Z ntrel $
  */
 
 /**
@@ -34,6 +34,7 @@
 
 #include "geany.h"
 #include "filetypes.h"
+#include "filetypesprivate.h"
 #include "highlighting.h"
 #include "support.h"
 #include "templates.h"
@@ -43,32 +44,10 @@
 #include "utils.h"
 #include "sciwrappers.h"
 #include "ui_utils.h"
+#include "symbols.h"
 
 #include <stdlib.h>
 
-#ifdef HAVE_REGCOMP
-# ifdef HAVE_REGEX_H
-#  include <regex.h>
-# else
-#  include "gnuregex.h"
-# endif
-#endif
-
-
-/* Private GeanyFiletype fields */
-typedef struct GeanyFiletypePrivate
-{
-	GtkWidget	*menu_item;			/* holds a pointer to the menu item for this filetype */
-	gboolean	keyfile_loaded;
-#ifdef HAVE_REGCOMP
-	regex_t		error_regex;
-	gboolean	error_regex_compiled;
-	gchar		*last_string; /* last one compiled */
-#endif
-	gboolean	custom;
-}
-GeanyFiletypePrivate;
-
 
 GPtrArray *filetypes_array = NULL;	/* Dynamic array of filetype pointers */
 
@@ -114,13 +93,10 @@ static void init_builtin_filetypes(void)
 
 #define NONE	/* these macros are only to ease navigation */
 	ft = filetypes[GEANY_FILETYPES_NONE];
-	ft->lang = -2;
-	ft->name = g_strdup(_("None"));
+	/* ft->name must not be translated as it is used for filetype lookup.
+	 * Use filetypes_get_display_name() instead. */
+	ft->name = g_strdup("None");
 	ft->title = g_strdup(_("None"));
-	ft->extension = NULL;
-	ft->pattern = utils_strv_new("*", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_NONE;
 
 #define C
@@ -128,33 +104,23 @@ static void init_builtin_filetypes(void)
 	ft->lang = 0;
 	ft->name = g_strdup("C");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("c");
-	ft->pattern = utils_strv_new("*.c", "*.h", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
+	ft->mime_type = g_strdup("text/x-csrc");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define CPP
 	ft = filetypes[GEANY_FILETYPES_CPP];
 	ft->lang = 1;
 	ft->name = g_strdup("C++");
-	filetype_make_title(ft, TITLE_FILE);
-	ft->extension = g_strdup("cpp");
-	ft->pattern = utils_strv_new("*.cpp", "*.cxx", "*.c++", "*.cc",
-		"*.h", "*.hpp", "*.hxx", "*.h++", "*.hh", "*.C", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-c++src");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define CS
 	ft = filetypes[GEANY_FILETYPES_CS];
 	ft->lang = 25;
 	ft->name = g_strdup("C#");
-	filetype_make_title(ft, TITLE_FILE);
-	ft->extension = g_strdup("cs");
-	ft->pattern = utils_strv_new("*.cs", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-csharp");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define VALA
@@ -162,10 +128,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 33;
 	ft->name = g_strdup("Vala");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("vala");
-	ft->pattern = utils_strv_new("*.vala", "*.vapi", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-vala");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define D
@@ -173,10 +136,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 17;
 	ft->name = g_strdup("D");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("d");
-	ft->pattern = utils_strv_new("*.d", "*.di", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-dsrc");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define JAVA
@@ -184,10 +144,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 2;
 	ft->name = g_strdup("Java");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("java");
-	ft->pattern = utils_strv_new("*.java", "*.jsp", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
+	ft->mime_type = g_strdup("text/x-java");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define PAS /* to avoid warnings when building under Windows, the symbol PASCAL is there defined */
@@ -195,11 +152,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 4;
 	ft->name = g_strdup("Pascal");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("pas");
-	ft->pattern = utils_strv_new("*.pas", "*.pp", "*.inc", "*.dpr",
-		"*.dpk", NULL);
-	ft->comment_open = g_strdup("{");
-	ft->comment_close = g_strdup("}");
+	ft->mime_type = g_strdup("text/x-pascal");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define ASM
@@ -207,10 +160,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 9;
 	ft->name = g_strdup("ASM");
 	ft->title = g_strdup_printf(_("%s source file"), "Assembler");
-	ft->extension = g_strdup("asm");
-	ft->pattern = utils_strv_new("*.asm", NULL);
-	ft->comment_open = g_strdup(";");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define BASIC
@@ -218,10 +167,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 26;
 	ft->name = g_strdup("FreeBasic");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("bas");
-	ft->pattern = utils_strv_new("*.bas", "*.bi", NULL);
-	ft->comment_open = g_strdup("'");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define FORTRAN
@@ -229,10 +174,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 18;
 	ft->name = g_strdup("Fortran");
 	ft->title = g_strdup_printf(_("%s source file"), "Fortran (F90)");
-	ft->extension = g_strdup("f90");
-	ft->pattern = utils_strv_new("*.f90", "*.f95", "*.f03", NULL);
-	ft->comment_open = g_strdup("!");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-fortran");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define F77
@@ -240,10 +182,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 30;
 	ft->name = g_strdup("F77");
 	ft->title = g_strdup_printf(_("%s source file"), "Fortran (F77)");
-	ft->extension = g_strdup("f");
-	ft->pattern = utils_strv_new("*.f", "*.for", "*.ftn", "*.f77", NULL);
-	ft->comment_open = g_strdup("c");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-fortran");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define GLSL
@@ -251,21 +190,13 @@ static void init_builtin_filetypes(void)
 	ft->lang = 31;
 	ft->name = g_strdup("GLSL");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("glsl");
-	ft->pattern = utils_strv_new("*.glsl", "*.frag", "*.vert", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define CAML
 	ft = filetypes[GEANY_FILETYPES_CAML];
-	ft->lang = -2;
 	ft->name = g_strdup("CAML");
 	ft->title = g_strdup_printf(_("%s source file"), "(O)Caml");
-	ft->extension = g_strdup("ml");
-	ft->pattern = utils_strv_new("*.ml", "*.mli", NULL);
-	ft->comment_open = g_strdup("(*");
-	ft->comment_close = g_strdup("*)");
+	ft->mime_type = g_strdup("text/x-ocaml");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define PERL
@@ -273,11 +204,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 5;
 	ft->name = g_strdup("Perl");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("pl");
-	ft->pattern = utils_strv_new("*.pl", "*.perl", "*.pm", "*.agi",
-		"*.pod", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/x-perl");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define PHP
@@ -285,11 +212,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 6;
 	ft->name = g_strdup("PHP");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("php");
-	ft->pattern = utils_strv_new("*.php", "*.php3", "*.php4", "*.php5",
-		"*.phtml", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/x-php");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define JAVASCRIPT
@@ -297,10 +220,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 23;
 	ft->name = g_strdup("Javascript");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("js");
-	ft->pattern = utils_strv_new("*.js", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/javascript");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define PYTHON
@@ -308,10 +228,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 7;
 	ft->name = g_strdup("Python");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("py");
-	ft->pattern = utils_strv_new("*.py", "*.pyw", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-python");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define RUBY
@@ -319,10 +236,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 14;
 	ft->name = g_strdup("Ruby");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("rb");
-	ft->pattern = utils_strv_new("*.rb", "*.rhtml", "*.ruby", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/x-ruby");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define TCL
@@ -330,10 +244,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 15;
 	ft->name = g_strdup("Tcl");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("tcl");
-	ft->pattern = utils_strv_new("*.tcl", "*.tk", "*.wish", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-tcl");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define LUA
@@ -341,10 +252,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 22;
 	ft->name = g_strdup("Lua");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("lua");
-	ft->pattern = utils_strv_new("*.lua", NULL);
-	ft->comment_open = g_strdup("--");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-lua");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define FERITE
@@ -352,10 +260,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 19;
 	ft->name = g_strdup("Ferite");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("fe");
-	ft->pattern = utils_strv_new("*.fe", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define HASKELL
@@ -363,10 +267,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 24;
 	ft->name = g_strdup("Haskell");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("hs");
-	ft->pattern = utils_strv_new("*.hs", "*.lhs", NULL);
-	ft->comment_open = g_strdup("--");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-haskell");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define MARKDOWN
@@ -374,10 +275,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 36;
 	ft->name = g_strdup("Markdown");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("md");
-	ft->pattern = utils_strv_new("*.mdml", "*.mdwn", "*.markdown", "*.md", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define TXT2TAGS
@@ -385,10 +282,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 37;
 	ft->name = g_strdup("Txt2tags");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("t2t");
-	ft->pattern = utils_strv_new("*.t2t", "*.txt2tags", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-txt2tags");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define ABC
@@ -396,10 +290,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 38;
 	ft->name = g_strdup("Abc");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("abc");
-	ft->pattern = utils_strv_new("*.abc", "*.abp", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define SH
@@ -407,11 +297,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 16;
 	ft->name = g_strdup("Sh");
 	ft->title = g_strdup(_("Shell script file"));
-	ft->extension = g_strdup("sh");
-	ft->pattern = utils_strv_new("*.sh", "configure", "configure.in",
-		"configure.in.in", "configure.ac", "*.ksh", "*.zsh", "*.ash", "*.bash", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/x-shellscript");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define MAKE
@@ -419,23 +305,14 @@ static void init_builtin_filetypes(void)
 	ft->lang = 3;
 	ft->name = g_strdup("Make");
 	ft->title = g_strdup(_("Makefile"));
-	ft->extension = g_strdup("mak");
-	ft->pattern = utils_strv_new(
-		"*.mak", "*.mk", "GNUmakefile", "makefile", "Makefile", "makefile.*", "Makefile.*", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-makefile");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define XML
 	ft = filetypes[GEANY_FILETYPES_XML];
-	ft->lang = -2;
 	ft->name = g_strdup("XML");
 	ft->title = g_strdup(_("XML document"));
-	ft->extension = g_strdup("xml");
-	ft->pattern = utils_strv_new(
-		"*.xml", "*.sgml", "*.xsl", "*.xslt", "*.xsd", "*.xhtml", NULL);
-	ft->comment_open = g_strdup("<!--");
-	ft->comment_close = g_strdup("-->");
+	ft->mime_type = g_strdup("application/xml");
 	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define DOCBOOK
@@ -443,10 +320,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 12;
 	ft->name = g_strdup("Docbook");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("docbook");
-	ft->pattern = utils_strv_new("*.docbook", NULL);
-	ft->comment_open = g_strdup("<!--");
-	ft->comment_close = g_strdup("-->");
+	ft->mime_type = g_strdup("application/docbook+xml");
 	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define HTML
@@ -454,11 +328,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 29;
 	ft->name = g_strdup("HTML");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("html");
-	ft->pattern = utils_strv_new(
-		"*.htm", "*.html", "*.shtml", "*.hta", "*.htd", "*.htt", "*.cfm", NULL);
-	ft->comment_open = g_strdup("<!--");
-	ft->comment_close = g_strdup("-->");
+	ft->mime_type = g_strdup("text/html");
 	ft->group = GEANY_FILETYPE_GROUP_MARKUP;
 
 #define CSS
@@ -466,10 +336,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 13;
 	ft->name = g_strdup("CSS");
 	ft->title = g_strdup(_("Cascading StyleSheet"));
-	ft->extension = g_strdup("css");
-	ft->pattern = utils_strv_new("*.css", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
+	ft->mime_type = g_strdup("text/css");
 	ft->group = GEANY_FILETYPE_GROUP_MARKUP;	/* not really markup but fit quite well to HTML */
 
 #define SQL
@@ -477,10 +344,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 11;
 	ft->name = g_strdup("SQL");
 	ft->title = g_strdup(_("SQL Dump file"));
-	ft->extension = g_strdup("sql");
-	ft->pattern = utils_strv_new("*.sql", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
+	ft->mime_type = g_strdup("text/x-sql");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define LATEX
@@ -488,10 +352,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 8;
 	ft->name = g_strdup("LaTeX");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("tex");
-	ft->pattern = utils_strv_new("*.tex", "*.sty", "*.idx", "*.ltx", NULL);
-	ft->comment_open = g_strdup("%");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-tex");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define VHDL
@@ -499,10 +360,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 21;
 	ft->name = g_strdup("VHDL");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("vhd");
-	ft->pattern = utils_strv_new("*.vhd", "*.vhdl", NULL);
-	ft->comment_open = g_strdup("--");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-vhdl");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define VERILOG
@@ -510,10 +368,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 39;
 	ft->name = g_strdup("Verilog");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("v");
-	ft->pattern = utils_strv_new("*.v", "*.verilog", NULL);
-	ft->comment_open = g_strdup("/*");
-	ft->comment_close = g_strdup("*/");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define DIFF
@@ -521,33 +375,34 @@ static void init_builtin_filetypes(void)
 	ft->lang = 20;
 	ft->name = g_strdup("Diff");
 	filetype_make_title(ft, TITLE_FILE);
-	ft->extension = g_strdup("diff");
-	ft->pattern = utils_strv_new("*.diff", "*.patch", "*.rej", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-patch");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
+#define LISP
+	ft = filetypes[GEANY_FILETYPES_LISP];
+	ft->name = g_strdup("Lisp");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
+
+#define ERLANG
+	ft = filetypes[GEANY_FILETYPES_ERLANG];
+	ft->name = g_strdup("Erlang");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->mime_type = g_strdup("text/x-erlang");
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
+
 #define CONF
 	ft = filetypes[GEANY_FILETYPES_CONF];
 	ft->lang = 10;
 	ft->name = g_strdup("Conf");
 	ft->title = g_strdup(_("Config file"));
-	ft->extension = g_strdup("conf");
-	ft->pattern = utils_strv_new("*.conf", "*.ini", "config", "*rc",
-		"*.cfg", "*.desktop", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define PO
 	ft = filetypes[GEANY_FILETYPES_PO];
-	ft->lang = -2;
 	ft->name = g_strdup("Po");
 	ft->title = g_strdup(_("Gettext translation file"));
-	ft->extension = g_strdup("po");
-	ft->pattern = utils_strv_new("*.po", "*.pot", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-gettext-translation");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define HAXE
@@ -555,10 +410,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 27;
 	ft->name = g_strdup("Haxe");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("hx");
-	ft->pattern = utils_strv_new("*.hx", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
 
 #define ACTIONSCRIPT
@@ -566,10 +417,7 @@ static void init_builtin_filetypes(void)
 	ft->lang = 34;
 	ft->name = g_strdup("ActionScript");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("as");
-	ft->pattern = utils_strv_new("*.as", NULL);
-	ft->comment_open = g_strdup("//");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("application/ecmascript");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define R
@@ -577,10 +425,6 @@ static void init_builtin_filetypes(void)
 	ft->lang = 40;
 	ft->name = g_strdup("R");
 	ft->title = g_strdup_printf(_("%s script file"), "R");
-	ft->extension = g_strdup("R");
-	ft->pattern = utils_strv_new("*.R", "*.r", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define REST
@@ -588,44 +432,27 @@ static void init_builtin_filetypes(void)
 	ft->lang = 28;
 	ft->name = g_strdup("reStructuredText");
 	filetype_make_title(ft, TITLE_FILE);
-	ft->extension = g_strdup("rst");
-	ft->pattern = utils_strv_new(
-		"*.rest", "*.reST", "*.rst", NULL);
-	ft->comment_open = NULL;
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define MATLAB
 	ft = filetypes[GEANY_FILETYPES_MATLAB];
 	ft->lang = 32;
-	ft->name = g_strdup("Matlab");
+	ft->name = g_strdup("Matlab/Octave");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("m");
-	ft->pattern = utils_strv_new("*.m", NULL);
-	ft->comment_open = g_strdup("%");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-matlab");
 	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 
 #define YAML
 	ft = filetypes[GEANY_FILETYPES_YAML];
-	ft->lang = -2;
 	ft->name = g_strdup("YAML");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("yaml");
-	ft->pattern = utils_strv_new("*.yaml", "*.yml", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define CMAKE
 	ft = filetypes[GEANY_FILETYPES_CMAKE];
-	ft->lang = -2;
 	ft->name = g_strdup("CMake");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("cmake");
-	ft->pattern = utils_strv_new("CMakeLists.txt", "*.cmake", "*.ctest", NULL);
-	ft->comment_open = g_strdup("#");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-cmake");
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define NSIS
@@ -633,22 +460,20 @@ static void init_builtin_filetypes(void)
 	ft->lang = 35;
 	ft->name = g_strdup("NSIS");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("nsis");
-	ft->pattern = utils_strv_new("*.nsi", "*.nsh", NULL);
-	ft->comment_open = g_strdup(";");
-	ft->comment_close = NULL;
 	ft->group = GEANY_FILETYPE_GROUP_MISC;
 
 #define ADA
 	ft = filetypes[GEANY_FILETYPES_ADA];
-	ft->lang = -2;
 	ft->name = g_strdup("Ada");
 	filetype_make_title(ft, TITLE_SOURCE_FILE);
-	ft->extension = g_strdup("adb");
-	ft->pattern = utils_strv_new("*.adb", "*.ads", NULL);
-	ft->comment_open = g_strdup("--");
-	ft->comment_close = NULL;
+	ft->mime_type = g_strdup("text/x-adasrc");
 	ft->group = GEANY_FILETYPE_GROUP_COMPILED;
+
+#define FORTH
+	ft = filetypes[GEANY_FILETYPES_FORTH];
+	ft->name = g_strdup("Forth");
+	filetype_make_title(ft, TITLE_SOURCE_FILE);
+	ft->group = GEANY_FILETYPE_GROUP_SCRIPT;
 }
 
 
@@ -658,6 +483,8 @@ static GeanyFiletype *filetype_new(void)
 	GeanyFiletype *ft = g_new0(GeanyFiletype, 1);
 
 	ft->lang = -2;	/* assume no tagmanager parser */
+	/* pattern must not be null */
+	ft->pattern = g_new0(gchar*, 1);
 	ft->project_list_entry = -1; /* no entry */
 
 	ft->priv = g_new0(GeanyFiletypePrivate, 1);
@@ -689,7 +516,13 @@ static void filetype_add(GeanyFiletype *ft)
 	g_ptr_array_add(filetypes_array, ft);
 	g_hash_table_insert(filetypes_hash, ft->name, ft);
 
-	filetypes_by_title = g_slist_insert_sorted(filetypes_by_title, ft, cmp_filetype);
+	/* list will be sorted later */
+	filetypes_by_title = g_slist_prepend(filetypes_by_title, ft);
+
+	if (!ft->mime_type)
+		ft->mime_type = g_strdup("text/plain");
+
+	ft->icon = ui_get_mime_icon(ft->mime_type, GTK_ICON_SIZE_MENU);
 }
 
 
@@ -709,7 +542,6 @@ static void add_custom_filetype(const gchar *filename)
 	ft = filetype_new();
 	ft->name = g_strdup(fn);
 	filetype_make_title(ft, TITLE_FILE);
-	ft->pattern = g_new0(gchar*, 1);
 	ft->group = GEANY_FILETYPE_GROUP_CUSTOM;
 	ft->priv->custom = TRUE;
 	filetype_add(ft);
@@ -767,6 +599,9 @@ void filetypes_init_types()
 	}
 	init_custom_filetypes(app->datadir);
 	init_custom_filetypes(utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, NULL));
+
+	/* sort last instead of on insertion to prevent exponential time */
+	filetypes_by_title = g_slist_sort(filetypes_by_title, cmp_filetype);
 }
 
 
@@ -893,6 +728,33 @@ static gboolean match_basename(gconstpointer pft, gconstpointer user_data)
 }
 
 
+static GeanyFiletype *check_builtin_filenames(const gchar *utf8_filename)
+{
+	gchar *lfn = NULL;
+	const gchar *path;
+	gboolean found = FALSE;
+
+#ifdef G_OS_WIN32
+	/* use lower case basename */
+	lfn = g_utf8_strdown(utf8_filename, -1);
+#else
+	lfn = g_strdup(utf8_filename);
+#endif
+	setptr(lfn, utils_get_locale_from_utf8(lfn));
+
+	path = utils_build_path(app->configdir, GEANY_FILEDEFS_SUBDIR, "filetypes.", NULL);
+	if (g_str_has_prefix(lfn, path))
+		found = TRUE;
+
+	path = utils_build_path(app->datadir, "filetypes.", NULL);
+	if (g_str_has_prefix(lfn, path))
+		found = TRUE;
+
+	g_free(lfn);
+	return found ? filetypes[GEANY_FILETYPES_CONF] : NULL;
+}
+
+
 /* Detect filetype only based on the filename extension.
  * utf8_filename can include the full path. */
 GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
@@ -900,6 +762,10 @@ GeanyFiletype *filetypes_detect_from_extension(const gchar *utf8_filename)
 	gchar *base_filename;
 	GeanyFiletype *ft;
 
+	ft = check_builtin_filenames(utf8_filename);
+	if (ft)
+		return ft;
+
 	/* to match against the basename of the file (because of Makefile*) */
 	base_filename = g_path_get_basename(utf8_filename);
 #ifdef G_OS_WIN32
@@ -1152,13 +1018,11 @@ static void set_error_regex(GeanyFiletype *ft, gchar *string)
 {
 	setptr(ft->error_regex_string, string);
 
-#ifdef HAVE_REGCOMP
 	if (ft->priv->error_regex_compiled)
 		regfree(&ft->priv->error_regex);
 
 	ft->priv->error_regex_compiled = FALSE;
 	/* regex will be compiled when needed */
-#endif
 }
 
 
@@ -1171,6 +1035,7 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	g_free(ft->name);
 	g_free(ft->title);
 	g_free(ft->extension);
+	g_free(ft->mime_type);
 	g_free(ft->comment_open);
 	g_free(ft->comment_close);
 	g_free(ft->context_action_cmd);
@@ -1178,6 +1043,8 @@ static void filetype_free(gpointer data, G_GNUC_UNUSED gpointer user_data)
 	g_free(ft->ftdefcmds);
 	g_free(ft->execcmds);
 	set_error_regex(ft, NULL);
+	if (ft->icon)
+		g_object_unref(ft->icon);
 
 	g_strfreev(ft->pattern);
 	g_free(ft->priv);
@@ -1245,7 +1112,7 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 	if (result == NULL) result = g_key_file_get_string(config, "settings", "context_action_cmd", NULL);
 	if (G_LIKELY(result != NULL))
 	{
-		filetypes[ft_id]->context_action_cmd = result;
+		setptr(filetypes[ft_id]->context_action_cmd, result);
 	}
 
 	result = utils_get_setting_string(configh, "settings", "tag_parser", NULL);
@@ -1270,6 +1137,11 @@ static void load_settings(gint ft_id, GKeyFile *config, GKeyFile *configh)
 		g_free(result);
 	}
 
+	ft->priv->symbol_list_sort_mode = utils_get_setting(integer, configh, config, "settings",
+		"symbol_list_sort_mode", SYMBOLS_SORT_BY_NAME);
+	ft->priv->xml_indent_tags = utils_get_setting(boolean, configh, config, "settings",
+		"xml_indent_tags", FALSE);
+
 	/* read build settings */
 	build_load_menu(config, GEANY_BCS_FT, (gpointer)ft);
 	build_load_menu(configh, GEANY_BCS_HOME_FT, (gpointer)ft);
@@ -1321,8 +1193,7 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 	config_home = g_key_file_new();
 	{
 		/* highlighting uses GEANY_FILETYPES_NONE for common settings */
-		gchar *ext = (ft_id != GEANY_FILETYPES_NONE) ?
-			filetypes_get_conf_extension(ft_id) : g_strdup("common");
+		gchar *ext = filetypes_get_conf_extension(ft_id);
 		gchar *f0 = g_strconcat(app->datadir, G_DIR_SEPARATOR_S "filetypes.", ext, NULL);
 		gchar *f = g_strconcat(app->configdir,
 			G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", ext, NULL);
@@ -1345,7 +1216,7 @@ void filetypes_load_config(gint ft_id, gboolean reload)
 
 gchar *filetypes_get_conf_extension(gint filetype_idx)
 {
-	gchar *result;
+	gchar *result, *ptr;
 	GeanyFiletype *ft = filetypes[filetype_idx];
 
 	if (ft->priv->custom)
@@ -1357,7 +1228,14 @@ gchar *filetypes_get_conf_extension(gint filetype_idx)
 		case GEANY_FILETYPES_CPP: result = g_strdup("cpp"); break;
 		case GEANY_FILETYPES_CS: result = g_strdup("cs"); break;
 		case GEANY_FILETYPES_MAKE: result = g_strdup("makefile"); break;
-		default: result = g_ascii_strdown(ft->name, -1); break;
+		case GEANY_FILETYPES_NONE: result = g_strdup("common"); break;
+		default:
+			result = g_ascii_strdown(ft->name, -1);
+			/* truncate at slash (e.g. for Matlab/Octave) */
+			ptr = strstr(result, "/");
+			if (ptr)
+				*ptr = 0x0;
+			break;
 	}
 	return result;
 }
@@ -1369,7 +1247,7 @@ void filetypes_save_commands(void)
 		G_DIR_SEPARATOR_S GEANY_FILEDEFS_SUBDIR G_DIR_SEPARATOR_S "filetypes.", NULL);
 	guint i;
 
-	for (i = 1; i < filetypes_array->len; i++)
+	for (i = 0; i < filetypes_array->len; i++)
 	{
 		GKeyFile *config_home;
 		gchar *fname, *ext, *data;
@@ -1466,7 +1344,6 @@ GeanyFiletype *filetypes_lookup_by_name(const gchar *name)
 }
 
 
-#ifdef HAVE_REGCOMP
 static gchar *get_regex_match_string(const gchar *message, regmatch_t *pmatch, gint match_idx)
 {
 	return g_strndup(&message[pmatch[match_idx].rm_so],
@@ -1485,11 +1362,10 @@ static void compile_regex(GeanyFiletype *ft, regex_t *regex, gchar *regstr)
 		gchar buf[256];
 		regerror(retval, regex, buf, sizeof buf);
 		ui_set_statusbar(TRUE, _("Bad regex for filetype %s: %s"),
-			ft->name, buf);
+			filetypes_get_display_name(ft), buf);
 	}
 	/* regex will be freed in set_error_regex(). */
 }
-#endif
 
 
 gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
@@ -1498,10 +1374,9 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	gchar *regstr;
 	gchar **tmp;
 	GeanyDocument *doc;
-#ifdef HAVE_REGCOMP
 	regex_t *regex;
 	regmatch_t pmatch[3];
-#endif
+
 	if (ft == NULL)
 	{
 		doc = document_get_current();
@@ -1512,11 +1387,6 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 	if (tmp == NULL)
 		return FALSE;
 	regstr = *tmp;
-#ifndef HAVE_REGCOMP
-	if (!NZV(regstr))
-		geany_debug("No regex support - maybe you should configure with --enable-gnu-regex!");
-	return FALSE;
-#else
 	regex = &ft->priv->error_regex;
 
 	*filename = NULL;
@@ -1567,7 +1437,6 @@ gboolean filetypes_parse_error_message(GeanyFiletype *ft, const gchar *message,
 		}
 	}
 	return *filename != NULL;
-#endif
 }
 
 
@@ -1605,16 +1474,15 @@ void filetypes_read_extensions(void)
 		gchar **list = g_key_file_get_string_list(
 			(userset) ? userconfig : sysconfig, "Extensions", filetypes[i]->name, &len, NULL);
 
-		if (G_LIKELY(list) && G_LIKELY(len > 0))
-		{
-			g_strfreev(filetypes[i]->pattern);
-			filetypes[i]->pattern = list;
+		g_strfreev(filetypes[i]->pattern);
+		/* Note: we allow 'Foo=' to remove all patterns */
+		if (!list)
+			list = g_new0(gchar*, 1);
+		filetypes[i]->pattern = list;
+
 #ifdef G_OS_WIN32
-			convert_filetype_extensions_to_lower_case(filetypes[i]->pattern, len);
+		convert_filetype_extensions_to_lower_case(filetypes[i]->pattern, len);
 #endif
-		}
-		else
-			g_strfreev(list);
 	}
 
 	g_free(sysconfigfile);
@@ -1622,6 +1490,7 @@ void filetypes_read_extensions(void)
 	g_key_file_free(sysconfig);
 	g_key_file_free(userconfig);
 
+	/* Redetect filetype of any documents with none set */
 	foreach_document(i)
 	{
 		GeanyDocument *doc = documents[i];
@@ -1675,3 +1544,12 @@ void filetypes_reload(void)
 }
 
 
+/** Gets @c ft->name or a translation for filetype None.
+ * @param ft .
+ * @return .
+ * @since Geany 0.20 */
+const gchar *filetypes_get_display_name(GeanyFiletype *ft)
+{
+	return ft->id == GEANY_FILETYPES_NONE ? _("None") : ft->name;
+}
+
