@@ -1,8 +1,8 @@
 /*
  *      geanyentryaction.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2008-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2008-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2008-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2008-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -38,9 +38,12 @@ G_BEGIN_DECLS
 typedef struct _GeanyEntryAction				GeanyEntryAction;
 typedef struct _GeanyEntryActionClass			GeanyEntryActionClass;
 
+struct _GeanyEntryActionPrivate;
+
 struct _GeanyEntryAction
 {
 	GtkAction parent;
+	struct _GeanyEntryActionPrivate *priv;
 };
 
 struct _GeanyEntryActionClass
