@@ -1,8 +1,8 @@
 /*
  *      geanyobject.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2007-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2007-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2007-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2007-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: geanyobject.h 3976 2009-07-16 15:04:21Z ntrel $
+ * $Id: geanyobject.h 5235 2010-09-16 15:14:09Z ntrel $
  */
 
 
@@ -34,14 +34,20 @@ typedef enum
 {
 	GCB_DOCUMENT_NEW,
 	GCB_DOCUMENT_OPEN,
+	GCB_DOCUMENT_BEFORE_SAVE,
 	GCB_DOCUMENT_SAVE,
+	GCB_DOCUMENT_FILETYPE_SET,
 	GCB_DOCUMENT_ACTIVATE,
 	GCB_DOCUMENT_CLOSE,
 	GCB_PROJECT_OPEN,
 	GCB_PROJECT_SAVE,
 	GCB_PROJECT_CLOSE,
+	GCB_PROJECT_DIALOG_CREATE,
+	GCB_PROJECT_DIALOG_CONFIRMED,
 	GCB_UPDATE_EDITOR_MENU,
 	GCB_EDITOR_NOTIFY,
+	GCB_GEANY_STARTUP_COMPLETE,
+	GCB_BUILD_START,
 	GCB_SAVE_SETTINGS,
 	GCB_LOAD_SETTINGS,
 	GCB_MAX
@@ -76,14 +82,20 @@ struct _GeanyObjectClass
 
 	void (*document_new)(GeanyDocument *doc);
 	void (*document_open)(GeanyDocument *doc);
+	void (*document_before_save)(GeanyDocument *doc);
 	void (*document_save)(GeanyDocument *doc);
+	void (*document_filetype_set)(GeanyDocument *doc, GeanyFiletype *filetype_old);
 	void (*document_activate)(GeanyDocument *doc);
 	void (*document_close)(GeanyDocument *doc);
 	void (*project_open)(GKeyFile *keyfile);
 	void (*project_save)(GKeyFile *keyfile);
 	void (*project_close)(void);
+	void (*project_dialog_create)(GtkWidget *notebook);
+	void (*project_dialog_confirmed)(GtkWidget *notebook);
 	void (*update_editor_menu)(const gchar *word, gint click_pos, GeanyDocument *doc);
 	gboolean (*editor_notify)(GeanyEditor *editor, gpointer scnt);
+	void (*geany_startup_complete)(void);
+	void (*build_start)(void);
 	void (*save_settings)(GKeyFile *keyfile);
 	void (*load_settings)(GKeyFile *keyfile);
 };
