@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: geanywraplabel.c 4862 2010-04-25 17:43:39Z eht16 $
+ *  $Id: geanywraplabel.c 5283 2010-10-06 15:32:40Z ntrel $
  */
 
 /*
@@ -27,7 +27,7 @@
  */
 
 
-#include <gtk/gtk.h>
+#include "geany.h"
 #include "utils.h"
 #include "geanywraplabel.h"
 
