@@ -18,13 +18,13 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keybindings.c 5147 2010-08-12 12:30:16Z ntrel $
+ * $Id: keybindings.c 5498 2010-12-24 13:31:39Z ntrel $
  */
 
 /**
  * @file keybindings.h
  * Configurable keyboard shortcuts.
- * - keybindings_send_command() mimics a built-in keybinding.
+ * - keybindings_send_command() mimics a built-in keybinding action.
  * - @ref GeanyKeyGroupID lists groups of built-in keybindings.
  * @see plugin_set_key_group().
  **/
@@ -57,21 +57,27 @@
 #include "geanywraplabel.h"
 #include "main.h"
 #include "search.h"
+#ifdef HAVE_VTE
+# include "vte.h"
+#endif
 
 
 GPtrArray *keybinding_groups;	/* array of GeanyKeyGroup pointers */
 
 /* keyfile group name for non-plugin KB groups */
-const gchar keybindings_keyfile_group_name[] = "Bindings";
+static const gchar keybindings_keyfile_group_name[] = "Bindings";
+
+/* core keybindings */
+static GeanyKeyBinding binding_ids[GEANY_KEYS_COUNT];
 
 static GtkAccelGroup *kb_accel_group = NULL;
 static const gboolean swap_alt_tab_order = FALSE;
 
-const gsize MAX_MRU_DOCS = 20;
+static const gsize MAX_MRU_DOCS = 20;
 static GQueue *mru_docs = NULL;
 static guint mru_pos = 0;
 
-static gboolean switch_dialog_cancelled = TRUE;
+static gboolean switch_in_progress = FALSE;
 static GtkWidget *switch_dialog = NULL;
 static GtkWidget *switch_dialog_label = NULL;
 
@@ -122,9 +128,13 @@ static void add_popup_menu_accels(void);
  * @since 0.19. */
 GeanyKeyBinding *keybindings_get_item(GeanyKeyGroup *group, gsize key_id)
 {
-	g_assert(key_id < group->count);
-
-	return &group->keys[key_id];
+	if (group->plugin)
+	{
+		g_assert(key_id < group->plugin_key_count);
+		return &group->plugin_keys[key_id];
+	}
+	g_assert(key_id < GEANY_KEYS_COUNT);
+	return &binding_ids[key_id];
 }
 
 
@@ -146,7 +156,12 @@ GeanyKeyBinding *keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
 		GeanyKeyCallback callback, guint key, GdkModifierType mod,
 		const gchar *kf_name, const gchar *label, GtkWidget *menu_item)
 {
-	GeanyKeyBinding *kb = keybindings_get_item(group, key_id);
+	GeanyKeyBinding *kb;
+
+	g_assert(group->name);
+	kb = keybindings_get_item(group, key_id);
+	g_assert(!kb->name);
+	g_ptr_array_add(group->key_items, kb);
 
 	if (group->plugin)
 	{
@@ -156,7 +171,7 @@ GeanyKeyBinding *keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
 	}
 	else
 	{
-		/* we don't touch them unless group->plugin is set, cast is safe */
+		/* we don't touch these strings unless group->plugin is set, const cast is safe */
 		kb->name = (gchar *)kf_name;
 		kb->label = (gchar *)label;
 	}
@@ -164,21 +179,21 @@ GeanyKeyBinding *keybindings_set_item(GeanyKeyGroup *group, gsize key_id,
 	kb->mods = mod;
 	kb->callback = callback;
 	kb->menu_item = menu_item;
+	kb->id = key_id;
 	return kb;
 }
 
 
 static GeanyKeyGroup *add_kb_group(GeanyKeyGroup *group,
-		const gchar *name, const gchar *label, gsize count, GeanyKeyBinding *keys,
-		GeanyKeyGroupCallback callback)
+		const gchar *name, const gchar *label, GeanyKeyGroupCallback callback, gboolean plugin)
 {
 	g_ptr_array_add(keybinding_groups, group);
 
 	group->name = name;
 	group->label = label;
-	group->count = count;
-	group->keys = keys;
 	group->callback = callback;
+	group->plugin = plugin;
+	group->key_items = g_ptr_array_new();
 	return group;
 }
 
@@ -188,16 +203,11 @@ static GeanyKeyGroup *add_kb_group(GeanyKeyGroup *group,
 	ui_lookup_widget(main_widgets.window, G_STRINGIFY(widget_name))
 
 /* Expansion for group_id = FILE:
- * static GeanyKeyBinding FILE_keys[GEANY_KEYS_FILE_COUNT]; */
-#define DECLARE_KEYS(group_id) \
-	static GeanyKeyBinding group_id ## _keys[GEANY_KEYS_ ## group_id ## _COUNT]
-
-/* Expansion for group_id = FILE:
- * add_kb_group(&groups[GEANY_KEY_GROUP_FILE], NULL, _("File menu"),
- * 	GEANY_KEYS_FILE_COUNT, FILE_keys, callback); */
+ * add_kb_group(&groups[GEANY_KEY_GROUP_FILE],
+ * 	keybindings_keyfile_group_name, _("File menu"), callback); */
 #define ADD_KB_GROUP(group_id, label, callback) \
-	add_kb_group(&groups[GEANY_KEY_GROUP_ ## group_id], keybindings_keyfile_group_name, label, \
-		GEANY_KEYS_ ## group_id ## _COUNT, group_id ## _keys, callback)
+	add_kb_group(&groups[GEANY_KEY_GROUP_ ## group_id],\
+		keybindings_keyfile_group_name, label, callback, FALSE)
 
 /* Init all fields of keys with default values.
  * The menu_item field is always the main menu item, popup menu accelerators are
@@ -206,23 +216,6 @@ static void init_default_kb(void)
 {
 	static GeanyKeyGroup groups[GEANY_KEY_GROUP_COUNT];
 	GeanyKeyGroup *group;
-	DECLARE_KEYS(FILE);
-	DECLARE_KEYS(PROJECT);
-	DECLARE_KEYS(EDITOR);
-	DECLARE_KEYS(CLIPBOARD);
-	DECLARE_KEYS(SELECT);
-	DECLARE_KEYS(FORMAT);
-	DECLARE_KEYS(INSERT);
-	DECLARE_KEYS(SETTINGS);
-	DECLARE_KEYS(SEARCH);
-	DECLARE_KEYS(GOTO);
-	DECLARE_KEYS(VIEW);
-	DECLARE_KEYS(FOCUS);
-	DECLARE_KEYS(NOTEBOOK);
-	DECLARE_KEYS(DOCUMENT);
-	DECLARE_KEYS(BUILD);
-	DECLARE_KEYS(TOOLS);
-	DECLARE_KEYS(HELP);
 
 	group = ADD_KB_GROUP(FILE, _("File"), cb_func_file_action);
 
@@ -339,7 +332,7 @@ static void init_default_kb(void)
 
 	keybindings_set_item(group, GEANY_KEYS_FORMAT_TOGGLECASE, NULL,
 		GDK_u, GDK_CONTROL_MASK | GDK_MOD1_MASK, "edit_togglecase",
-		_("Toggle Case of Selection"), LW(menu_toggle_case2));
+		_("T_oggle Case of Selection"), LW(menu_toggle_case2));
 	keybindings_set_item(group, GEANY_KEYS_FORMAT_COMMENTLINETOGGLE, NULL,
 		GDK_e, GDK_CONTROL_MASK, "edit_commentlinetoggle", _("Toggle line commentation"),
 		LW(menu_toggle_line_commentation1));
@@ -380,6 +373,10 @@ static void init_default_kb(void)
 	keybindings_set_item(group, GEANY_KEYS_INSERT_ALTWHITESPACE, NULL,
 		0, 0, "edit_insertwhitespace", _("_Insert Alternative White Space"),
 		LW(insert_alternative_white_space1));
+	keybindings_set_item(group, GEANY_KEYS_INSERT_LINEBEFORE, NULL,
+		0, 0, "edit_insertlinebefore", _("Insert New Line Before Current"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_INSERT_LINEAFTER, NULL,
+		0, 0, "edit_insertlineafter", _("Insert New Line After Current"), NULL);
 
 	group = ADD_KB_GROUP(SETTINGS, _("Settings"), NULL);
 
@@ -399,9 +396,9 @@ static void init_default_kb(void)
 		GDK_g, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "menu_findprevious", _("Find Previous"),
 		LW(find_previous1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_FINDNEXTSEL, NULL,
-		0, 0, "menu_findnextsel", _("Find Next Selection"), LW(find_nextsel1));
+		0, 0, "menu_findnextsel", _("Find Next _Selection"), LW(find_nextsel1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_FINDPREVSEL, NULL,
-		0, 0, "menu_findprevsel", _("Find Previous Selection"), LW(find_prevsel1));
+		0, 0, "menu_findprevsel", _("Find Pre_vious Selection"), LW(find_prevsel1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_REPLACE, NULL,
 		GDK_h, GDK_CONTROL_MASK, "menu_replace", _("Replace"), LW(replace1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_FINDINFILES, NULL, GDK_f,
@@ -412,11 +409,11 @@ static void init_default_kb(void)
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_PREVIOUSMESSAGE, NULL,
 		0, 0, "menu_previousmessage", _("Previous Message"), LW(previous_message1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_FINDUSAGE, NULL,
-		0, 0, "popup_findusage", _("Find Usage"), NULL);
+		0, 0, "popup_findusage", _("Find Usage"), LW(find_usage1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_FINDDOCUMENTUSAGE, NULL,
-		0, 0, "popup_finddocumentusage", _("Find Document Usage"), NULL);
+		0, 0, "popup_finddocumentusage", _("Find Document Usage"), LW(find_document_usage1));
 	keybindings_set_item(group, GEANY_KEYS_SEARCH_MARKALL, NULL,
-		GDK_m, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "find_markall", _("Mark All"), NULL);
+		GDK_m, GDK_CONTROL_MASK | GDK_SHIFT_MASK, "find_markall", _("_Mark All"), LW(mark_all1));
 
 	group = ADD_KB_GROUP(GOTO, _("Go to"), cb_func_goto_action);
 
@@ -439,9 +436,9 @@ static void init_default_kb(void)
 		GDK_comma, GDK_CONTROL_MASK, "edit_gotopreviousmarker",
 		_("_Go to Previous Marker"), LW(go_to_previous_marker1));
 	keybindings_set_item(group, GEANY_KEYS_GOTO_TAGDEFINITION, NULL,
-		0, 0, "popup_gototagdefinition", _("Go to Tag Definition"), NULL);
+		0, 0, "popup_gototagdefinition", _("Go to Tag Definition"), LW(goto_tag_definition1));
 	keybindings_set_item(group, GEANY_KEYS_GOTO_TAGDECLARATION, NULL,
-		0, 0, "popup_gototagdeclaration", _("Go to Tag Declaration"), NULL);
+		0, 0, "popup_gototagdeclaration", _("Go to Tag Declaration"), LW(goto_tag_declaration1));
 	keybindings_set_item(group, GEANY_KEYS_GOTO_LINESTART, NULL,
 		GDK_Home, 0, "edit_gotolinestart", _("Go to Start of Line"), NULL);
 	keybindings_set_item(group, GEANY_KEYS_GOTO_LINEEND, NULL,
@@ -476,24 +473,24 @@ static void init_default_kb(void)
 
 	keybindings_set_item(group, GEANY_KEYS_FOCUS_EDITOR, NULL,
 		GDK_F2, 0, "switch_editor", _("Switch to Editor"), NULL);
-	keybindings_set_item(group, GEANY_KEYS_FOCUS_SCRIBBLE, NULL,
-		GDK_F6, 0, "switch_scribble", _("Switch to Scribble"), NULL);
-	keybindings_set_item(group, GEANY_KEYS_FOCUS_VTE, NULL,
-		GDK_F4, 0, "switch_vte", _("Switch to VTE"), NULL);
 	keybindings_set_item(group, GEANY_KEYS_FOCUS_SEARCHBAR, NULL,
 		GDK_F7, 0, "switch_search_bar", _("Switch to Search Bar"), NULL);
-	keybindings_set_item(group, GEANY_KEYS_FOCUS_SIDEBAR, NULL,
-		0, 0, "switch_sidebar", _("Switch to Sidebar"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_FOCUS_MESSAGE_WINDOW, NULL,
+		0, 0, "switch_message_window", _("Switch to Message Window"), NULL);
 	keybindings_set_item(group, GEANY_KEYS_FOCUS_COMPILER, NULL,
 		0, 0, "switch_compiler", _("Switch to Compiler"), NULL);
 	keybindings_set_item(group, GEANY_KEYS_FOCUS_MESSAGES, NULL,
 		0, 0, "switch_messages", _("Switch to Messages"), NULL);
-	keybindings_set_item(group, GEANY_KEYS_FOCUS_MESSAGE_WINDOW, NULL,
-		0, 0, "switch_message_window", _("Switch to Message Window"), NULL);
-	keybindings_set_item(group, GEANY_KEYS_FOCUS_SIDEBAR_DOCUMENT_LIST, NULL,
-		0, 0, "switch_sidebar_doc_list", _("Switch to Sidebar Document List"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_FOCUS_SCRIBBLE, NULL,
+		GDK_F6, 0, "switch_scribble", _("Switch to Scribble"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_FOCUS_VTE, NULL,
+		GDK_F4, 0, "switch_vte", _("Switch to VTE"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_FOCUS_SIDEBAR, NULL,
+		0, 0, "switch_sidebar", _("Switch to Sidebar"), NULL);
 	keybindings_set_item(group, GEANY_KEYS_FOCUS_SIDEBAR_SYMBOL_LIST, NULL,
 		0, 0, "switch_sidebar_symbol_list", _("Switch to Sidebar Symbol List"), NULL);
+	keybindings_set_item(group, GEANY_KEYS_FOCUS_SIDEBAR_DOCUMENT_LIST, NULL,
+		0, 0, "switch_sidebar_doc_list", _("Switch to Sidebar Document List"), NULL);
 
 	group = ADD_KB_GROUP(NOTEBOOK, _("Notebook tab"), NULL);
 
@@ -576,7 +573,7 @@ static void on_notebook_switch_page(void)
 
 	/* when closing current doc, old is NULL.
 	 * Don't add to the mru list when switch dialog is visible. */
-	if (old && switch_dialog_cancelled)
+	if (old && !switch_in_progress)
 	{
 		g_queue_remove(mru_docs, old);
 		g_queue_push_head(mru_docs, old);
@@ -618,12 +615,11 @@ void keybindings_init(void)
 	g_signal_connect(geany_object, "document-close",
 		G_CALLBACK(on_document_close), NULL);
 
+	memset(binding_ids, 0, sizeof binding_ids);
 	keybinding_groups = g_ptr_array_sized_new(GEANY_KEY_GROUP_COUNT);
-
 	kb_accel_group = gtk_accel_group_new();
 
 	init_default_kb();
-
 	gtk_window_add_accel_group(GTK_WINDOW(main_widgets.window), kb_accel_group);
 
 	g_signal_connect(main_widgets.window, "key-press-event", G_CALLBACK(on_key_press_event), NULL);
@@ -640,15 +636,10 @@ static void keybindings_foreach(KBItemCallback cb, gpointer user_data)
 	GeanyKeyGroup *group;
 	GeanyKeyBinding *kb;
 
-	for (g = 0; g < keybinding_groups->len; g++)
+	foreach_ptr_array(group, g, keybinding_groups)
 	{
-		group = g_ptr_array_index(keybinding_groups, g);
-		for (i = 0; i < group->count; i++)
-		{
-			kb = &group->keys[i];
-
+		foreach_ptr_array(kb, i, group->key_items)
 			cb(group, kb, user_data);
-		}
 	}
 }
 
@@ -708,7 +699,7 @@ void keybindings_load_keyfile(void)
 
 static void add_menu_accel(GeanyKeyGroup *group, guint kb_id, GtkWidget *menuitem)
 {
-	GeanyKeyBinding *kb = &group->keys[kb_id];
+	GeanyKeyBinding *kb = keybindings_get_item(group, kb_id);
 
 	if (kb->key != 0)
 		gtk_widget_add_accelerator(menuitem, "activate", kb_accel_group,
@@ -734,6 +725,7 @@ static void add_popup_menu_accels(void)
 
 	group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_INSERT);
 	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_INSERT_DATE, insert_date_custom2);
+	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_INSERT_ALTWHITESPACE, insert_alternative_white_space1);
 
 	group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_FILE);
 	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_FILE_OPENSELECTED, menu_open_selected_file2);
@@ -743,9 +735,7 @@ static void add_popup_menu_accels(void)
 	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_SEARCH_FINDDOCUMENTUSAGE, find_document_usage1);
 
 	group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_GOTO);
-	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_GOTO_LINE, go_to_line);
 	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_GOTO_TAGDEFINITION, goto_tag_definition1);
-	GEANY_ADD_POPUP_ACCEL(GEANY_KEYS_GOTO_TAGDECLARATION, goto_tag_declaration1);
 
 	/* Format and Commands share the menu bar submenus */
 	/* Build menu items are set if the build menus are created */
@@ -792,6 +782,12 @@ void keybindings_write_to_file(void)
 
 void keybindings_free(void)
 {
+	GeanyKeyGroup *group;
+	gsize g;
+
+	foreach_ptr_array(group, g, keybinding_groups)
+		keybindings_free_group(group);
+
 	g_ptr_array_free(keybinding_groups, TRUE);
 	g_queue_free(mru_docs);
 }
@@ -813,24 +809,20 @@ static void fill_shortcut_labels_treeview(GtkWidget *tree)
 
 	store = gtk_list_store_new(3, G_TYPE_STRING, G_TYPE_STRING, PANGO_TYPE_WEIGHT);
 
-	for (g = 0; g < keybinding_groups->len; g++)
+	foreach_ptr_array(group, g, keybinding_groups)
 	{
-		group = g_ptr_array_index(keybinding_groups, g);
-
 		if (g > 0)
 		{
 			gtk_list_store_append(store, &iter);
 			gtk_list_store_set(store, &iter, -1);
 		}
-
 		gtk_list_store_append(store, &iter);
 		gtk_list_store_set(store, &iter, 0, group->label, 2, PANGO_WEIGHT_BOLD, -1);
 
-		for (i = 0; i < group->count; i++)
+		foreach_ptr_array(kb, i, group->key_items)
 		{
 			gchar *shortcut, *label;
 
-			kb = &group->keys[i];
 			label = keybindings_get_label(kb);
 			shortcut = gtk_accelerator_get_label(kb->key, kb->mods);
 
@@ -841,7 +833,6 @@ static void fill_shortcut_labels_treeview(GtkWidget *tree)
 			g_free(label);
 		}
 	}
-
 	gtk_tree_view_set_model(GTK_TREE_VIEW(tree), GTK_TREE_MODEL(store));
 	g_object_unref(store);
 }
@@ -879,7 +870,7 @@ static GtkWidget *create_dialog(void)
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	text_renderer = gtk_cell_renderer_text_new();
-    column = gtk_tree_view_column_new_with_attributes(NULL, text_renderer, "text", 1, NULL);
+	column = gtk_tree_view_column_new_with_attributes(NULL, text_renderer, "text", 1, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
 	fill_shortcut_labels_treeview(tree);
@@ -936,7 +927,7 @@ void keybindings_show_shortcuts(void)
 static gboolean check_fixed_kb(guint keyval, guint state)
 {
 	/* check alt-0 to alt-9 for setting current notebook page */
-	if (state & GDK_MOD1_MASK && keyval >= GDK_0 && keyval <= GDK_9)
+	if (state == GDK_MOD1_MASK && keyval >= GDK_0 && keyval <= GDK_9)
 	{
 		gint page = keyval - GDK_0 - 1;
 		gint npages = gtk_notebook_get_n_pages(GTK_NOTEBOOK(main_widgets.notebook));
@@ -1079,6 +1070,8 @@ static gboolean set_sensitive(gpointer widget)
 static gboolean check_vte(GdkModifierType state, guint keyval)
 {
 	guint i;
+	GeanyKeyBinding *kb;
+	GeanyKeyGroup *group;
 	GtkWidget *widget;
 
 	if (! vc->enable_bash_keys)
@@ -1092,10 +1085,9 @@ static gboolean check_vte(GdkModifierType state, guint keyval)
 		return FALSE;
 
 	/* make focus commands override any bash commands */
-	for (i = 0; i < GEANY_KEYS_FOCUS_COUNT; i++)
+	group = g_ptr_array_index(keybinding_groups, GEANY_KEY_GROUP_FOCUS);
+	foreach_ptr_array(kb, i, group->key_items)
 	{
-		GeanyKeyBinding *kb = keybindings_lookup_item(GEANY_KEY_GROUP_FOCUS, i);
-
 		if (state == kb->mods && keyval == kb->key)
 			return FALSE;
 	}
@@ -1151,17 +1143,10 @@ static guint key_kp_translate(guint key_in)
 }
 
 
-/* Stripped down version of the main keypress event handler which can be used
- * to process foreign events. Instead of executing the keybinding, a pointer to the
- * keybinding structure is returned.
- * Additionally, the group_id and binding_id are filled with the appropriate indexes
- * if non-NULL. */
-const GeanyKeyBinding *keybindings_check_event(GdkEventKey *ev, gint *group_id, gint *binding_id)
+/* Check if event keypress matches keybinding combo */
+gboolean keybindings_check_event(GdkEventKey *ev, GeanyKeyBinding *kb)
 {
 	guint state, keyval;
-	gsize g, i;
-	GeanyKeyGroup *group;
-	GeanyKeyBinding *kb;
 
 	if (ev->keyval == 0)
 		return FALSE;
@@ -1176,24 +1161,7 @@ const GeanyKeyBinding *keybindings_check_event(GdkEventKey *ev, gint *group_id,
 	if (keyval >= GDK_KP_Space && keyval < GDK_KP_Equal)
 		keyval = key_kp_translate(keyval);
 
-	for (g = 0; g < keybinding_groups->len; g++)
-	{
-		group = g_ptr_array_index(keybinding_groups, g);
-
-		for (i = 0; i < group->count; i++)
-		{
-			kb = &group->keys[i];
-			if (keyval == kb->key && state == kb->mods)
-			{
-				if (group_id != NULL)
-					*group_id = g;
-				if (binding_id != NULL)
-					*binding_id = i;
-				return kb;
-			}
-		}
-	}
-	return NULL;
+	return (keyval == kb->key && state == kb->mods);
 }
 
 
@@ -1233,24 +1201,21 @@ static gboolean on_key_press_event(GtkWidget *widget, GdkEventKey *ev, gpointer
 	if (check_menu_key(doc, keyval, state, ev->time))
 		return TRUE;
 
-	for (g = 0; g < keybinding_groups->len; g++)
+	foreach_ptr_array(group, g, keybinding_groups)
 	{
-		group = g_ptr_array_index(keybinding_groups, g);
-
-		for (i = 0; i < group->count; i++)
+		foreach_ptr_array(kb, i, group->key_items)
 		{
-			kb = &group->keys[i];
 			if (keyval == kb->key && state == kb->mods)
 			{
 				/* call the corresponding callback function for this shortcut */
 				if (kb->callback)
 				{
-					kb->callback(i);
+					kb->callback(kb->id);
 					return TRUE;
 				}
 				else if (group->callback)
 				{
-					if (group->callback(i))
+					if (group->callback(kb->id))
 						return TRUE;
 					else
 						continue;	/* not handled */
@@ -1298,16 +1263,14 @@ GeanyKeyBinding *keybindings_lookup_item(guint group_id, guint key_id)
 	group = g_ptr_array_index(keybinding_groups, group_id);
 
 	g_return_val_if_fail(group, NULL);
-	g_return_val_if_fail(key_id < group->count, NULL);
-
-	return &group->keys[key_id];
+	return keybindings_get_item(group, key_id);
 }
 
 
 /** Mimics a (built-in only) keybinding action.
  * 	Example: @code keybindings_send_command(GEANY_KEY_GROUP_FILE, GEANY_KEYS_FILE_OPEN); @endcode
- * 	@param group_id The index for the key group that contains the @a key_id keybinding.
- * 	@param key_id The keybinding command index. */
+ * 	@param group_id @ref GeanyKeyGroupID keybinding group index that contains the @a key_id keybinding.
+ * 	@param key_id @ref GeanyKeyBindingID keybinding index. */
 void keybindings_send_command(guint group_id, guint key_id)
 {
 	GeanyKeyBinding *kb;
@@ -1536,7 +1499,7 @@ static gboolean cb_func_build_action(guint key_id)
 		return TRUE;
 
 	menu_items = build_get_menu_items(doc->file_type->id);
-/* TODO make it a table??*/
+	/* TODO make it a table??*/
 	switch (key_id)
 	{
 		case GEANY_KEYS_BUILD_COMPILE:
@@ -1751,12 +1714,15 @@ static void cb_func_switch_tabright(G_GNUC_UNUSED guint key_id)
 static gboolean on_key_release_event(GtkWidget *widget, GdkEventKey *ev, gpointer user_data)
 {
 	/* user may have rebound keybinding to a different modifier than Ctrl, so check all */
-	if (!switch_dialog_cancelled && is_modifier_key(ev->keyval))
+	if (switch_in_progress && is_modifier_key(ev->keyval))
 	{
-		switch_dialog_cancelled = TRUE;
+		switch_in_progress = FALSE;
 
-		if (switch_dialog && GTK_WIDGET_VISIBLE(switch_dialog))
-			gtk_widget_hide(switch_dialog);
+		if (switch_dialog)
+		{
+			gtk_widget_destroy(switch_dialog);
+			switch_dialog = NULL;
+		}
 
 		mru_pos = 0;
 	}
@@ -1809,23 +1775,27 @@ static GtkWidget *create_switch_dialog(void)
 }
 
 
-static gboolean on_switch_timeout(G_GNUC_UNUSED gpointer data)
+static void update_filename_label()
 {
-	if (switch_dialog_cancelled)
+	if (!switch_dialog)
 	{
-		return FALSE;
-	}
-	if (! switch_dialog || !GTK_WIDGET_VISIBLE(switch_dialog))
-		mru_pos = 2;	/* skip past the previous document */
-	else
-		mru_pos += 1;
-
-	if (! switch_dialog)
 		switch_dialog = create_switch_dialog();
+		gtk_widget_show_all(switch_dialog);
+	}
 
 	geany_wrap_label_set_text(GTK_LABEL(switch_dialog_label),
 		DOC_FILENAME(document_get_current()));
-	gtk_widget_show_all(switch_dialog);
+}
+
+
+static gboolean on_switch_timeout(G_GNUC_UNUSED gpointer data)
+{
+	if (!switch_in_progress || switch_dialog)
+	{
+		return FALSE;
+	}
+
+	update_filename_label();
 	return FALSE;
 }
 
@@ -1848,19 +1818,25 @@ static void cb_func_switch_tablastused(G_GNUC_UNUSED guint key_id)
 
 	/* if there's a modifier key, we can switch back in MRU order each time unless
 	 * the key is released */
-	if (! switch_dialog_cancelled)
+	if (!switch_in_progress)
 	{
-		on_switch_timeout(NULL);	/* update filename label */
-	}
-	else
-	if (keybindings_lookup_item(GEANY_KEY_GROUP_NOTEBOOK,
-		GEANY_KEYS_NOTEBOOK_SWITCHTABLASTUSED)->mods)
-	{
-		switch_dialog_cancelled = FALSE;
+		switch_in_progress = TRUE;
+
+		/* because switch_in_progress was not set when we called
+		 * gtk_notebook_set_current_page() above, this function inserted last_doc
+		 * into the queue => on mru_pos = 0 there is last_doc, on mru_pos = 1
+		 * there is the currently displayed doc, so we want to continue from 2
+		 * next time this function is called */
+		mru_pos = 2;
 
 		/* delay showing dialog to give user time to let go of any modifier keys */
 		g_timeout_add(600, on_switch_timeout, NULL);
 	}
+	else
+	{
+		update_filename_label();
+		mru_pos += 1;
+	}
 }
 
 
@@ -2346,16 +2322,17 @@ static void reflow_paragraph(GeanyEditor *editor)
 	ScintillaObject *sci = editor->sci;
 	gboolean sel;
 	gint column = -1;
+	const GeanyEditorPrefs *eprefs = editor_get_prefs(editor);
 
 	if (editor->line_breaking)
 	{
 		/* use line break column if enabled */
-		column = editor_prefs.line_break_column;
+		column = eprefs->line_break_column;
 	}
-	else if (editor_get_long_line_type() != 2)
+	else if (eprefs->long_line_type != 2)
 	{
 		/* use long line if enabled */
-		column = editor_get_long_line_column();
+		column = eprefs->long_line_column;
 	}
 	else
 	{
@@ -2369,7 +2346,8 @@ static void reflow_paragraph(GeanyEditor *editor)
 	{
 		gint line, pos;
 
-		keybindings_send_command(GEANY_KEY_GROUP_SELECT, GEANY_KEYS_SELECT_PARAGRAPH);
+		editor_select_indent_block(editor);
+
 		/* deselect last line break */
 		pos = sci_get_selection_end(sci);
 		line = sci_get_line_from_position(sci, pos);
@@ -2466,6 +2444,14 @@ static gboolean cb_func_select_action(guint key_id)
 		g_signal_emit_by_name(msgwindow.scribble, "select-all", TRUE);
 		return TRUE;
 	}
+	/* special case for Select All in the VTE widget */
+#ifdef HAVE_VTE
+	else if (key_id == GEANY_KEYS_SELECT_ALL && vte_info.have_vte && focusw == vc->vte)
+	{
+		vte_select_all();
+		return TRUE;
+	}
+#endif
 	/* special case for Select All in the toolbar search widget */
 	else if (key_id == GEANY_KEYS_SELECT_ALL && focusw == toolbar_search_entry)
 	{
@@ -2559,6 +2545,27 @@ static gboolean cb_func_document_action(guint key_id)
 }
 
 
+static void insert_line_after(GeanyEditor *editor)
+{
+	ScintillaObject *sci = editor->sci;
+
+	sci_send_command(sci, SCI_LINEEND);
+	sci_send_command(sci, SCI_NEWLINE);
+}
+
+
+static void insert_line_before(GeanyEditor *editor)
+{
+	ScintillaObject *sci = editor->sci;
+	gint line = sci_get_current_line(sci);
+	gint indentpos = sci_get_line_indent_position(sci, line);
+
+	sci_set_current_position(sci, indentpos, TRUE);
+	sci_send_command(sci, SCI_NEWLINE);
+	sci_send_command(sci, SCI_LINEUP);
+}
+
+
 /* common function for insert keybindings, only valid when scintilla has focus. */
 static gboolean cb_func_insert_action(guint key_id)
 {
@@ -2578,6 +2585,12 @@ static gboolean cb_func_insert_action(guint key_id)
 			gtk_menu_item_activate(GTK_MENU_ITEM(
 				ui_lookup_widget(main_widgets.window, "insert_date_custom1")));
 			break;
+		case GEANY_KEYS_INSERT_LINEAFTER:
+			insert_line_after(doc->editor);
+			break;
+		case GEANY_KEYS_INSERT_LINEBEFORE:
+			insert_line_before(doc->editor);
+			break;
 	}
 	return TRUE;
 }
@@ -2600,7 +2613,7 @@ void keybindings_update_combo(GeanyKeyBinding *kb, guint key, GdkModifierType mo
 }
 
 
-/* used for plugins */
+/* used for plugins, can be called repeatedly. */
 GeanyKeyGroup *keybindings_set_group(GeanyKeyGroup *group, const gchar *section_name,
 		const gchar *label, gsize count, GeanyKeyGroupCallback callback)
 {
@@ -2611,33 +2624,33 @@ GeanyKeyGroup *keybindings_set_group(GeanyKeyGroup *group, const gchar *section_
 	g_return_val_if_fail(!g_str_equal(section_name, keybindings_keyfile_group_name), NULL);
 
 	if (!group)
-		group = g_new0(GeanyKeyGroup, 1);
-
-	if (!group->keys || count > group->count)
 	{
-		/* allow resizing existing array of keys */
-		group->keys = g_renew(GeanyKeyBinding, group->keys, count);
-		memset(group->keys + group->count, 0, (count - group->count) * sizeof(GeanyKeyBinding));
+		group = g_new0(GeanyKeyGroup, 1);
+		add_kb_group(group, section_name, label, callback, TRUE);
 	}
-	group->plugin = TRUE;
-	add_kb_group(group, section_name, label, count, group->keys, callback);
+	g_free(group->plugin_keys);
+	group->plugin_keys = g_new0(GeanyKeyBinding, count);
+	group->plugin_key_count = count;
+	g_ptr_array_set_size(group->key_items, 0);
 	return group;
 }
 
 
-/* used for plugins */
 void keybindings_free_group(GeanyKeyGroup *group)
 {
 	GeanyKeyBinding *kb;
 
-	g_assert(group->plugin);
+	g_ptr_array_free(group->key_items, TRUE);
 
-	foreach_c_array(kb, group->keys, group->count)
+	if (group->plugin)
 	{
-		g_free(kb->name);
-		g_free(kb->label);
+		foreach_c_array(kb, group->plugin_keys, group->plugin_key_count)
+		{
+			g_free(kb->name);
+			g_free(kb->label);
+		}
+		g_free(group->plugin_keys);
+		g_ptr_array_remove_fast(keybinding_groups, group);
+		g_free(group);
 	}
-	g_free(group->keys);
-	g_ptr_array_remove_fast(keybinding_groups, group);
-	g_free(group);
 }
