@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: keyfile.c 5134 2010-08-11 14:16:42Z ntrel $
+ * $Id: keyfile.c 5471 2010-12-07 13:16:57Z ntrel $
  */
 
 /*
@@ -187,17 +187,17 @@ static void init_pref_groups(void)
 		"use_gtk_word_boundaries", TRUE);
 	stash_group_add_boolean(group, &editor_prefs.complete_snippets_whilst_editing,
 		"complete_snippets_whilst_editing", FALSE);
-	stash_group_add_boolean(group, &interface_prefs.show_symbol_list_expanders,
-		"show_symbol_list_expanders", TRUE);
-	stash_group_add_boolean(group, &interface_prefs.compiler_tab_autoscroll,
-		"compiler_tab_autoscroll", TRUE);
-	stash_group_add_boolean(group, &ui_prefs.allow_always_save,
-		"allow_always_save", FALSE);
 	stash_group_add_boolean(group, &file_prefs.use_safe_file_saving,
 		"use_safe_file_saving", FALSE);
+	stash_group_add_boolean(group, &file_prefs.gio_unsafe_save_backup,
+		"gio_unsafe_save_backup", FALSE);
 	/* for backwards-compatibility */
 	stash_group_add_integer(group, &editor_prefs.indentation->hard_tab_width,
 		"indent_hard_tab_width", 8);
+	stash_group_add_integer(group, (gint*)&search_prefs.find_selection_type,
+		"find_selection_type", GEANY_FIND_SEL_CURRENT_WORD);
+
+	/* Note: Interface-related hidden prefs are in ui_init_prefs() */
 }
 
 
@@ -270,7 +270,7 @@ static gchar *get_session_file_string(GeanyDocument *doc)
 	 * (g_path_skip_root() to skip C:\... on Windows) */
 	g_strdelimit((gchar*) utils_path_skip_root(locale_filename), ";", ':');
 
-	fname = g_strdup_printf("%d;%s;%d;%d;%d;%d;%d;%s;%d",
+	fname = g_strdup_printf("%d;%s;%d;%d;%d;%d;%d;%s;%d;%d",
 		sci_get_current_position(doc->editor->sci),
 		ft->name,
 		doc->readonly,
@@ -279,7 +279,8 @@ static gchar *get_session_file_string(GeanyDocument *doc)
 		doc->editor->auto_indent,
 		doc->editor->line_wrapping,
 		locale_filename,
-		doc->editor->line_breaking);
+		doc->editor->line_breaking,
+		doc->editor->indent_width);
 	g_free(locale_filename);
 	return fname;
 }
@@ -374,9 +375,9 @@ static void save_dialog_prefs(GKeyFile *config)
 	g_key_file_set_boolean(config, PACKAGE, "show_line_endings", editor_prefs.show_line_endings);
 	g_key_file_set_boolean(config, PACKAGE, "show_markers_margin", editor_prefs.show_markers_margin);
 	g_key_file_set_boolean(config, PACKAGE, "show_linenumber_margin", editor_prefs.show_linenumber_margin);
-	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_global_enabled);
-	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_global_type);
-	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_global_column);
+	g_key_file_set_boolean(config, PACKAGE, "long_line_enabled", editor_prefs.long_line_enabled);
+	g_key_file_set_integer(config, PACKAGE, "long_line_type", editor_prefs.long_line_type);
+	g_key_file_set_integer(config, PACKAGE, "long_line_column", editor_prefs.long_line_column);
 	g_key_file_set_string(config, PACKAGE, "long_line_color", editor_prefs.long_line_color);
 
 	/* editor */
@@ -406,6 +407,7 @@ static void save_dialog_prefs(GKeyFile *config)
 		g_key_file_set_string(config, PACKAGE, "pref_editor_default_open_encoding", encodings[file_prefs.default_open_encoding].charset);
 	g_key_file_set_integer(config, PACKAGE, "default_eol_character", file_prefs.default_eol_character);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_new_line", file_prefs.final_new_line);
+	g_key_file_set_boolean(config, PACKAGE, "pref_editor_ensure_convert_line_endings", file_prefs.ensure_convert_new_lines);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_replace_tabs", file_prefs.replace_tabs);
 	g_key_file_set_boolean(config, PACKAGE, "pref_editor_trail_space", file_prefs.strip_trailing_spaces);
 
@@ -448,7 +450,7 @@ static void save_dialog_prefs(GKeyFile *config)
 	/* VTE */
 #ifdef HAVE_VTE
 	g_key_file_set_boolean(config, "VTE", "load_vte", vte_info.load_vte);
-	if (vte_info.load_vte && vc != NULL)
+	if (vte_info.have_vte)
 	{
 		gchar *tmp_string;
 
@@ -663,15 +665,15 @@ static void load_dialog_prefs(GKeyFile *config)
 	interface_prefs.use_native_windows_dialogs = utils_get_setting_boolean(config, PACKAGE, "use_native_windows_dialogs", FALSE);
 
 	/* display, editor */
-	editor_prefs.long_line_global_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
-	editor_prefs.long_line_global_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
-	if (editor_prefs.long_line_global_type == 2) /* backward compatibility */
+	editor_prefs.long_line_enabled = utils_get_setting_boolean(config, PACKAGE, "long_line_enabled", TRUE);
+	editor_prefs.long_line_type = utils_get_setting_integer(config, PACKAGE, "long_line_type", 0);
+	if (editor_prefs.long_line_type == 2) /* backward compatibility */
 	{
-		editor_prefs.long_line_global_type = 0;
-		editor_prefs.long_line_global_enabled = FALSE;
+		editor_prefs.long_line_type = 0;
+		editor_prefs.long_line_enabled = FALSE;
 	}
 	editor_prefs.long_line_color = utils_get_setting_string(config, PACKAGE, "long_line_color", "#C2EBC2");
-	editor_prefs.long_line_global_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
+	editor_prefs.long_line_column = utils_get_setting_integer(config, PACKAGE, "long_line_column", 72);
 	editor_prefs.symbolcompletion_min_chars = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_min_chars", GEANY_MIN_SYMBOLLIST_CHARS);
 	editor_prefs.symbolcompletion_max_height = utils_get_setting_integer(config, PACKAGE, "symbolcompletion_max_height", GEANY_MAX_SYMBOLLIST_HEIGHT);
 	editor_prefs.line_wrapping = utils_get_setting_boolean(config, PACKAGE, "line_wrapping", FALSE); /* default is off for better performance */
@@ -724,6 +726,7 @@ static void load_dialog_prefs(GKeyFile *config)
 	}
 	file_prefs.default_eol_character = utils_get_setting_integer(config, PACKAGE, "default_eol_character", GEANY_DEFAULT_EOL_CHARACTER);
 	file_prefs.replace_tabs = utils_get_setting_boolean(config, PACKAGE, "pref_editor_replace_tabs", FALSE);
+	file_prefs.ensure_convert_new_lines = utils_get_setting_boolean(config, PACKAGE, "pref_editor_ensure_convert_line_endings", FALSE);
 	file_prefs.final_new_line = utils_get_setting_boolean(config, PACKAGE, "pref_editor_new_line", TRUE);
 	file_prefs.strip_trailing_spaces = utils_get_setting_boolean(config, PACKAGE, "pref_editor_trail_space", FALSE);
 
@@ -764,7 +767,7 @@ static void load_dialog_prefs(GKeyFile *config)
 		vc->font = utils_get_setting_string(config, "VTE", "font", "Monospace 10");
 		vc->scroll_on_key = utils_get_setting_boolean(config, "VTE", "scroll_on_key", TRUE);
 		vc->scroll_on_out = utils_get_setting_boolean(config, "VTE", "scroll_on_out", TRUE);
-		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", FALSE);
+		vc->enable_bash_keys = utils_get_setting_boolean(config, "VTE", "enable_bash_keys", TRUE);
 		vc->ignore_menu_bar_accel = utils_get_setting_boolean(config, "VTE", "ignore_menu_bar_accel", FALSE);
 		vc->follow_path = utils_get_setting_boolean(config, "VTE", "follow_path", FALSE);
 		vc->run_in_vte = utils_get_setting_boolean(config, "VTE", "run_in_vte", FALSE);
@@ -935,7 +938,7 @@ gboolean configuration_load(void)
 	gchar *configfile = g_strconcat(app->configdir, G_DIR_SEPARATOR_S, "geany.conf", NULL);
 	GKeyFile *config = g_key_file_new();
 
-	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	if (! g_file_test(configfile, G_FILE_TEST_IS_REGULAR))
 	{	/* config file does not (yet) exist, so try to load a global config file which may be */
 		/* created by distributors */
 		geany_debug("No user config file found, trying to use global configuration.");
@@ -984,7 +987,7 @@ static gboolean open_session_file(gchar **tmp, guint len)
 	if (len > 8)
 		line_breaking = atoi(tmp[8]);
 
-	if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	if (g_file_test(locale_filename, G_FILE_TEST_IS_REGULAR))
 	{
 		GeanyFiletype *ft = filetypes_lookup_by_name(ft_name);
 		GeanyDocument *doc = document_open_file_full(
@@ -994,7 +997,11 @@ static gboolean open_session_file(gchar **tmp, guint len)
 
 		if (doc)
 		{
-			editor_set_indent_type(doc->editor, indent_type);
+			gint indent_width = doc->editor->indent_width;
+
+			if (len > 9)
+				indent_width = atoi(tmp[9]);
+			editor_set_indent(doc->editor, indent_type, indent_width);
 			editor_set_line_wrapping(doc->editor, line_wrapping);
 			doc->editor->line_breaking = line_breaking;
 			doc->editor->auto_indent = auto_indent;
