@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: main.c 4957 2010-05-29 18:35:51Z eht16 $
+ * $Id: main.c 5472 2010-12-07 13:25:11Z ntrel $
  */
 
 /**
@@ -95,6 +95,15 @@ GeanyStatus	 main_status;
 CommandLineOptions cl_options;	/* fields initialised in parse_command_line_options */
 
 
+static const gchar geany_lib_versions[] = "GTK %u.%u.%u, GLib %u.%u.%u"
+#ifdef HAVE_GIO
+	", GIO"
+#endif
+#ifdef USE_INCLUDED_REGEX
+	", built-in regex"
+#endif
+	;
+
 static gboolean want_plugins;
 
 /* command-line options */
@@ -183,7 +192,6 @@ static void apply_settings(void)
 		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "menu_show_sidebar1")), FALSE);
 		ignore_callback = FALSE;
 	}
-	ui_sidebar_show_hide();
 
 	toolbar_apply_settings();
 	toolbar_update_ui();
@@ -205,11 +213,12 @@ static void apply_settings(void)
 	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(main_widgets.notebook), interface_prefs.show_notebook_tabs);
 
 #ifdef HAVE_VTE
-	if (! vte_info.load_vte)
-		gtk_widget_hide(ui_lookup_widget(main_widgets.window, "send_selection_to_vte1"));
-#else
-	gtk_widget_hide(ui_lookup_widget(main_widgets.window, "send_selection_to_vte1"));
+	if (! vte_info.have_vte)
 #endif
+	{
+		gtk_widget_set_sensitive(
+			ui_lookup_widget(main_widgets.window, "send_selection_to_vte1"), FALSE);
+	}
 
 	if (interface_prefs.sidebar_pos != GTK_POS_LEFT)
 		ui_swap_sidebar_pos();
@@ -252,6 +261,8 @@ static void main_init(void)
 	main_widgets.notebook = ui_lookup_widget(main_widgets.window, "notebook1");
 	main_widgets.editor_menu = create_edit_menu1();
 	main_widgets.tools_menu = ui_lookup_widget(main_widgets.window, "tools1_menu");
+	main_widgets.message_window_notebook = ui_lookup_widget(main_widgets.window, "notebook_info");
+	main_widgets.project_menu = ui_lookup_widget(main_widgets.window, "menu_project1_menu");
 
 	ui_widgets.toolbar_menu = create_toolbar_popup_menu1();
 	ui_init();
@@ -380,8 +391,8 @@ static void setup_paths(void)
 
 	g_free(install_dir);
 #else
-	data_dir = g_strconcat(GEANY_DATADIR, "/geany/", NULL); /* e.g. /usr/share/geany */
-	doc_dir = g_strconcat(GEANY_DOCDIR, "/html/", NULL);
+	data_dir = g_strconcat(GEANY_DATADIR, "/geany", NULL); /* e.g. /usr/share/geany */
+	doc_dir = g_strconcat(GEANY_DOCDIR, "/html", NULL);
 #endif
 
 	/* convert path names to locale encoding */
@@ -400,8 +411,8 @@ static void setup_paths(void)
  *  This is because the main window is realized (i.e. actually drawn on the screen) at the
  *  end of the startup process.
  *
- *  @note Maybe you want to use the @ref geany-startup-complete signal to get notified about
- *        the completed startup process.
+ *  @note Maybe you want to use the @link pluginsignals.c @c "geany-startup-complete" signal @endlink
+ *        to get notified about the completed startup process.
  *
  *  @return @c TRUE if the Geany main window has been realized or @c FALSE otherwise.
  *
@@ -511,12 +522,13 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 
 	if (show_version)
 	{
-		printf(PACKAGE " %s ", main_get_version_string());
-		printf(_("(built on %s with GTK %d.%d.%d, GLib %d.%d.%d)"),
-				__DATE__, GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
-				GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
-		printf("\n");
-
+		printf(PACKAGE " %s (", main_get_version_string());
+		/* note for translators: library versions are printed after this */
+		printf(_("built on %s with "), __DATE__);
+		printf(geany_lib_versions,
+			GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+			GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+		printf(")\n");
 		exit(0);
 	}
 
@@ -585,12 +597,6 @@ static void parse_command_line_options(gint *argc, gchar ***argv)
 }
 
 
-#ifdef G_OS_WIN32
-# define DIR_SEP "\\" /* on Windows we need an additional dir separator */
-#else
-# define DIR_SEP ""
-#endif
-
 static gint create_config_dir(void)
 {
 	gint saved_errno = 0;
@@ -664,7 +670,7 @@ static gint create_config_dir(void)
 			gchar *text = g_strconcat(
 "Copy files from ", app->datadir, " to this directory to overwrite "
 "them. To use the defaults, just delete the file in this directory.\nFor more information read "
-"the documentation (in ", app->docdir, DIR_SEP "index.html or visit " GEANY_HOMEPAGE ").", NULL);
+"the documentation (in ", app->docdir, G_DIR_SEPARATOR_S "index.html or visit " GEANY_HOMEPAGE ").", NULL);
 			utils_write_file(filedefs_readme, text);
 			g_free(text);
 		}
@@ -685,7 +691,7 @@ static gint create_config_dir(void)
 		{
 			gchar *text = g_strconcat(
 "There are several template files in this directory. For these templates you can use wildcards.\n\
-For more information read the documentation (in ", app->docdir, DIR_SEP "index.html or visit " GEANY_HOMEPAGE ").",
+For more information read the documentation (in ", app->docdir, G_DIR_SEPARATOR_S "index.html or visit " GEANY_HOMEPAGE ").",
 					NULL);
 			utils_write_file(templates_readme, text);
 			g_free(text);
@@ -758,7 +764,7 @@ gboolean main_handle_filename(const gchar *locale_filename)
 	if (column >= 0)
 		cl_options.goto_column = column;
 
-	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR | G_FILE_TEST_IS_SYMLINK))
+	if (g_file_test(filename, G_FILE_TEST_IS_REGULAR))
 	{
 		doc = document_open_file(filename, FALSE, NULL, NULL);
 		/* add recent file manually if opening_session_files is set */
@@ -793,6 +799,13 @@ static gboolean open_cl_files(gint argc, gchar **argv)
 	for (i = 1; i < argc; i++)
 	{
 		gchar *filename = main_get_argv_filename(argv[i]);
+
+		if (g_file_test(filename, G_FILE_TEST_IS_DIR))
+		{
+			g_free(filename);
+			continue;
+		}
+
 #ifdef G_OS_WIN32
 		/* It seems argv elements are encoded in CP1252 on a German Windows */
 		setptr(filename, g_locale_to_utf8(filename, -1, NULL, NULL, NULL));
@@ -982,11 +995,14 @@ gint main(gint argc, gchar **argv)
 #endif
 
 	locale = get_locale();
-	geany_debug("Geany %s, GTK+ %u.%u.%u, GLib %u.%u.%u (%s)",
+	geany_debug("Geany %s, %s",
 		main_get_version_string(),
-		gtk_major_version, gtk_minor_version, gtk_micro_version,
-		glib_major_version, glib_minor_version, glib_micro_version,
 		locale);
+	geany_debug(geany_lib_versions,
+		gtk_major_version, gtk_minor_version, gtk_micro_version,
+		glib_major_version, glib_minor_version, glib_micro_version);
+	geany_debug("System data dir: %s", app->datadir);
+	geany_debug("User config dir: %s", app->configdir);
 
 	/* create the object so Geany signals can be connected in init() functions */
 	geany_object = geany_object_new();
@@ -997,8 +1013,9 @@ gint main(gint argc, gchar **argv)
 	encodings_init();
 	editor_init();
 
+	/* init stash groups before loading keyfile */
 	configuration_init();
-	/* init stash code before loading keyfile */
+	ui_init_prefs();
 	search_init();
 	project_init();
 #ifdef HAVE_PLUGINS
@@ -1055,6 +1072,8 @@ gint main(gint argc, gchar **argv)
 		plugins_load_active();
 #endif
 
+	ui_sidebar_show_hide();
+
 	/* set the active sidebar page after plugins have been loaded */
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(main_widgets.sidebar_notebook), ui_prefs.sidebar_page);
 
@@ -1161,6 +1180,7 @@ void main_quit()
 	sidebar_finalize();
 	configuration_finalize();
 	filetypes_free_types();
+	ui_finalize();
 	log_finalize();
 
 	tm_workspace_free(TM_WORK_OBJECT(app->tm_workspace));
