@@ -1,8 +1,8 @@
 /*
  *      msgwindow.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,14 +18,16 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.c 4010 2009-07-21 22:12:13Z eht16 $
+ * $Id: msgwindow.c 5237 2010-09-16 15:31:23Z ntrel $
  */
 
-/*
+/**
+ * @file msgwindow.h
  * Message window functions (status, compiler, messages windows).
  * Also compiler error message parsing and grep file and line parsing.
- */
-
+ *
+ * @see GeanyMainWidgets::message_window_notebook to append a new notebook page.
+ **/
 
 #include "geany.h"
 
@@ -75,13 +77,33 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 static void on_scribble_populate(GtkTextView *textview, GtkMenu *arg1, gpointer user_data);
 
 
-void msgwin_init()
+void msgwin_show_hide_tabs(void)
+{
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_status), interface_prefs.msgwin_status_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_compiler), interface_prefs.msgwin_compiler_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.tree_msg), interface_prefs.msgwin_messages_visible);
+	ui_widget_show_hide(gtk_widget_get_parent(msgwindow.scribble), interface_prefs.msgwin_scribble_visible);
+}
+
+
+/** Sets the Messages path for opening any parsed filenames without absolute path
+ * from message lines.
+ * @param messages_dir The directory. **/
+void msgwin_set_messages_dir(const gchar *messages_dir)
+{
+	g_free(msgwindow.messages_dir);
+	msgwindow.messages_dir = g_strdup(messages_dir);
+}
+
+
+void msgwin_init(void)
 {
 	msgwindow.notebook = ui_lookup_widget(main_widgets.window, "notebook_info");
 	msgwindow.tree_status = ui_lookup_widget(main_widgets.window, "treeview3");
 	msgwindow.tree_msg = ui_lookup_widget(main_widgets.window, "treeview4");
 	msgwindow.tree_compiler = ui_lookup_widget(main_widgets.window, "treeview5");
-	msgwindow.find_in_files_dir = NULL;
+	msgwindow.scribble = ui_lookup_widget(main_widgets.window, "textview_scribble");
+	msgwindow.messages_dir = NULL;
 
 	prepare_status_tree_view();
 	prepare_msg_tree_view();
@@ -90,32 +112,34 @@ void msgwin_init()
 	msgwindow.popup_msg_menu = create_message_popup_menu(MSG_MESSAGE);
 	msgwindow.popup_compiler_menu = create_message_popup_menu(MSG_COMPILER);
 
-	ui_widget_modify_font_from_string(
-		ui_lookup_widget(main_widgets.window, "textview_scribble"), interface_prefs.msgwin_font);
-
-	g_signal_connect(ui_lookup_widget(main_widgets.window, "textview_scribble"),
-		"populate-popup", G_CALLBACK(on_scribble_populate), NULL);
+	ui_widget_modify_font_from_string(msgwindow.scribble, interface_prefs.msgwin_font);
+	g_signal_connect(msgwindow.scribble, "populate-popup", G_CALLBACK(on_scribble_populate), NULL);
 }
 
 
-void msgwin_finalize()
+void msgwin_finalize(void)
 {
-	g_free(msgwindow.find_in_files_dir);
+	g_free(msgwindow.messages_dir);
 }
 
 
 static gboolean on_msgwin_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer data)
 {
-	if (event->keyval == GDK_Return ||
-		event->keyval == GDK_ISO_Enter ||
-		event->keyval == GDK_KP_Enter ||
-		event->keyval == GDK_space)
+	if (ui_is_keyval_enter_or_return(event->keyval) || event->keyval == GDK_space)
 	{
-		GdkEventButton button_event;
-
-		button_event.button = 1;
-		button_event.time = event->time;
-		on_msgwin_button_press_event(NULL, &button_event, data);
+		switch (GPOINTER_TO_INT(data))
+		{
+			case MSG_COMPILER:
+			{	/* single click in the compiler treeview */
+				msgwin_goto_compiler_file_line(event->keyval);
+				break;
+			}
+			case MSG_MESSAGE:
+			{	/* single click in the message treeview (results of 'Find usage') */
+				msgwin_goto_messages_file_line(event->keyval);
+				break;
+			}
+		}
 	}
 	return FALSE;
 }
@@ -240,13 +264,14 @@ static const GdkColor *get_color(gint msg_color)
  **/
 void msgwin_compiler_add(gint msg_color, const gchar *format, ...)
 {
-	gchar string[512];
+	gchar *string;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 	msgwin_compiler_add_string(msg_color, string);
+	g_free(string);
 }
 
 
@@ -255,11 +280,17 @@ void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 	GtkTreeIter iter;
 	GtkTreePath *path;
 	const GdkColor *color = get_color(msg_color);
+	gchar *utf8_msg;
+
+	if (! g_utf8_validate(msg, -1, NULL))
+		utf8_msg = utils_get_utf8_from_locale(msg);
+	else
+		utf8_msg = (gchar *) msg;
 
 	gtk_list_store_append(msgwindow.store_compiler, &iter);
-	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, msg, -1);
+	gtk_list_store_set(msgwindow.store_compiler, &iter, 0, color, 1, utf8_msg, -1);
 
-	if (ui_prefs.msgwindow_visible)
+	if (ui_prefs.msgwindow_visible && interface_prefs.compiler_tab_autoscroll)
 	{
 		path = gtk_tree_model_get_path(
 			gtk_tree_view_get_model(GTK_TREE_VIEW(msgwindow.tree_compiler)), &iter);
@@ -267,9 +298,12 @@ void msgwin_compiler_add_string(gint msg_color, const gchar *msg)
 		gtk_tree_path_free(path);
 	}
 
-	/* calling build_menu_update for every build message would be overkill */
-	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_next_error, TRUE);
-	gtk_widget_set_sensitive(build_get_menu_items(-1)->item_previous_error, TRUE);
+	/* calling build_menu_update for every build message would be overkill, TODO really should call it once when all done */
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->menu_item[GBG_FIXED][GBF_NEXT_ERROR], TRUE);
+	gtk_widget_set_sensitive(build_get_menu_items(-1)->menu_item[GBG_FIXED][GBF_PREV_ERROR], TRUE);
+
+	if (utf8_msg != msg)
+		g_free(utf8_msg);
 }
 
 
@@ -302,14 +336,15 @@ void msgwin_show_hide(gboolean show)
  **/
 void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
 {
-	gchar string[512];
+	gchar *string;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 
 	msgwin_msg_add_string(msg_color, line, doc, string);
+	g_free(string);
 }
 
 
@@ -320,8 +355,10 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 	const GdkColor *color = get_color(msg_color);
 	gchar *tmp;
 	gsize len;
+	gchar *utf8_msg;
 
-	if (! ui_prefs.msgwindow_visible) msgwin_show_hide(TRUE);
+	if (! ui_prefs.msgwindow_visible)
+		msgwin_show_hide(TRUE);
 
 	/* work around a strange problem when adding very long lines(greater than 4000 bytes)
 	 * cut the string to a maximum of 1024 bytes and discard the rest */
@@ -332,15 +369,22 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 	else
 		tmp = g_strdup(string);
 
+	if (! g_utf8_validate(tmp, -1, NULL))
+		utf8_msg = utils_get_utf8_from_locale(tmp);
+	else
+		utf8_msg = tmp;
+
 	gtk_list_store_append(msgwindow.store_msg, &iter);
-	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, tmp, -1);
+	gtk_list_store_set(msgwindow.store_msg, &iter, 0, line, 1, doc, 2, color, 3, utf8_msg, -1);
 
 	g_free(tmp);
+	if (utf8_msg != tmp)
+		g_free(utf8_msg);
 }
 
 
 /**
- *  Log a status message *without* setting the status bar.
+ *  Logs a status message *without* setting the status bar.
  *  (Use ui_set_statusbar() to display text on the statusbar)
  *
  *  @param format @c printf()-style format string.
@@ -349,18 +393,19 @@ void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const
 void msgwin_status_add(const gchar *format, ...)
 {
 	GtkTreeIter iter;
-	gchar string[512];
+	gchar *string;
 	gchar *statusmsg, *time_str;
 	va_list args;
 
 	va_start(args, format);
-	g_vsnprintf(string, 512, format, args);
+	string = g_strdup_vprintf(format, args);
 	va_end(args);
 
 	/* add a timestamp to status messages */
 	time_str = utils_get_current_time_string();
 	statusmsg = g_strconcat(time_str, ": ", string, NULL);
 	g_free(time_str);
+	g_free(string);
 
 	/* add message to Status window */
 	gtk_list_store_append(msgwindow.store_status, &iter);
@@ -496,7 +541,7 @@ on_hide_message_window                 (GtkMenuItem     *menuitem,
 
 static GtkWidget *create_message_popup_menu(gint type)
 {
-	GtkWidget *message_popup_menu, *clear, *copy, *image;
+	GtkWidget *message_popup_menu, *clear, *copy, *copy_all, *image;
 
 	message_popup_menu = gtk_menu_new();
 
@@ -506,19 +551,22 @@ static GtkWidget *create_message_popup_menu(gint type)
 	g_signal_connect(clear, "activate",
 		G_CALLBACK(on_message_treeview_clear_activate), GINT_TO_POINTER(type));
 
-	copy = gtk_image_menu_item_new_from_stock("gtk-copy", NULL);
-	gtk_widget_show(copy);
-	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
-	g_signal_connect(copy, "activate",
-		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
-
-	copy = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
+	copy = gtk_image_menu_item_new_with_mnemonic(_("C_opy"));
 	gtk_widget_show(copy);
 	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy);
 	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
 	gtk_widget_show(image);
 	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy), image);
 	g_signal_connect(copy, "activate",
+		G_CALLBACK(on_compiler_treeview_copy_activate), GINT_TO_POINTER(type));
+
+	copy_all = gtk_image_menu_item_new_with_mnemonic(_("Copy _All"));
+	gtk_widget_show(copy_all);
+	gtk_container_add(GTK_CONTAINER(message_popup_menu), copy_all);
+	image = gtk_image_new_from_stock("gtk-copy", GTK_ICON_SIZE_MENU);
+	gtk_widget_show(image);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(copy_all), image);
+	g_signal_connect(copy_all, "activate",
 		G_CALLBACK(on_compiler_treeview_copy_all_activate), GINT_TO_POINTER(type));
 
 	msgwin_menu_add_common_items(GTK_MENU(message_popup_menu));
@@ -575,13 +623,72 @@ find_prev_build_dir(GtkTreePath *cur, GtkTreeModel *model, gchar **prefix)
 }
 
 
-gboolean msgwin_goto_compiler_file_line()
+static gboolean goto_compiler_file_line(const gchar *filename, gint line, guint keyval)
+{
+	if (!filename || line <= -1)
+		return FALSE;
+
+	/* If the path doesn't exist, try the current document.
+	 * This happens when we receive build messages in the wrong order - after the
+	 * 'Leaving directory' messages */
+	if (!g_file_test(filename, G_FILE_TEST_EXISTS))
+	{
+		gchar *cur_dir = utils_get_current_file_dir_utf8();
+		gchar *name;
+
+		if (cur_dir)
+		{
+			/* we let the user know we couldn't find the parsed filename from the message window */
+			setptr(cur_dir, utils_get_locale_from_utf8(cur_dir));
+			name = g_path_get_basename(filename);
+			setptr(name, g_build_path(G_DIR_SEPARATOR_S, cur_dir, name, NULL));
+			g_free(cur_dir);
+
+			if (g_file_test(name, G_FILE_TEST_EXISTS))
+			{
+				ui_set_statusbar(FALSE, _("Could not find file '%s' - trying the current document path."),
+					filename);
+				filename = name;
+			}
+			else
+				g_free(name);
+		}
+	}
+
+	{
+		gchar *utf8_filename = utils_get_utf8_from_locale(filename);
+		GeanyDocument *doc = document_find_by_filename(utf8_filename);
+		GeanyDocument *old_doc = document_get_current();
+
+		g_free(utf8_filename);
+
+		if (doc == NULL)	/* file not already open */
+			doc = document_open_file(filename, FALSE, NULL, NULL);
+
+		if (doc != NULL)
+		{
+			gboolean ret;
+
+			if (! doc->changed && editor_prefs.use_indicators)	/* if modified, line may be wrong */
+				editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
+
+			ret = navqueue_goto_line(old_doc, doc, line);
+			if (ret && ui_is_keyval_enter_or_return(keyval))
+				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+
+			return ret;
+		}
+	}
+	return FALSE;
+}
+
+
+gboolean msgwin_goto_compiler_file_line(guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	gchar *string;
-	gboolean ret = FALSE;
 	GdkColor *color;
 
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(msgwindow.tree_compiler));
@@ -603,41 +710,21 @@ gboolean msgwin_goto_compiler_file_line()
 			gint line;
 			gchar *filename, *dir;
 			GtkTreePath *path;
-			/* save the beginning of the string to use when free'ing it after it was stripped */
-			gchar *string_start = string;
+			gboolean ret;
 
 			path = gtk_tree_model_get_path(model, &iter);
 			find_prev_build_dir(path, model, &dir);
 			gtk_tree_path_free(path);
 			msgwin_parse_compiler_error_line(string, dir, &filename, &line);
+			g_free(string);
+			g_free(dir);
 
-			if (dir != NULL)
-				g_free(dir);
-
-			if (filename != NULL && line > -1)
-			{
-				gchar *utf8_filename = utils_get_utf8_from_locale(filename);
-				GeanyDocument *doc = document_find_by_filename(utf8_filename);
-				GeanyDocument *old_doc = document_get_current();
-
-				g_free(utf8_filename);
-
-				if (doc == NULL)	/* file not already open */
-					doc = document_open_file(filename, FALSE, NULL, NULL);
-
-				if (doc != NULL)
-				{
-					if (! doc->changed)	/* if modified, line may be wrong */
-						editor_indicator_set_on_line(doc->editor, GEANY_INDICATOR_ERROR, line - 1);
-
-					ret = navqueue_goto_line(old_doc, doc, line);
-				}
-			}
+			ret = goto_compiler_file_line(filename, line, keyval);
 			g_free(filename);
-			g_free(string_start);
+			return ret;
 		}
 	}
-	return ret;
+	return FALSE;
 }
 
 
@@ -909,7 +996,7 @@ void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 }
 
 
-gboolean msgwin_goto_messages_file_line()
+gboolean msgwin_goto_messages_file_line(guint keyval)
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
@@ -929,17 +1016,24 @@ gboolean msgwin_goto_messages_file_line()
 		if (line >= 0 && DOC_VALID(doc))
 		{
 			ret = navqueue_goto_line(old_doc, doc, line);
+			if (ret && ui_is_keyval_enter_or_return(keyval))
+				gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
 		}
 		else if (line < 0 && string != NULL)
 		{
 			gchar *filename;
+
 			msgwin_parse_grep_line(string, &filename, &line);
 			if (filename != NULL && line > -1)
 			{
 				/* use document_open_file to find an already open file, or open it in place */
 				doc = document_open_file(filename, FALSE, NULL, NULL);
 				if (doc != NULL)
+				{
 					ret = navqueue_goto_line(old_doc, doc, line);
+					if (ret && ui_is_keyval_enter_or_return(keyval))
+						gtk_widget_grab_focus(GTK_WIDGET(doc->editor->sci));
+				}
 			}
 			g_free(filename);
 		}
@@ -961,7 +1055,7 @@ static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *
 	*filename = NULL;
 	*line = -1;
 
-	if (string == NULL || msgwindow.find_in_files_dir == NULL)
+	if (string == NULL)
 		return;
 
 	/* conflict:3:conflicting types for `foo' */
@@ -972,7 +1066,10 @@ static void msgwin_parse_grep_line(const gchar *string, gchar **filename, gint *
 	data.file_idx = 0;
 
 	parse_file_line(&data, filename, line);
-	make_absolute(filename, msgwindow.find_in_files_dir);
+
+	/* FIF dir should be set, but a plugin might not have set it */
+	if (msgwindow.messages_dir != NULL)
+		make_absolute(filename, msgwindow.messages_dir);
 }
 
 
@@ -987,12 +1084,12 @@ static gboolean on_msgwin_button_press_event(GtkWidget *widget, GdkEventButton *
 		{
 			case MSG_COMPILER:
 			{	/* single click in the compiler treeview */
-				msgwin_goto_compiler_file_line();
+				msgwin_goto_compiler_file_line(0);
 				break;
 			}
 			case MSG_MESSAGE:
 			{	/* single click in the message treeview (results of 'Find usage') */
-				msgwin_goto_messages_file_line();
+				msgwin_goto_messages_file_line(0);
 				break;
 			}
 		}
@@ -1042,7 +1139,7 @@ void msgwin_switch_tab(gint tabnum, gboolean show)
 
 	switch (tabnum)
 	{
-		case MSG_SCRATCH: widget = ui_lookup_widget(main_widgets.window, "textview_scribble"); break;
+		case MSG_SCRATCH: widget = msgwindow.scribble; break;
 		case MSG_COMPILER: widget = msgwindow.tree_compiler; break;
 		case MSG_STATUS: widget = msgwindow.tree_status; break;
 		case MSG_MESSAGE: widget = msgwindow.tree_msg; break;
@@ -1061,6 +1158,7 @@ void msgwin_switch_tab(gint tabnum, gboolean show)
 		gtk_widget_grab_focus(widget);
 }
 
+
 /**
  *  Removes all messages from a tab specified by @a tabnum in the messages window.
  *
