@@ -1,8 +1,8 @@
 /*
  *      msgwindow.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2005-2008 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2008 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2005-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,16 +18,9 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: msgwindow.h 2705 2008-06-18 17:03:08Z ntrel $
+ * $Id: msgwindow.h 5237 2010-09-16 15:31:23Z ntrel $
  */
 
-/**
- *  @file msgwindow.h
- *  Message window functions (status, compiler, messages windows).
- *  Also compiler error message parsing and grep file and line parsing.
- **/
-
-
 #ifndef GEANY_MSGWINDOW_H
 #define GEANY_MSGWINDOW_H 1
 
@@ -64,11 +57,12 @@ typedef struct
 	GtkWidget		*tree_compiler;
 	GtkWidget		*tree_status;
 	GtkWidget		*tree_msg;
+	GtkWidget		*scribble;
 	GtkWidget		*popup_status_menu;
 	GtkWidget		*popup_msg_menu;
 	GtkWidget		*popup_compiler_menu;
 	GtkWidget		*notebook;
-	gchar			*find_in_files_dir;
+	gchar			*messages_dir;
 } MessageWindow;
 
 extern MessageWindow msgwindow;
@@ -84,25 +78,29 @@ void msgwin_switch_tab(gint tabnum, gboolean show);
 
 void msgwin_clear_tab(gint tabnum);
 
-void msgwin_msg_add_fmt(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
+void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *format, ...)
 			G_GNUC_PRINTF (4, 5);
 
-void msgwin_msg_add(gint msg_color, gint line, GeanyDocument *doc, const gchar *string);
+void msgwin_msg_add_string(gint msg_color, gint line, GeanyDocument *doc, const gchar *string);
 
-void msgwin_compiler_add_fmt(gint msg_color, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
+void msgwin_compiler_add(gint msg_color, const gchar *format, ...) G_GNUC_PRINTF (2, 3);
 
-void msgwin_compiler_add(gint msg_color, const gchar *msg);
+void msgwin_compiler_add_string(gint msg_color, const gchar *msg);
 
 void msgwin_status_add(const gchar *format, ...) G_GNUC_PRINTF (1, 2);
 
+void msgwin_show_hide_tabs(void);
+
+void msgwin_set_messages_dir(const gchar *messages_dir);
+
 
 void msgwin_menu_add_common_items(GtkMenu *menu);
 
-gboolean msgwin_goto_compiler_file_line(void);
+gboolean msgwin_goto_compiler_file_line(guint keyval);
 
 void msgwin_parse_compiler_error_line(const gchar *string, const gchar *dir,
 									  gchar **filename, gint *line);
 
-gboolean msgwin_goto_messages_file_line(void);
+gboolean msgwin_goto_messages_file_line(guint keyval);
 
 #endif
