@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: plugins.c 5132 2010-08-11 12:50:38Z ntrel $
+ * $Id: plugins.c 5507 2011-01-04 12:41:22Z ntrel $
  */
 
 /* Code to manage, load and unload plugins. */
@@ -122,7 +122,11 @@ static EditorFuncs editor_funcs = {
 	&editor_get_eol_char_name,
 	&editor_get_eol_char_len,
 	&editor_get_eol_char,
-	&editor_insert_text_block
+	&editor_insert_text_block,
+	&editor_get_eol_char_mode,
+	&editor_goto_pos,
+	&editor_find_snippet,
+	&editor_insert_snippet
 };
 
 static ScintillaFuncs scintilla_funcs = {
@@ -185,7 +189,8 @@ static SciFuncs sci_funcs = {
 	&sci_goto_line,
 	&sci_find_text,
 	&sci_set_line_indentation,
-	&sci_get_line_indentation
+	&sci_get_line_indentation,
+	&sci_get_lexer
 };
 
 static TemplateFuncs template_funcs = {
@@ -213,7 +218,8 @@ static UtilsFuncs utils_funcs = {
 	&utils_str_middle_truncate,
 	&utils_str_remove_chars,
 	&utils_get_file_list_full,
-	&utils_copy_environment
+	&utils_copy_environment,
+	&utils_find_open_xml_tag
 };
 
 static UIUtilsFuncs uiutils_funcs = {
@@ -233,14 +239,16 @@ static UIUtilsFuncs uiutils_funcs = {
 	&ui_menu_add_document_items,
 	&ui_widget_modify_font_from_string,
 	&ui_is_keyval_enter_or_return,
-	&ui_get_gtk_settings_integer
+	&ui_get_gtk_settings_integer,
+	&ui_combo_box_add_to_history
 };
 
 static DialogFuncs dialog_funcs = {
 	&dialogs_show_question,
 	&dialogs_show_msgbox,
 	&dialogs_show_save_as,
-	&dialogs_show_input_numeric
+	&dialogs_show_input_numeric,
+	&dialogs_show_input
 };
 
 /* Macro to prevent confusing macro being generated in geanyfunctions.h */
@@ -256,7 +264,8 @@ static MsgWinFuncs msgwin_funcs = {
 	&msgwin_compiler_add,
 	&msgwin_msg_add,
 	&msgwin_clear_tab,
-	&msgwin_switch_tab
+	&msgwin_switch_tab,
+	&msgwin_set_messages_dir
 };
 
 static EncodingFuncs encoding_funcs = {
@@ -286,13 +295,17 @@ static SearchFuncs search_funcs = {
 
 static HighlightingFuncs highlighting_funcs = {
 	&highlighting_get_style,
-	&highlighting_set_styles
+	&highlighting_set_styles,
+	&highlighting_is_string_style,
+	&highlighting_is_comment_style,
+	&highlighting_is_code_style
 };
 
 static FiletypeFuncs filetype_funcs = {
 	&filetypes_detect_from_file,
 	&filetypes_lookup_by_name,
-	&filetypes_index
+	&filetypes_index,
+	&filetypes_get_display_name
 };
 
 static NavQueueFuncs navqueue_funcs = {
@@ -378,7 +391,8 @@ geany_data_init(void)
 		&build_info,
 		filetypes_by_title
 	};
-	memcpy(&geany_data, &gd, sizeof(GeanyData));
+
+	geany_data = gd;
 }
 
 
@@ -811,6 +825,7 @@ load_plugins_from_path(const gchar *path)
 {
 	GSList *list, *item;
 	gchar *fname, *tmp;
+	gint count = 0;
 
 	list = utils_get_file_list(path, NULL, NULL);
 
@@ -821,12 +836,16 @@ load_plugins_from_path(const gchar *path)
 			continue;
 
 		fname = g_strconcat(path, G_DIR_SEPARATOR_S, item->data, NULL);
-		plugin_new(fname, FALSE, TRUE);
+		if (plugin_new(fname, FALSE, TRUE))
+			count++;
 		g_free(fname);
 	}
 
 	g_slist_foreach(list, (GFunc) g_free, NULL);
 	g_slist_free(list);
+
+	if (count)
+		geany_debug("Found %d plugin(s) in '%s'.", count, path);
 }
 
 
