@@ -1,8 +1,8 @@
 /*
  *      pluginutils.h - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
- *      Copyright 2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2009-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2009-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -19,21 +19,34 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: pluginutils.h 4041 2009-07-29 17:40:20Z ntrel $
+ * $Id: pluginutils.h 4850 2010-04-22 12:03:37Z ntrel $
  */
 
 
 #ifndef PLUGINUTILS_H
 #define PLUGINUTILS_H
 
-#include "plugindata.h"
+#ifdef HAVE_PLUGINS
 
-void plugin_add_toolbar_item(GeanyPlugin *plugin, GtkToolItem *item);
+#include "keybindings.h"	/* GeanyKeyGroupCallback */
 
-void plugin_module_make_resident(GeanyPlugin *plugin);
 
-void plugin_signal_connect(GeanyPlugin *plugin,
-		GObject *object, gchar *signal_name, gboolean after,
+/* avoid including plugindata.h otherwise this redefines the GEANY() macro */
+struct GeanyPlugin;
+
+
+void plugin_add_toolbar_item(struct GeanyPlugin *plugin, GtkToolItem *item);
+
+void plugin_module_make_resident(struct GeanyPlugin *plugin);
+
+void plugin_signal_connect(struct GeanyPlugin *plugin,
+		GObject *object, const gchar *signal_name, gboolean after,
 		GCallback callback, gpointer user_data);
 
+struct GeanyKeyGroup *plugin_set_key_group(struct GeanyPlugin *plugin,
+		const gchar *section_name, gsize count, GeanyKeyGroupCallback callback);
+
+void plugin_show_configure(struct GeanyPlugin *plugin);
+
+#endif /* HAVE_PLUGINS */
 #endif /* PLUGINUTILS_H */
