@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: prefs.c 4929 2010-05-16 18:22:24Z eht16 $
+ * $Id: prefs.c 5432 2010-11-23 17:36:29Z ntrel $
  */
 
 /*
@@ -28,7 +28,7 @@
 
 /*
  * Preferences dialog support functions.
- * New prefs should use Stash code in keyfile.c - init_pref_groups().
+ * New 'simple' prefs should use Stash code in keyfile.c - init_pref_groups().
  */
 
 #include <stdlib.h>
@@ -69,12 +69,10 @@
 #endif
 
 
-
 GeanyPrefs prefs;
 GeanyToolPrefs tool_prefs;
 
 
-static gchar *dialog_key_name;
 static GtkTreeIter g_iter;
 static GtkTreeStore *store = NULL;
 static GtkTreeView *tree = NULL;
@@ -82,10 +80,10 @@ static GtkWidget *dialog_label;
 static gboolean edited = FALSE;
 
 static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path, gchar *new_text, gpointer user_data);
-static gboolean kb_keytype_dialog_response_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
-static void kb_dialog_response_cb(GtkWidget *dialog, gint response, gpointer user_data);
+static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data);
+static void kb_grab_key_dialog_response_cb(GtkWidget *dialog, gint response, gpointer user_data);
 static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
-		guint key, GdkModifierType mods, const gchar *action);
+		guint key, GdkModifierType mods, const gchar *shortcut);
 static void on_toolbar_show_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_show_notebook_tabs_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 static void on_enable_plugins_toggled(GtkToggleButton *togglebutton, gpointer user_data);
@@ -178,11 +176,8 @@ static void kb_tree_view_change_button_clicked_cb(GtkWidget *button, gpointer da
 			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), dialog_label);
 
 			g_signal_connect(dialog, "key-press-event",
-								G_CALLBACK(kb_keytype_dialog_response_cb), NULL);
-			g_signal_connect(dialog, "response", G_CALLBACK(kb_dialog_response_cb), NULL);
-
-			/* copy name to global variable to hold it, will be freed in on_dialog_response() */
-			dialog_key_name = g_strdup(name);
+								G_CALLBACK(kb_grab_key_dialog_key_press_cb), NULL);
+			g_signal_connect(dialog, "response", G_CALLBACK(kb_grab_key_dialog_response_cb), NULL);
 
 			gtk_widget_show_all(dialog);
 			g_free(str);
@@ -283,7 +278,7 @@ static void kb_init_tree(void)
 	column = gtk_tree_view_column_new_with_attributes(_("Shortcut"), renderer, "text", KB_TREE_SHORTCUT, NULL);
 	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);
 
-	/* set policy settings for the scollwedwindow around the treeview again, because glade
+	/* set policy settings for the scrolled window around the treeview again, because glade
 	 * doesn't keep the settings */
 	gtk_scrolled_window_set_policy(
 			GTK_SCROLLED_WINDOW(ui_lookup_widget(ui_widgets.prefs_dialog, "scrolledwindow8")),
@@ -308,23 +303,19 @@ static void kb_init(void)
 	if (store == NULL)
 		kb_init_tree();
 
-	for (g = 0; g < keybinding_groups->len; g++)
+	foreach_ptr_array(group, g, keybinding_groups)
 	{
-		group = g_ptr_array_index(keybinding_groups, g);
-
 		gtk_tree_store_append(store, &parent, NULL);
 		gtk_tree_store_set(store, &parent, KB_TREE_ACTION, group->label,
 			KB_TREE_INDEX, g, -1);
 
-		for (i = 0; i < group->count; i++)
+		foreach_ptr_array(kb, i, group->key_items)
 		{
-			kb = &group->keys[i];
-
 			label = keybindings_get_label(kb);
 			key_string = gtk_accelerator_name(kb->key, kb->mods);
 			gtk_tree_store_append(store, &iter, &parent);
 			gtk_tree_store_set(store, &iter, KB_TREE_ACTION, label,
-				KB_TREE_SHORTCUT, key_string, KB_TREE_INDEX, i, -1);
+				KB_TREE_SHORTCUT, key_string, KB_TREE_INDEX, kb->id, -1);
 			g_free(key_string);
 			g_free(label);
 		}
@@ -333,7 +324,7 @@ static void kb_init(void)
 }
 
 
-/* note: new prefs should use Stash code in keyfile.c */
+/* note: new 'simple' prefs should use Stash code in keyfile.c */
 static void prefs_init_dialog(void)
 {
 	GtkWidget *widget;
@@ -406,12 +397,12 @@ static void prefs_init_dialog(void)
 	gtk_font_button_set_font_name(GTK_FONT_BUTTON(widget), interface_prefs.editor_font);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_long_line");
-	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.long_line_global_column);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(widget), editor_prefs.long_line_column);
 
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_long_line");
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.long_line_global_enabled);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), editor_prefs.long_line_enabled);
 
-	switch (editor_prefs.long_line_global_type)
+	switch (editor_prefs.long_line_type)
 	{
 		case 0: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_line"); break;
 		case 1: widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_background"); break;
@@ -520,6 +511,9 @@ static void prefs_init_dialog(void)
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_new_line");
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), file_prefs.final_new_line);
 
+	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_ensure_convert_new_lines");
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), file_prefs.ensure_convert_new_lines);
+
 	/* Editor settings */
 	widget = ui_lookup_widget(ui_widgets.prefs_dialog, "entry_toggle_mark");
 	gtk_entry_set_text(GTK_ENTRY(widget), editor_prefs.comment_toggle_mark);
@@ -750,12 +744,56 @@ static void prefs_init_dialog(void)
 }
 
 
+/* read the treeview shortcut fields into keybindings */
+static void kb_update(void)
+{
+	GtkTreeModel *model = GTK_TREE_MODEL(store);
+	GtkTreeIter child, parent;
+	guint gid = 0;
+
+	/* get first parent */
+	if (! gtk_tree_model_iter_children(model, &parent, NULL))
+		return;
+
+	/* foreach parent */
+	while (TRUE)
+	{
+		/* get first child */
+		if (! gtk_tree_model_iter_children(model, &child, &parent))
+			return;
+
+		/* foreach child */
+		while (TRUE)
+		{
+			guint kid;
+			gchar *str;
+			guint key;
+			GdkModifierType mods;
+			GeanyKeyBinding *kb;
+
+			gtk_tree_model_get(model, &child, KB_TREE_INDEX, &kid, KB_TREE_SHORTCUT, &str, -1);
+			gtk_accelerator_parse(str, &key, &mods);
+			g_free(str);
+			kb = keybindings_lookup_item(gid, kid);
+			if (kb->key != key || kb->mods != mods)
+				keybindings_update_combo(kb, key, mods);
+
+			if (! gtk_tree_model_iter_next(model, &child))
+				break;
+		}
+		if (! gtk_tree_model_iter_next(model, &parent))
+			return;
+		gid++;
+	}
+}
+
+
 /*
  * callbacks
  */
-/* note: new prefs should use Stash code in keyfile.c */
+/* note: new 'simple' prefs should use Stash code in keyfile.c */
 static void
-on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
+on_prefs_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 {
 	if (response == GTK_RESPONSE_OK || response == GTK_RESPONSE_APPLY)
 	{
@@ -826,17 +864,17 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		interface_prefs.sidebar_openfiles_visible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_long_line");
-		editor_prefs.long_line_global_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+		editor_prefs.long_line_enabled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_long_line_line");
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-			editor_prefs.long_line_global_type = 0;
+			editor_prefs.long_line_type = 0;
 		else
 			/* now only the "background" radio remains */
-			editor_prefs.long_line_global_type = 1;
+			editor_prefs.long_line_type = 1;
 
-		if (editor_prefs.long_line_global_column == 0)
-			editor_prefs.long_line_global_enabled = FALSE;
+		if (editor_prefs.long_line_column == 0)
+			editor_prefs.long_line_enabled = FALSE;
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_show_notebook_tabs");
 		interface_prefs.show_notebook_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -926,6 +964,9 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_new_line");
 		file_prefs.final_new_line = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
+		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_ensure_convert_new_lines");
+		file_prefs.ensure_convert_new_lines = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
+
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_replace_tabs");
 		file_prefs.replace_tabs = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
 
@@ -938,7 +979,7 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "spin_long_line");
 		/* note: use stash for new code - it updates spin buttons itself */
 		gtk_spin_button_update(GTK_SPIN_BUTTON(widget));
-		editor_prefs.long_line_global_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
+		editor_prefs.long_line_column = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "check_folding");
 		editor_prefs.folding = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
@@ -1077,8 +1118,10 @@ on_prefs_button_clicked(GtkDialog *dialog, gint response, gpointer user_data)
 
 		/* Keybindings */
 		if (edited)
+		{
+			kb_update();
 			keybindings_write_to_file();
-
+		}
 
 		/* Printing */
 		widget = ui_lookup_widget(ui_widgets.prefs_dialog, "radio_print_gtk");
@@ -1257,38 +1300,44 @@ static GeanyKeyBinding *kb_lookup_kb_from_iter(G_GNUC_UNUSED GtkTreeModel *model
 }
 
 
-static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext, gchar *path, gchar *new_text, gpointer user_data)
+static void kb_change_iter_shortcut(GtkTreeIter *iter, const gchar *new_text)
+{
+	guint lkey;
+	GdkModifierType lmods;
+	GeanyKeyBinding *kb;
+
+	gtk_accelerator_parse(new_text, &lkey, &lmods);
+
+	kb = kb_lookup_kb_from_iter(GTK_TREE_MODEL(store), iter);
+
+	if (kb_find_duplicate(ui_widgets.prefs_dialog, kb, lkey, lmods, new_text))
+		return;
+
+	/* set the values here, because of the above check, setting it in
+	 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
+	gtk_tree_store_set(store, iter, KB_TREE_SHORTCUT, new_text, -1);
+
+	edited = TRUE;
+}
+
+
+static void kb_cell_edited_cb(GtkCellRendererText *cellrenderertext,
+		gchar *path, gchar *new_text, gpointer user_data)
 {
 	if (path != NULL && new_text != NULL)
 	{
 		GtkTreeIter iter;
-		guint lkey;
-		GdkModifierType lmods;
-		GeanyKeyBinding *kb;
 
 		gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(store), &iter, path);
 		if (gtk_tree_model_iter_has_child(GTK_TREE_MODEL(store), &iter))
 			return;	/* ignore group items */
 
-		gtk_accelerator_parse(new_text, &lkey, &lmods);
-
-		kb = kb_lookup_kb_from_iter(GTK_TREE_MODEL(store), &iter);
-
-		if (kb_find_duplicate(ui_widgets.prefs_dialog, kb, lkey, lmods, new_text))
-			return;
-
-		/* set the values here, because of the above check, setting it in
-		 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
-		keybindings_update_combo(kb, lkey, lmods);
-
-		gtk_tree_store_set(store, &iter, KB_TREE_SHORTCUT, new_text, -1);
-
-		edited = TRUE;
+		kb_change_iter_shortcut(&iter, new_text);
 	}
 }
 
 
-static gboolean kb_keytype_dialog_response_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data)
+static gboolean kb_grab_key_dialog_key_press_cb(GtkWidget *dialog, GdkEventKey *event, gpointer user_data)
 {
 	gchar *str;
 	gint state;
@@ -1307,32 +1356,13 @@ static gboolean kb_keytype_dialog_response_cb(GtkWidget *dialog, GdkEventKey *ev
 }
 
 
-static void kb_dialog_response_cb(GtkWidget *dialog, gint response, G_GNUC_UNUSED gpointer iter)
+static void kb_grab_key_dialog_response_cb(GtkWidget *dialog, gint response, G_GNUC_UNUSED gpointer iter)
 {
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
-		guint lkey;
-		GdkModifierType lmods;
-		GeanyKeyBinding *kb;
+		const gchar *new_text = gtk_label_get_text(GTK_LABEL(dialog_label));
 
-		kb = kb_lookup_kb_from_iter(GTK_TREE_MODEL(store), &g_iter);
-
-		gtk_accelerator_parse(gtk_label_get_text(GTK_LABEL(dialog_label)), &lkey, &lmods);
-
-		if (kb_find_duplicate(dialog, kb, lkey, lmods, gtk_label_get_text(GTK_LABEL(dialog_label))))
-			return;
-
-		/* set the values here, because of the above check, setting it in
-		 * gtk_accelerator_parse would return a wrong key combination if it is duplicate */
-		keybindings_update_combo(kb, lkey, lmods);
-
-		gtk_tree_store_set(store, &g_iter,
-			KB_TREE_SHORTCUT, gtk_label_get_text(GTK_LABEL(dialog_label)), -1);
-
-		g_free(dialog_key_name);
-		dialog_key_name = NULL;
-
-		edited = TRUE;
+		kb_change_iter_shortcut(&g_iter, new_text);
 	}
 	gtk_widget_destroy(dialog);
 }
@@ -1381,25 +1411,22 @@ static void kb_clear_tree_shortcut(gsize group_id, gsize keybinding_id)
 /* test if the entered key combination is already used
  * returns true if cancelling duplicate */
 static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
-		guint key, GdkModifierType mods, const gchar *action)
+		guint key, GdkModifierType mods, const gchar *shortcut)
 {
 	gsize g, i;
+	GeanyKeyGroup *group;
+	GeanyKeyBinding *kb;
 
 	/* allow duplicate if there is no key combination */
 	if (key == 0 && mods == 0)
 		return FALSE;
 
-	for (g = 0; g < keybinding_groups->len; g++)
+	foreach_ptr_array(group, g, keybinding_groups)
 	{
-		GeanyKeyGroup *group = g_ptr_array_index(keybinding_groups, g);
-
-		for (i = 0; i < group->count; i++)
+		foreach_ptr_array(kb, i, group->key_items)
 		{
-			GeanyKeyBinding *keys = group->keys;
-			GeanyKeyBinding *kb = &keys[i];
-
 			/* search another item with the same key,
-			 * but don't search the key we're looking for keys[idx] */
+			 * but don't search the key we're looking for(!) */
 			if (kb->key == key && kb->mods == mods
 				&& ! (kb->key == search_kb->key && kb->mods == search_kb->mods))
 			{
@@ -1410,14 +1437,13 @@ static gboolean kb_find_duplicate(GtkWidget *parent, GeanyKeyBinding *search_kb,
 					_("_Override"), GTK_RESPONSE_YES,
 					_("Override that keybinding?"),
 					_("The combination '%s' is already used for \"%s\"."),
-					action, label);
+					shortcut, label);
 
 				g_free(label);
 
 				if (ret == GTK_RESPONSE_YES)
 				{
-					keybindings_update_combo(kb, 0, 0);
-					kb_clear_tree_shortcut(g, i);
+					kb_clear_tree_shortcut(g, kb->id);
 					/* carry on looking for other duplicates if overriding */
 					continue;
 				}
@@ -1552,15 +1578,12 @@ static void open_preferences_help(void)
 static gboolean prefs_dialog_key_press_response_cb(GtkWidget *dialog, GdkEventKey *event,
 												   gpointer data)
 {
-	gint group, keybinding;
+	GeanyKeyBinding *kb = keybindings_lookup_item(GEANY_KEY_GROUP_HELP, GEANY_KEYS_HELP_HELP);
 
-	if (keybindings_check_event(event, &group, &keybinding) != NULL)
+	if (keybindings_check_event(event, kb))
 	{
-		if (group == GEANY_KEY_GROUP_HELP && keybinding == GEANY_KEYS_HELP_HELP)
-		{
-			open_preferences_help();
-			return TRUE;
-		}
+		open_preferences_help();
+		return TRUE;
 	}
 	return FALSE;
 }
@@ -1690,7 +1713,7 @@ void prefs_show_dialog(void)
 			GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_ENTRY(ui_lookup_widget(ui_widgets.prefs_dialog, "extra_plugin_path_entry")));
 
 		g_signal_connect(ui_widgets.prefs_dialog, "response",
-			G_CALLBACK(on_prefs_button_clicked), NULL);
+			G_CALLBACK(on_prefs_dialog_response), NULL);
 		g_signal_connect(ui_widgets.prefs_dialog, "delete-event",
 			G_CALLBACK(gtk_widget_hide_on_delete), NULL);
 
