@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: project.c 5142 2010-08-11 14:41:15Z ntrel $
+ * $Id: project.c 5432 2010-11-23 17:36:29Z ntrel $
  */
 
 /** @file project.h
@@ -191,6 +191,9 @@ void project_new(void)
 	{
 		if (update_config(e))
 		{
+			write_config(TRUE);
+			ui_set_statusbar(TRUE, _("Project \"%s\" created."), app->project->name);
+
 			ui_add_recent_project_file(app->project->file_name);
 			break;
 		}
@@ -320,7 +323,7 @@ static void update_ui(void)
 
 static void remove_foreach_project_filetype(gpointer data, gpointer user_data)
 {
-	GeanyFiletype *ft = (GeanyFiletype*)data;
+	GeanyFiletype *ft = data;
 	if (ft != NULL)
 	{
 		setptr(ft->projfilecmds, NULL);
@@ -382,18 +385,12 @@ void project_close(gboolean open_default)
 }
 
 
-static void on_set_use_base_path_clicked(GtkWidget *unused1, gpointer user_data)
-{
-	build_set_non_ft_wd_to_proj((BuildTableData)user_data);
-}
-
-
 static gint build_page_num = 0;
 
+
 static void create_properties_dialog(PropertyDialogElements *e)
 {
 	GtkWidget *table, *notebook, *build_table;
-	GtkWidget *button;
 	GtkWidget *bbox;
 	GtkWidget *label;
 	GtkWidget *swin;
@@ -407,34 +404,34 @@ static void create_properties_dialog(PropertyDialogElements *e)
 
 	ui_entry_add_clear_icon(GTK_ENTRY(ui_lookup_widget(e->dialog, "spin_indent_width")));
 
-	table = gtk_table_new(6, 2, FALSE);
+	table = gtk_table_new(5, 2, FALSE);
 	gtk_container_set_border_width(GTK_CONTAINER(table), 6);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 5);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
 
-	label = gtk_label_new(_("Name:"));
+	label = gtk_label_new(_("Filename:"));
 	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
 
+	e->file_name = gtk_label_new("");
+	gtk_label_set_selectable(GTK_LABEL(e->file_name), TRUE);
+	gtk_table_attach(GTK_TABLE(table), e->file_name, 1, 2, 0, 1,
+					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(e->file_name), 0, 0);
+
+	label = gtk_label_new(_("Name:"));
+	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
+					(GtkAttachOptions) (GTK_FILL),
+					(GtkAttachOptions) (0), 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
+
 	e->name = gtk_entry_new();
 	ui_entry_add_clear_icon(GTK_ENTRY(e->name));
 	gtk_entry_set_max_length(GTK_ENTRY(e->name), MAX_NAME_LEN);
-	gtk_table_attach(GTK_TABLE(table), e->name, 1, 2, 0, 1,
-					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-
-	label = gtk_label_new(_("Filename:"));
-	gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
-					(GtkAttachOptions) (GTK_FILL),
-					(GtkAttachOptions) (0), 0, 0);
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-
-	e->file_name = gtk_entry_new();
-	ui_entry_add_clear_icon(GTK_ENTRY(e->file_name));
-	gtk_editable_set_editable(GTK_EDITABLE(e->file_name), FALSE);	/* read-only */
-	gtk_table_attach(GTK_TABLE(table), e->file_name, 1, 2, 1, 2,
+	gtk_table_attach(GTK_TABLE(table), e->name, 1, 2, 1, 2,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 
@@ -481,22 +478,6 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	build_page_num = gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), build_table, label, 2);
 	e->notebook = notebook;
 
-	label = gtk_label_new(_("Set the Build non-filetype working directories to use base path:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-
-	button = gtk_button_new_with_label(_("Set"));
-	ui_widget_set_tooltip_text(button,
-		_("Set the working directories (on the Build tab) "
-		"for the non-filetype build commands to use the base path"));
-	gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
-	g_signal_connect(button, "clicked", G_CALLBACK(on_set_use_base_path_clicked), e->build_properties);
-	bbox = gtk_hbox_new(FALSE, 6);
-	gtk_box_pack_start(GTK_BOX(bbox), label, TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(bbox), button, FALSE, FALSE, 0);
-	gtk_table_attach(GTK_TABLE(table), bbox, 0, 2, 4, 5,
-					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					(GtkAttachOptions) (GTK_FILL), 0, 0);
-
 	g_signal_connect(ui_lookup_widget(e->dialog, "radio_long_line_custom"), "toggled",
 		G_CALLBACK(on_radio_long_line_custom_toggled), ui_lookup_widget(e->dialog, "spin_long_line"));
 
@@ -514,7 +495,7 @@ static void create_properties_dialog(PropertyDialogElements *e)
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin),
 				GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(swin), GTK_WIDGET(e->patterns));
-	gtk_table_attach(GTK_TABLE(table), swin, 1, 2, 6, 7,
+	gtk_table_attach(GTK_TABLE(table), swin, 1, 2, 4, 5,
 					(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 #endif
@@ -542,7 +523,7 @@ static void show_project_properties(gboolean show_build)
 
 	/* fill the elements with the appropriate data */
 	gtk_entry_set_text(GTK_ENTRY(e->name), p->name);
-	gtk_entry_set_text(GTK_ENTRY(e->file_name), p->file_name);
+	gtk_label_set_text(GTK_LABEL(e->file_name), p->file_name);
 	gtk_entry_set_text(GTK_ENTRY(e->base_path), p->base_path);
 
 	radio_long_line_custom = ui_lookup_widget(e->dialog, "radio_long_line_custom");
@@ -585,6 +566,7 @@ static void show_project_properties(gboolean show_build)
 	}
 #endif
 
+	g_signal_emit_by_name(geany_object, "project-dialog-create", e->notebook);
 	gtk_widget_show_all(e->dialog);
 
 	/* note: notebook page must be shown before setting current page */
@@ -597,7 +579,9 @@ static void show_project_properties(gboolean show_build)
 	{
 		if (update_config(e))
 		{
-			stash_group_update(indent_group, e->dialog);
+			g_signal_emit_by_name(geany_object, "project-dialog-confirmed", e->notebook);
+			write_config(TRUE);
+			ui_set_statusbar(TRUE, _("Project \"%s\" saved."), app->project->name);
 			break;
 		}
 	}
@@ -651,7 +635,7 @@ static GeanyProject *create_project(void)
 	project->priv = &priv;
 
 	project->long_line_behaviour = 1 /* use global settings */;
-	project->long_line_column = editor_prefs.long_line_global_column;
+	project->long_line_column = editor_prefs.long_line_column;
 
 	app->project = project;
 	return project;
@@ -686,7 +670,11 @@ static gboolean update_config(const PropertyDialogElements *e)
 		return FALSE;
 	}
 
-	file_name = gtk_entry_get_text(GTK_ENTRY(e->file_name));
+	if (app->project == NULL)
+		file_name = gtk_entry_get_text(GTK_ENTRY(e->file_name));
+	else
+		file_name = gtk_label_get_text(GTK_LABEL(e->file_name));
+
 	if (! NZV(file_name))
 	{
 		SHOW_ERR(_("You have specified an invalid project filename."));
@@ -768,6 +756,8 @@ static gboolean update_config(const PropertyDialogElements *e)
 		gtk_text_buffer_get_end_iter(buffer, &end);
 		setptr(p->description, g_strdup(gtk_text_buffer_get_text(buffer, &start, &end, FALSE)));
 
+		stash_group_update(indent_group, e->dialog);
+
 		/* read the project build menu */
 		oldvalue = ft ? ft->projfilecmds : NULL;
 		build_read_project(ft, e->build_properties);
@@ -809,11 +799,6 @@ static gboolean update_config(const PropertyDialogElements *e)
 		g_free(tmp);
 #endif
 	}
-	write_config(TRUE);
-	if (new_project)
-		ui_set_statusbar(TRUE, _("Project \"%s\" created."), p->name);
-	else
-		ui_set_statusbar(TRUE, _("Project \"%s\" saved."), p->name);
 
 	update_ui();
 
@@ -1013,7 +998,7 @@ static gboolean load_config(const gchar *filename)
 	p->long_line_behaviour = utils_get_setting_integer(config, "long line marker",
 		"long_line_behaviour", 1 /* follow global */);
 	p->long_line_column = utils_get_setting_integer(config, "long line marker",
-		"long_line_column", editor_prefs.long_line_global_column);
+		"long_line_column", editor_prefs.long_line_column);
 	apply_editor_prefs();
 
 	build_load_menu(config, GEANY_BCS_PROJ, (gpointer)p);
