@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: search.c 5153 2010-08-12 16:51:55Z ntrel $
+ * $Id: search.c 5299 2010-10-16 12:37:20Z eht16 $
  */
 
 /*
@@ -42,6 +42,7 @@
 #include "project.h"
 #include "keyfile.h"
 #include "stash.h"
+#include "toolbar.h"
 
 #include <unistd.h>
 #include <string.h>
@@ -73,30 +74,24 @@ enum
 	GEANY_RESPONSE_REPLACE_IN_SEL
 };
 
-enum
-{
-	FIF_FGREP,
-	FIF_GREP,
-	FIF_EGREP
-};
-
 
 GeanySearchData search_data;
-
 GeanySearchPrefs search_prefs;
 
 
 static struct
 {
-	gint fif_mode;
-	gchar *fif_extra_options;
+	gboolean fif_regexp;
 	gboolean fif_case_sensitive;
 	gboolean fif_match_whole_word;
 	gboolean fif_invert_results;
 	gboolean fif_recursive;
 	gboolean fif_use_extra_options;
+	gchar *fif_extra_options;
+	gboolean fif_use_files;
+	gchar *fif_files;
 }
-settings = {0, NULL, FALSE, FALSE, FALSE, FALSE, FALSE};
+settings = {FALSE, FALSE, FALSE, FALSE, FALSE, FALSE, NULL, FALSE, NULL};
 
 static StashGroup *fif_prefs = NULL;
 
@@ -124,11 +119,12 @@ static struct
 {
 	GtkWidget	*dialog;
 	GtkWidget	*dir_combo;
+	GtkWidget	*files_combo;
 	GtkWidget	*search_combo;
 	GtkWidget	*encoding_combo;
 	gint		position[2]; /* x, y */
 }
-fif_dlg = {NULL, NULL, NULL, NULL, {0, 0}};
+fif_dlg = {NULL, NULL, NULL, NULL, NULL, {0, 0}};
 
 
 static gboolean search_read_io(GIOChannel *source, GIOCondition condition, gpointer data);
@@ -154,9 +150,6 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 static void
 on_replace_entry_activate(GtkEntry *entry, gpointer user_data);
 
-static gboolean
-on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
-
 static void
 on_find_in_files_dialog_response(GtkDialog *dialog, gint response, gpointer user_data);
 
@@ -186,13 +179,8 @@ static void init_prefs(void)
 	group = stash_group_new("search");
 	fif_prefs = group;
 	configuration_add_pref_group(group, FALSE);
-	stash_group_add_radio_buttons(group, &settings.fif_mode, "fif_mode", FIF_FGREP,
-		"radio_fgrep", FIF_FGREP,
-		"radio_grep", FIF_GREP,
-		"radio_egrep", FIF_EGREP,
-		NULL);
-	stash_group_add_entry(group, &settings.fif_extra_options,
-		"fif_extra_options", "", "entry_extra");
+	stash_group_add_toggle_button(group, &settings.fif_regexp,
+		"fif_regexp", FALSE, "check_regexp");
 	stash_group_add_toggle_button(group, &settings.fif_case_sensitive,
 		"fif_case_sensitive", TRUE, "check_case");
 	stash_group_add_toggle_button(group, &settings.fif_match_whole_word,
@@ -201,8 +189,14 @@ static void init_prefs(void)
 		"fif_invert_results", FALSE, "check_invert");
 	stash_group_add_toggle_button(group, &settings.fif_recursive,
 		"fif_recursive", FALSE, "check_recursive");
+	stash_group_add_entry(group, &settings.fif_extra_options,
+		"fif_extra_options", "", "entry_extra");
 	stash_group_add_toggle_button(group, &settings.fif_use_extra_options,
 		"fif_use_extra_options", FALSE, "check_extra");
+	stash_group_add_entry(group, &settings.fif_files,
+		"fif_files", "", "entry_files");
+	stash_group_add_toggle_button(group, &settings.fif_use_files,
+		"fif_use_files", FALSE, "check_files");
 }
 
 
@@ -231,8 +225,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 			  *checkbox7, *hbox, *fbox, *mbox;
 
 	check_regexp = gtk_check_button_new_with_mnemonic(_("_Use regular expressions"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_regexp",
-					g_object_ref(check_regexp), (GDestroyNotify) g_object_unref);
+	ui_hookup_widget(dialog, check_regexp, "check_regexp");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_regexp), FALSE);
 	ui_widget_set_tooltip_text(check_regexp, _("Use POSIX-like regular expressions. "
 		"For detailed information about using regular expressions, please read the documentation."));
@@ -242,8 +235,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	if (dialog != GTK_DIALOG(find_dlg.dialog))
 	{
 		check_back = gtk_check_button_new_with_mnemonic(_("Search _backwards"));
-		g_object_set_data_full(G_OBJECT(dialog), "check_back",
-						g_object_ref(check_back), (GDestroyNotify)g_object_unref);
+		ui_hookup_widget(dialog, check_back, "check_back");
 		gtk_button_set_focus_on_click(GTK_BUTTON(check_back), FALSE);
 	}
 	else
@@ -256,8 +248,7 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 		g_object_unref(label_size);
 	}
 	checkbox7 = gtk_check_button_new_with_mnemonic(_("Use _escape sequences"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_escape",
-					g_object_ref(checkbox7), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(dialog, checkbox7, "check_escape");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox7), FALSE);
 	ui_widget_set_tooltip_text(checkbox7,
 		_("Replace \\\\, \\t, \\n, \\r and \\uXXXX (Unicode chararacters) with the "
@@ -270,18 +261,15 @@ static GtkWidget *add_find_checkboxes(GtkDialog *dialog)
 	gtk_container_add(GTK_CONTAINER(fbox), check_back);
 
 	checkbox1 = gtk_check_button_new_with_mnemonic(_("C_ase sensitive"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_case",
-					g_object_ref(checkbox1), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(dialog, checkbox1, "check_case");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox1), FALSE);
 
 	checkbox2 = gtk_check_button_new_with_mnemonic(_("Match only a _whole word"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_word",
-					g_object_ref(checkbox2), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(dialog, checkbox2, "check_word");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox2), FALSE);
 
 	checkbox5 = gtk_check_button_new_with_mnemonic(_("Match from s_tart of word"));
-	g_object_set_data_full(G_OBJECT(dialog), "check_wordstart",
-					g_object_ref(checkbox5), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(dialog, checkbox5, "check_wordstart");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox5), FALSE);
 
 	/* Matching options */
@@ -314,27 +302,26 @@ static void setup_find_next(const gchar *text)
 }
 
 
-/* Search for next match of the current "selection"
- * For X11 based systems, this will try to use the system-wide
- * x-selection first. If it doesn't find anything suitable in
- * the x-selection (or if we are on Win32) it will try to use
- * the scintilla selection or current token instead.
+/* Search for next match of the current "selection".
+ * Optionally for X11 based systems, this will try to use the system-wide
+ * x-selection first.
+ * If it doesn't find a suitable search string it will try to use
+ * the current word instead, or just repeat the last search.
  * Search flags are always zero.
  */
 void search_find_selection(GeanyDocument *doc, gboolean search_backwards)
 {
 	gchar *s = NULL;
-#ifdef G_OS_UNIX
-	GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
-#endif
 
 	g_return_if_fail(doc != NULL);
 
 #ifdef G_OS_UNIX
-	s = gtk_clipboard_wait_for_text(clipboard);
-	if (s)
+	if (search_prefs.find_selection_type == GEANY_FIND_SEL_X)
 	{
-		if (strchr(s,'\n') || strchr(s, '\r'))
+		GtkClipboard *clipboard = gtk_clipboard_get(GDK_SELECTION_PRIMARY);
+
+		s = gtk_clipboard_wait_for_text(clipboard);
+		if (s && (strchr(s,'\n') || strchr(s, '\r')))
 		{
 			g_free(s);
 			s = NULL;
@@ -342,8 +329,14 @@ void search_find_selection(GeanyDocument *doc, gboolean search_backwards)
 	}
 #endif
 
-	if (!s)
+	if (!s && sci_has_selection(doc->editor->sci))
+		s = sci_get_selection_contents(doc->editor->sci);
+
+	if (!s && search_prefs.find_selection_type != GEANY_FIND_SEL_AGAIN)
+	{
+		/* get the current word */
 		s = editor_get_default_selection(doc->editor, TRUE, NULL);
+	}
 
 	if (s)
 	{
@@ -353,6 +346,15 @@ void search_find_selection(GeanyDocument *doc, gboolean search_backwards)
 			editor_display_current_line(doc->editor, 0.3F);
 		g_free(s);
 	}
+	else if (search_prefs.find_selection_type == GEANY_FIND_SEL_AGAIN)
+	{
+		/* Repeat last search (in case selection was lost) */
+		search_find_again(search_backwards);
+	}
+	else
+	{
+		utils_beep();
+	}
 }
 
 
@@ -403,8 +405,7 @@ static void create_find_dialog(void)
 	button = ui_button_new_with_image(GTK_STOCK_GO_BACK, _("_Previous"));
 	gtk_dialog_add_action_widget(GTK_DIALOG(find_dlg.dialog), button,
 		GEANY_RESPONSE_FIND_PREVIOUS);
-	g_object_set_data_full(G_OBJECT(find_dlg.dialog), "btn_previous",
-					g_object_ref(button), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(find_dlg.dialog, button, "btn_previous");
 
 	button = ui_button_new_with_image(GTK_STOCK_GO_FORWARD, _("_Next"));
 	gtk_dialog_add_action_widget(GTK_DIALOG(find_dlg.dialog), button,
@@ -419,8 +420,7 @@ static void create_find_dialog(void)
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry))), 50);
 	find_dlg.entry = GTK_BIN(entry)->child;
-	g_object_set_data_full(G_OBJECT(find_dlg.dialog), "entry",
-					g_object_ref(entry), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(find_dlg.dialog, entry, "entry");
 
 	g_signal_connect(gtk_bin_get_child(GTK_BIN(entry)), "activate",
 			G_CALLBACK(on_find_entry_activate), NULL);
@@ -464,8 +464,7 @@ static void create_find_dialog(void)
 
 	/* close window checkbox */
 	check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
-	g_object_set_data_full(G_OBJECT(find_dlg.dialog), "check_close",
-					g_object_ref(check_close), (GDestroyNotify) g_object_unref);
+	ui_hookup_widget(find_dlg.dialog, check_close, "check_close");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
 	ui_widget_set_tooltip_text(check_close,
 			_("Disable this option to keep the dialog open"));
@@ -530,6 +529,18 @@ static void send_replace_dialog_response(GtkButton *button, gpointer user_data)
 }
 
 
+static gboolean
+on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
+{
+	if (event->keyval == GDK_Tab)
+	{
+		gtk_widget_grab_focus(GTK_WIDGET(user_data));
+		return TRUE;
+	}
+	return FALSE;
+}
+
+
 static void create_replace_dialog(void)
 {
 	GtkWidget *label_find, *label_replace, *entry_find, *entry_replace,
@@ -570,8 +581,7 @@ static void create_replace_dialog(void)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label_find), entry_find);
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_find))), 50);
-	g_object_set_data_full(G_OBJECT(replace_dlg.dialog), "entry_find",
-		g_object_ref(entry_find), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(replace_dlg.dialog, entry_find, "entry_find");
 	replace_dlg.find_entry = GTK_BIN(entry_find)->child;
 
 	entry_replace = gtk_combo_box_entry_new_text();
@@ -579,10 +589,10 @@ static void create_replace_dialog(void)
 	gtk_label_set_mnemonic_widget(GTK_LABEL(label_replace), entry_replace);
 	gtk_entry_set_max_length(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 248);
 	gtk_entry_set_width_chars(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(entry_replace))), 50);
-	g_object_set_data_full(G_OBJECT(replace_dlg.dialog), "entry_replace",
-		g_object_ref(entry_replace), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(replace_dlg.dialog, entry_replace, "entry_replace");
 	replace_dlg.replace_entry = GTK_BIN(entry_replace)->child;
 
+	/* tab from find to the replace entry */
 	g_signal_connect(gtk_bin_get_child(GTK_BIN(entry_find)),
 			"key-press-event", G_CALLBACK(on_widget_key_pressed_set_focus),
 			gtk_bin_get_child(GTK_BIN(entry_replace)));
@@ -638,8 +648,7 @@ static void create_replace_dialog(void)
 
 	/* close window checkbox */
 	check_close = gtk_check_button_new_with_mnemonic(_("Close _dialog"));
-	g_object_set_data_full(G_OBJECT(replace_dlg.dialog), "check_close",
-					g_object_ref(check_close), (GDestroyNotify) g_object_unref);
+	ui_hookup_widget(replace_dlg.dialog, check_close, "check_close");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_close), FALSE);
 	ui_widget_set_tooltip_text(check_close,
 			_("Disable this option to keep the dialog open"));
@@ -691,7 +700,7 @@ void search_show_replace_dialog(void)
 }
 
 
-static void on_extra_options_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+static void on_widget_toggled_set_sensitive(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	/* disable extra option entry when checkbutton not checked */
 	gtk_widget_set_sensitive(GTK_WIDGET(user_data),
@@ -701,10 +710,10 @@ static void on_extra_options_toggled(GtkToggleButton *togglebutton, gpointer use
 
 static void create_fif_dialog(void)
 {
-	GtkWidget *dir_combo, *combo, *e_combo, *entry;
+	GtkWidget *dir_combo, *combo, *fcombo, *e_combo, *entry;
 	GtkWidget *label, *label1, *label2, *checkbox1, *checkbox2, *check_wholeword,
-		*check_recursive, *check_extra, *entry_extra;
-	GtkWidget *dbox, *sbox, *cbox, *rbox, *rbtn, *hbox, *vbox, *ebox;
+		*check_recursive, *check_extra, *entry_extra, *check_regexp, *check;
+	GtkWidget *dbox, *sbox, *lbox, *rbox, *hbox, *vbox, *ebox;
 	GtkSizeGroup *size_group;
 	gchar *encoding_string;
 	guint i;
@@ -722,23 +731,8 @@ static void create_fif_dialog(void)
 	gtk_dialog_set_default_response(GTK_DIALOG(fif_dlg.dialog),
 		GTK_RESPONSE_ACCEPT);
 
-	label1 = gtk_label_new_with_mnemonic(_("_Directory:"));
-	gtk_misc_set_alignment(GTK_MISC(label1), 1, 0.5);
-
-	dir_combo = gtk_combo_box_entry_new_text();
-	entry = gtk_bin_get_child(GTK_BIN(dir_combo));
-	ui_entry_add_clear_icon(GTK_ENTRY(entry));
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label1), entry);
-	gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
-	gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
-	fif_dlg.dir_combo = dir_combo;
-
-	dbox = ui_path_box_new(NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
-		GTK_ENTRY(entry));
-	gtk_box_pack_start(GTK_BOX(dbox), label1, FALSE, FALSE, 0);
-
 	label = gtk_label_new_with_mnemonic(_("_Search for:"));
-	gtk_misc_set_alignment(GTK_MISC(label), 1, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
 
 	combo = gtk_combo_box_entry_new_text();
 	entry = gtk_bin_get_child(GTK_BIN(combo));
@@ -753,8 +747,57 @@ static void create_fif_dialog(void)
 	gtk_box_pack_start(GTK_BOX(sbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(sbox), combo, TRUE, TRUE, 0);
 
+	/* make labels same width */
+	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
+	gtk_size_group_add_widget(size_group, label);
+
+	check = gtk_check_button_new_with_mnemonic(_("Fi_les:"));
+	ui_hookup_widget(fif_dlg.dialog, check, "check_files");
+	gtk_button_set_focus_on_click(GTK_BUTTON(check), FALSE);
+	gtk_size_group_add_widget(size_group, check);
+
+	/* tab from search to the files checkbox */
+	g_signal_connect(entry, "key-press-event",
+		G_CALLBACK(on_widget_key_pressed_set_focus), check);
+
+	fcombo = gtk_combo_box_entry_new_text();
+	gtk_widget_set_sensitive(fcombo, FALSE);
+	entry = gtk_bin_get_child(GTK_BIN(fcombo));
+	ui_entry_add_clear_icon(GTK_ENTRY(entry));
+	gtk_entry_set_activates_default(GTK_ENTRY(entry), TRUE);
+	ui_widget_set_tooltip_text(entry, _("File patterns, e.g. *.c *.h"));
+	ui_hookup_widget(fif_dlg.dialog, entry, "entry_files");
+	fif_dlg.files_combo = fcombo;
+
+	/* enable entry when check is checked */
+	g_signal_connect(check, "toggled",
+		G_CALLBACK(on_widget_toggled_set_sensitive), fcombo);
+
+	hbox = gtk_hbox_new(FALSE, 6);
+	gtk_box_pack_start(GTK_BOX(hbox), check, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(hbox), fcombo, TRUE, TRUE, 0);
+
+	label1 = gtk_label_new_with_mnemonic(_("_Directory:"));
+	gtk_misc_set_alignment(GTK_MISC(label1), 0, 0.5);
+
+	dir_combo = gtk_combo_box_entry_new_text();
+	entry = gtk_bin_get_child(GTK_BIN(dir_combo));
+	ui_entry_add_clear_icon(GTK_ENTRY(entry));
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label1), entry);
+	gtk_entry_set_max_length(GTK_ENTRY(entry), 248);
+	gtk_entry_set_width_chars(GTK_ENTRY(entry), 50);
+	fif_dlg.dir_combo = dir_combo;
+
+	/* tab from files to the dir entry */
+	g_signal_connect(gtk_bin_get_child(GTK_BIN(fcombo)), "key-press-event",
+		G_CALLBACK(on_widget_key_pressed_set_focus), entry);
+
+	dbox = ui_path_box_new(NULL, GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER,
+		GTK_ENTRY(entry));
+	gtk_box_pack_start(GTK_BOX(dbox), label1, FALSE, FALSE, 0);
+
 	label2 = gtk_label_new_with_mnemonic(_("E_ncoding:"));
-	gtk_misc_set_alignment(GTK_MISC(label2), 1, 0.5);
+	gtk_misc_set_alignment(GTK_MISC(label2), 0, 0.5);
 
 	e_combo = gtk_combo_box_new_text();
 	for (i = 0; i < GEANY_ENCODINGS_MAX; i++)
@@ -771,78 +814,56 @@ static void create_fif_dialog(void)
 	gtk_box_pack_start(GTK_BOX(ebox), label2, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(ebox), e_combo, TRUE, TRUE, 0);
 
-	size_group = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
 	gtk_size_group_add_widget(size_group, label1);
-	gtk_size_group_add_widget(size_group, label);
 	gtk_size_group_add_widget(size_group, label2);
 	g_object_unref(G_OBJECT(size_group));	/* auto destroy the size group */
 
-	rbox = gtk_vbox_new(FALSE, 0);
-	rbtn = gtk_radio_button_new_with_mnemonic(NULL, _("Fixed s_trings"));
-	/* Make fixed strings the default to speed up searching all files in directory. */
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rbtn), TRUE);
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "radio_fgrep",
-					g_object_ref(rbtn), (GDestroyNotify)g_object_unref);
-	gtk_button_set_focus_on_click(GTK_BUTTON(rbtn), FALSE);
-	gtk_container_add(GTK_CONTAINER(rbox), rbtn);
+	gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), dbox, TRUE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), ebox, TRUE, FALSE, 0);
 
-	rbtn = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(rbtn),
-				_("_Grep regular expressions"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "radio_grep",
-					g_object_ref(rbtn), (GDestroyNotify)g_object_unref);
-	ui_widget_set_tooltip_text(rbtn, _("See grep's manual page for more information"));
-	gtk_button_set_focus_on_click(GTK_BUTTON(rbtn), FALSE);
-	gtk_container_add(GTK_CONTAINER(rbox), rbtn);
-
-	rbtn = gtk_radio_button_new_with_mnemonic_from_widget(GTK_RADIO_BUTTON(rbtn),
-				_("_Extended regular expressions"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "radio_egrep",
-					g_object_ref(rbtn), (GDestroyNotify)g_object_unref);
-	ui_widget_set_tooltip_text(rbtn, _("See grep's manual page for more information"));
-	gtk_button_set_focus_on_click(GTK_BUTTON(rbtn), FALSE);
-	gtk_container_add(GTK_CONTAINER(rbox), rbtn);
+	check_regexp = gtk_check_button_new_with_mnemonic(_("_Use regular expressions"));
+	ui_hookup_widget(fif_dlg.dialog, check_regexp, "check_regexp");
+	gtk_button_set_focus_on_click(GTK_BUTTON(check_regexp), FALSE);
+	ui_widget_set_tooltip_text(check_regexp, _("See grep's manual page for more information"));
 
 	check_recursive = gtk_check_button_new_with_mnemonic(_("_Recurse in subfolders"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "check_recursive",
-					g_object_ref(check_recursive), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(fif_dlg.dialog, check_recursive, "check_recursive");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_recursive), FALSE);
 
 	checkbox1 = gtk_check_button_new_with_mnemonic(_("C_ase sensitive"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "check_case",
-					g_object_ref(checkbox1), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(fif_dlg.dialog, checkbox1, "check_case");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox1), FALSE);
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox1), TRUE);
 
 	check_wholeword = gtk_check_button_new_with_mnemonic(_("Match only a _whole word"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "check_wholeword",
-					g_object_ref(check_wholeword), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(fif_dlg.dialog, check_wholeword, "check_wholeword");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_wholeword), FALSE);
 
 	checkbox2 = gtk_check_button_new_with_mnemonic(_("_Invert search results"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "check_invert",
-					g_object_ref(checkbox2), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(fif_dlg.dialog, checkbox2, "check_invert");
 	gtk_button_set_focus_on_click(GTK_BUTTON(checkbox2), FALSE);
 	ui_widget_set_tooltip_text(checkbox2,
 			_("Invert the sense of matching, to select non-matching lines"));
 
-	cbox = gtk_vbox_new(FALSE, 0);
-	gtk_container_add(GTK_CONTAINER(cbox), checkbox1);
-	gtk_container_add(GTK_CONTAINER(cbox), check_wholeword);
-	gtk_container_add(GTK_CONTAINER(cbox), checkbox2);
-	gtk_container_add(GTK_CONTAINER(cbox), check_recursive);
+	lbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(lbox), check_regexp);
+	gtk_container_add(GTK_CONTAINER(lbox), checkbox2);
+	gtk_container_add(GTK_CONTAINER(lbox), check_recursive);
+
+	rbox = gtk_vbox_new(FALSE, 0);
+	gtk_container_add(GTK_CONTAINER(rbox), checkbox1);
+	gtk_container_add(GTK_CONTAINER(rbox), check_wholeword);
+	gtk_container_add(GTK_CONTAINER(rbox), gtk_label_new(NULL));
 
 	hbox = gtk_hbox_new(FALSE, 6);
+	gtk_container_add(GTK_CONTAINER(hbox), lbox);
 	gtk_container_add(GTK_CONTAINER(hbox), rbox);
-	gtk_container_add(GTK_CONTAINER(hbox), cbox);
-
-	gtk_box_pack_start(GTK_BOX(vbox), dbox, TRUE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), sbox, TRUE, FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(vbox), ebox, TRUE, FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
 	check_extra = gtk_check_button_new_with_mnemonic(_("E_xtra options:"));
-	g_object_set_data_full(G_OBJECT(fif_dlg.dialog), "check_extra",
-					g_object_ref(check_extra), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(fif_dlg.dialog, check_extra, "check_extra");
 	gtk_button_set_focus_on_click(GTK_BUTTON(check_extra), FALSE);
 
 	entry_extra = gtk_entry_new();
@@ -853,15 +874,13 @@ static void create_fif_dialog(void)
 
 	/* enable entry_extra when check_extra is checked */
 	g_signal_connect(check_extra, "toggled",
-		G_CALLBACK(on_extra_options_toggled), entry_extra);
+		G_CALLBACK(on_widget_toggled_set_sensitive), entry_extra);
 
 	hbox = gtk_hbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), check_extra, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), entry_extra, TRUE, TRUE, 0);
 	gtk_container_add(GTK_CONTAINER(vbox), hbox);
 
-	g_signal_connect(dir_combo, "key-press-event",
-			G_CALLBACK(on_widget_key_pressed_set_focus), combo);
 	g_signal_connect(fif_dlg.dialog, "response",
 			G_CALLBACK(on_find_in_files_dialog_response), NULL);
 	g_signal_connect(fif_dlg.dialog, "delete-event",
@@ -1022,7 +1041,7 @@ gint search_mark_all(GeanyDocument *doc, const gchar *search_text, gint flags)
 		if (len)
 			editor_indicator_set_on_range(doc->editor, GEANY_INDICATOR_SEARCH, pos, pos + len);
 
-		ttf.chrg.cpMin = ttf.chrgText.cpMax + 1;
+		ttf.chrg.cpMin = ttf.chrgText.cpMax;
 		count++;
 	}
 	return count;
@@ -1096,7 +1115,7 @@ on_find_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			if (! utils_str_replace_escape(search_data.text, search_data.flags & SCFIND_REGEXP))
 				goto fail;
 		}
-		ui_combo_box_add_to_history(GTK_COMBO_BOX(user_data), search_data.text);
+		ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(user_data), search_data.text, 0);
 
 		switch (response)
 		{
@@ -1219,6 +1238,8 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 		fail:
 		utils_beep();
 		gtk_widget_grab_focus(replace_dlg.find_entry);
+		g_free(find);
+		g_free(replace);
 		return;
 	}
 	if (search_flags_re & SCFIND_REGEXP)
@@ -1234,10 +1255,10 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 			goto fail;
 	}
 
-	ui_combo_box_add_to_history(GTK_COMBO_BOX(
-		gtk_widget_get_parent(replace_dlg.find_entry)), find);
-	ui_combo_box_add_to_history(GTK_COMBO_BOX(
-		gtk_widget_get_parent(replace_dlg.replace_entry)), replace);
+	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(
+		gtk_widget_get_parent(replace_dlg.find_entry)), find, 0);
+	ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(
+		gtk_widget_get_parent(replace_dlg.replace_entry)), replace, 0);
 
 	switch (response)
 	{
@@ -1289,49 +1310,25 @@ on_replace_dialog_response(GtkDialog *dialog, gint response, gpointer user_data)
 }
 
 
-static gboolean
-on_widget_key_pressed_set_focus(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
-{
-	/* catch tabulator key to set the focus in the replace entry instead of
-	 * setting it to the combo box */
-	if (event->keyval == GDK_Tab)
-	{
-		gtk_widget_grab_focus(GTK_WIDGET(user_data));
-		return TRUE;
-	}
-	return FALSE;
-}
-
-
 static GString *get_grep_options(void)
 {
-	gboolean invert = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-					ui_lookup_widget(fif_dlg.dialog, "check_invert")));
-	gboolean case_sens = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-					ui_lookup_widget(fif_dlg.dialog, "check_case")));
-	gboolean whole_word = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-					ui_lookup_widget(fif_dlg.dialog, "check_wholeword")));
-	gboolean recursive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-					ui_lookup_widget(fif_dlg.dialog, "check_recursive")));
-	gboolean extra = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(
-					ui_lookup_widget(fif_dlg.dialog, "check_extra")));
 	GString *gstr = g_string_new("-nHI");	/* line numbers, filenames, ignore binaries */
 
-	if (invert)
+	if (settings.fif_invert_results)
 		g_string_append_c(gstr, 'v');
-	if (! case_sens)
+	if (! settings.fif_case_sensitive)
 		g_string_append_c(gstr, 'i');
-	if (whole_word)
+	if (settings.fif_match_whole_word)
 		g_string_append_c(gstr, 'w');
-	if (recursive)
+	if (settings.fif_recursive)
 		g_string_append_c(gstr, 'r');
 
-	if (settings.fif_mode == FIF_FGREP)
+	if (!settings.fif_regexp)
 		g_string_append_c(gstr, 'F');
-	else if (settings.fif_mode == FIF_EGREP)
+	else
 		g_string_append_c(gstr, 'E');
 
-	if (extra)
+	if (settings.fif_use_extra_options)
 	{
 		g_strstrip(settings.fif_extra_options);
 
@@ -1341,6 +1338,19 @@ static GString *get_grep_options(void)
 			g_string_append(gstr, settings.fif_extra_options);
 		}
 	}
+	g_strstrip(settings.fif_files);
+	if (settings.fif_use_files && *settings.fif_files)
+	{
+		GString *tmp;
+
+		/* put --include= before each pattern */
+		tmp = g_string_new(settings.fif_files);
+		do {} while (utils_string_replace_all(tmp, "  ", " "));
+		g_string_prepend_c(tmp, ' ');
+		utils_string_replace_all(tmp, " ", " --include=");
+		g_string_append(gstr, tmp->str);
+		g_string_free(tmp, TRUE);
+	}
 	return gstr;
 }
 
@@ -1377,8 +1387,9 @@ on_find_in_files_dialog_response(GtkDialog *dialog, gint response,
 
 			if (search_find_in_files(search_text, locale_dir, opts->str, enc))
 			{
-				ui_combo_box_add_to_history(GTK_COMBO_BOX(search_combo), search_text);
-				ui_combo_box_add_to_history(GTK_COMBO_BOX(dir_combo), utf8_dir);
+				ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(search_combo), search_text, 0);
+				ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(fif_dlg.files_combo), NULL, 0);
+				ui_combo_box_add_to_history(GTK_COMBO_BOX_ENTRY(dir_combo), utf8_dir, 0);
 				gtk_widget_hide(fif_dlg.dialog);
 			}
 			g_free(locale_dir);
@@ -1482,8 +1493,7 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
 
 		ui_progress_bar_start(_("Searching..."));
 
-		g_free(msgwindow.find_in_files_dir);
-		msgwindow.find_in_files_dir = g_strdup(dir);
+		msgwin_set_messages_dir(dir);
 		/* we can pass 'enc' without strdup'ing it here because it's a global const string and
 		 * always exits longer than the lifetime of this function */
 		utils_set_up_io_channel(stdout_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
@@ -1509,7 +1519,7 @@ search_find_in_files(const gchar *utf8_search_text, const gchar *dir, const gcha
  * Returns NULL if no files were found, otherwise returned vector should be fully freed. */
 static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 {
-	guint prefix_len, list_len, i, j;
+	guint prefix_len, list_len, i;
 	gchar **argv;
 	GSList *list, *item;
 	GError *error = NULL;
@@ -1524,21 +1534,18 @@ static gchar **search_get_argv(const gchar **argv_prefix, const gchar *dir)
 		g_error_free(error);
 		return NULL;
 	}
-	if (list == NULL) return NULL;
+	if (list == NULL)
+		return NULL;
 
 	argv = g_new(gchar*, prefix_len + list_len + 1);
 
 	for (i = 0; i < prefix_len; i++)
 		argv[i] = g_strdup(argv_prefix[i]);
 
-	item = list;
-	for (j = 0; j < list_len; j++)
-	{
+	foreach_slist(item, list)
 		argv[i++] = item->data;
-		item = g_slist_next(item);
-	}
-	argv[i] = NULL;
 
+	argv[i] = NULL;
 	g_slist_free(list);
 	return argv;
 }
@@ -1951,3 +1958,25 @@ guint search_replace_range(ScintillaObject *sci, struct Sci_TextToFind *ttf,
 }
 
 
+void search_find_again(gboolean change_direction)
+{
+	GeanyDocument *doc = document_get_current();
+
+	g_return_if_fail(doc != NULL);
+
+	if (search_data.text)
+	{
+		gboolean forward = ! search_data.backwards;
+		gint result = document_find_text(doc, search_data.text, search_data.flags,
+			change_direction ? forward : !forward, FALSE, NULL);
+
+		if (result > -1)
+			editor_display_current_line(doc->editor, 0.3F);
+
+		if (search_data.search_bar)
+			ui_set_search_entry_background(
+				toolbar_get_widget_child_by_name("SearchEntry"), (result > -1));
+	}
+}
+
+
