@@ -19,7 +19,7 @@
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *      MA 02110-1301, USA.
  *
- * $Id: stash.c 4795 2010-03-31 16:21:28Z ntrel $
+ * $Id: stash.c 5337 2010-10-28 11:45:14Z ntrel $
  */
 
 /**
@@ -50,30 +50,21 @@
  * property. Macros could be added for common widget properties such as @c GtkExpander:"expanded".
  *
  * @section settings-example Settings Example
+ * Here we have some settings for how to make a cup - whether it should be made of china
+ * and who's going to make it. (Yes, it's a stupid example).
  * @include stash-example.c
  * @note You might want to handle the warning/error conditions differently from above.
  *
  * @section prefs-example GUI Prefs Example
- * For prefs, it's the same as the above example but you add widget prefs instead of e.g.
- * boolean settings.
+ * For prefs, it's the same as the above example but you tell Stash to add widget prefs instead of
+ * just data settings.
  *
  * This example uses lookup strings for widgets as they are more flexible than widget pointers.
- * @code
-gboolean want_handle;
-GtkWidget *parent;
-GtkWidget *my_check_button;
-
-stash_group_add_toggle_button(group, &want_handle, "handle", TRUE, "check_handle");
-...
-gtk_container_add(GTK_CONTAINER(parent), my_check_button);
-ui_hookup_widget(parent, my_check_button, "check_handle");
-...
-stash_group_display(group, parent);
-* @endcode
-* Now let the user manipulate widgets. To synchronize the Stash settings afterwards, call:
-* @code
-stash_group_update(group, parent);
- * @endcode
+ * Code to load and save the settings is omitted - see the first example instead.
+ *
+ * Here we show a dialog with a toggle button for whether the cup should have a handle.
+ * @include stash-gui-example.c
+ * @note This example should also work for other widget containers besides dialogs, e.g. popup menus.
  */
 
 /* Implementation Note
@@ -86,24 +77,24 @@ stash_group_update(group, parent);
  *
  * Usually the prefs code isn't what user code will spend most of its time doing, so this
  * should be efficient enough. But, if desired we could add a stash_group_set_size() function
- * to reduce reallocation.
+ * to reduce reallocation (or perhaps use a different container).
  *
- * TODO: Maybe using GSlice chunks with an extra 'next' pointer would be more (memory) efficient.
+ * Note: Maybe using GSlice chunks with an extra 'next' pointer would be more efficient.
  */
 
 
-#include <gtk/gtk.h>
+#include "geany.h"		/* necessary for utils.h, otherwise use gtk/gtk.h */
+#include "utils.h"		/* only for foreach_*, utils_get_setting_*(). Stash should not depend on Geany. */
 
 #include "stash.h"
-#include "utils.h"		/* only for foreach_*, utils_get_setting_*(). Stash should not depend on Geany. */
 
 
 struct StashPref
 {
 	GType setting_type;			/* e.g. G_TYPE_INT */
-	gpointer setting;
+	gpointer setting;			/* Address of a variable */
 	const gchar *key_name;
-	gpointer default_value;
+	gpointer default_value;		/* Default value, e.g. (gpointer)1 */
 	GType widget_type;			/* e.g. GTK_TYPE_TOGGLE_BUTTON */
 	StashWidgetID widget_id;	/* (GtkWidget*) or (gchar*) */
 	gpointer fields;			/* extra fields */
@@ -163,7 +154,7 @@ static void handle_boolean_setting(StashGroup *group, StashPref *se,
 static void handle_integer_setting(StashGroup *group, StashPref *se,
 		GKeyFile *config, SettingAction action)
 {
-	gboolean *setting = se->setting;
+	gint *setting = se->setting;
 
 	switch (action)
 	{
@@ -233,15 +224,21 @@ static void keyfile_action(SettingAction action, StashGroup *group, GKeyFile *ke
 
 	foreach_array(StashPref, entry, group->entries)
 	{
-		/* don't overwrite write_once prefs */
-		if (group->write_once && action == SETTING_WRITE &&
-			g_key_file_has_key(keyfile, group->name, entry->key_name, NULL))
-			continue;
+		gpointer tmp = entry->setting;
+
 		/* don't override settings with default values */
 		if (!group->use_defaults && action == SETTING_READ &&
 			!g_key_file_has_key(keyfile, group->name, entry->key_name, NULL))
 			continue;
 
+		/* don't overwrite write_once prefs */
+		if (group->write_once && action == SETTING_WRITE)
+		{
+			if (g_key_file_has_key(keyfile, group->name, entry->key_name, NULL))
+				continue;
+			/* We temporarily use the default value for writing */
+			entry->setting = &entry->default_value;
+		}
 		switch (entry->setting_type)
 		{
 			case G_TYPE_BOOLEAN:
@@ -251,13 +248,15 @@ static void keyfile_action(SettingAction action, StashGroup *group, GKeyFile *ke
 			case G_TYPE_STRING:
 				handle_string_setting(group, entry, keyfile, action); break;
 			default:
-				/* G_TYPE_STRV is not a constant */
+				/* Note: G_TYPE_STRV is not a constant, can't use case label */
 				if (entry->setting_type == G_TYPE_STRV)
 					handle_strv_setting(group, entry, keyfile, action);
 				else
 					g_warning("Unhandled type for %s::%s in %s()!", group->name, entry->key_name,
 						G_STRFUNC);
 		}
+		if (group->write_once && action == SETTING_WRITE)
+			entry->setting = tmp;
 	}
 }
 
