@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: symbols.c 5004 2010-06-06 18:02:49Z eht16 $
+ * $Id: symbols.c 5428 2010-11-23 13:17:42Z ntrel $
  */
 
 /**
@@ -949,7 +949,7 @@ static const gchar *get_symbol_name(GeanyDocument *doc, const TMTag *tag, gboole
 	if (!found_parent && scope &&
 		strpbrk(scope, GEANY_WORDCHARS) == scope)
 	{
-		const gchar *sep = symbols_get_context_separator(FILETYPE_ID(doc->file_type));
+		const gchar *sep = symbols_get_context_separator(doc->file_type->id);
 
 		g_string_append(buffer, scope);
 		g_string_append(buffer, sep);
@@ -1096,7 +1096,7 @@ static GdkPixbuf *get_child_icon(GtkTreeStore *tree_store, GtkTreeIter *parent)
 
 static void add_tree_tag(GeanyDocument *doc, const TMTag *tag, GHashTable *parent_hash)
 {
-	filetype_id ft_id = FILETYPE_ID(doc->file_type);
+	filetype_id ft_id = doc->file_type->id;
 	GtkTreeStore *tree_store = doc->priv->tag_store;
 	GtkTreeIter *parent = NULL;
 
@@ -1172,7 +1172,7 @@ static void add_tree_tags(GeanyDocument *doc, const GList *tags)
 	for (item = tags; item; item = g_list_next(item))
 	{
 		const TMTag *tag = item->data;
-		const gchar *name = get_parent_name(tag, FILETYPE_ID(doc->file_type));
+		const gchar *name = get_parent_name(tag, doc->file_type->id);
 
 		if (name)
 			g_hash_table_insert(parent_hash, (gpointer)name, NULL);
@@ -1441,7 +1441,8 @@ void symbols_show_load_tags_dialog(void)
 
 			if (ft != NULL && symbols_load_global_tags(fname, ft))
 				/* For translators: the first wildcard is the filetype, the second the filename */
-				ui_set_statusbar(TRUE, _("Loaded %s tags file '%s'."), ft->name, utf8_fname);
+				ui_set_statusbar(TRUE, _("Loaded %s tags file '%s'."),
+					filetypes_get_display_name(ft), utf8_fname);
 			else
 				ui_set_statusbar(TRUE, _("Could not load tags file '%s'."), utf8_fname);
 
@@ -1541,7 +1542,7 @@ static void load_user_tags(filetype_id ft_id)
 }
 
 
-gboolean symbols_goto_tag(const gchar *name, gboolean definition)
+static gboolean goto_tag(const gchar *name, gboolean definition)
 {
 	const gint forward_types = tm_tag_prototype_t | tm_tag_externvar_t;
 	gint type;
@@ -1564,18 +1565,38 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 		GeanyDocument *new_doc = document_find_by_real_path(
 			tmtag->atts.entry.file->work_object.file_name);
 
-		/* not found in opened document, should open */
-		if (new_doc == NULL)
+		if (new_doc)
 		{
+			/* If we are already on the tag line, swap definition/declaration */
+			if (new_doc == old_doc &&
+				tmtag->atts.entry.line == (guint)sci_get_current_line(old_doc->editor->sci) + 1)
+			{
+				if (goto_tag(name, !definition))
+					return TRUE;
+			}
+		}
+		else
+		{
+			/* not found in opened document, should open */
 			new_doc = document_open_file(tmtag->atts.entry.file->work_object.file_name, FALSE, NULL, NULL);
 		}
 
 		if (navqueue_goto_line(old_doc, new_doc, tmtag->atts.entry.line))
 			return TRUE;
 	}
+	return FALSE;
+}
+
+
+gboolean symbols_goto_tag(const gchar *name, gboolean definition)
+{
+	if (goto_tag(name, definition))
+		return TRUE;
+
 	/* if we are here, there was no match and we are beeping ;-) */
 	utils_beep();
-	if (type == forward_types)
+
+	if (!definition)
 		ui_set_statusbar(FALSE, _("Forward declaration \"%s\" not found."), name);
 	else
 		ui_set_statusbar(FALSE, _("Definition of \"%s\" not found."), name);
@@ -1587,7 +1608,7 @@ gboolean symbols_goto_tag(const gchar *name, gboolean definition)
 static gint get_function_fold_number(GeanyDocument *doc)
 {
 	/* for Java the functions are always one fold level above the class scope */
-	if (FILETYPE_ID(doc->file_type) == GEANY_FILETYPES_JAVA)
+	if (doc->file_type->id == GEANY_FILETYPES_JAVA)
 		return SC_FOLDLEVELBASE + 1;
 	else
 		return SC_FOLDLEVELBASE;
