@@ -1,8 +1,8 @@
 /*
  *      tools.c - this file is part of Geany, a fast and lightweight IDE
  *
- *      Copyright 2006-2009 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
- *      Copyright 2006-2009 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
+ *      Copyright 2006-2010 Enrico Tröger <enrico(dot)troeger(at)uvena(dot)de>
+ *      Copyright 2006-2010 Nick Treleaven <nick(dot)treleaven(at)btinternet(dot)com>
  *
  *      This program is free software; you can redistribute it and/or modify
  *      it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: tools.c 3763 2009-05-02 17:04:35Z eht16 $
+ * $Id: tools.c 5268 2010-10-01 12:43:32Z ntrel $
  */
 
 /*
@@ -76,7 +76,7 @@ static void cc_add_command(struct cc_dialog *cc, gint idx)
 	entry = gtk_entry_new();
 	if (idx >= 0)
 		gtk_entry_set_text(GTK_ENTRY(entry), ui_prefs.custom_commands[idx]);
-	ui_entry_add_clear_icon(entry);
+	ui_entry_add_clear_icon(GTK_ENTRY(entry));
 	gtk_entry_set_max_length(GTK_ENTRY(entry), 255);
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 30);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
@@ -242,7 +242,12 @@ void tools_execute_custom_command(GeanyDocument *doc, const gchar *command)
 	if (! sci_has_selection(doc->editor->sci))
 		return;
 
-	argv = g_strsplit(command, " ", -1);
+	if (!g_shell_parse_argv(command, NULL, &argv, &error))
+	{
+		ui_set_statusbar(TRUE, _("Custom command failed: %s"), error->message);
+		g_error_free(error);
+		return;
+	}
 	ui_set_statusbar(TRUE, _("Passing data and executing custom command: %s"), command);
 
 	cc_error_occurred = FALSE;
@@ -257,10 +262,10 @@ void tools_execute_custom_command(GeanyDocument *doc, const gchar *command)
 			g_child_watch_add(pid, (GChildWatchFunc) cc_exit_cb, doc);
 
 		/* use GIOChannel to monitor stdout */
-		utils_set_up_io_channel(stdout_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		utils_set_up_io_channel(stdout_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 				FALSE, cc_iofunc, NULL);
 		/* copy program's stderr to Geany's stdout to help error tracking */
-		utils_set_up_io_channel(stderr_fd, G_IO_IN|G_IO_PRI|G_IO_ERR|G_IO_HUP|G_IO_NVAL,
+		utils_set_up_io_channel(stderr_fd, G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP | G_IO_NVAL,
 				FALSE, cc_iofunc_err, (gpointer)command);
 
 		/* get selection */
@@ -343,22 +348,20 @@ static void cc_show_dialog_custom_commands(void)
 	{
 		/* get all hboxes which contain a label and an entry element */
 		GList *children = gtk_container_get_children(GTK_CONTAINER(cc.box));
-		GList *tmp;
+		GList *node, *list;
 		GSList *result_list = NULL;
 		gint j = 0;
 		gint len = 0;
 		gchar **result = NULL;
 		const gchar *text;
 
-		while (children != NULL)
+		foreach_list(node, children)
 		{
 			/* get the contents of each hbox */
-			tmp = gtk_container_get_children(GTK_CONTAINER(children->data));
+			list = gtk_container_get_children(GTK_CONTAINER(node->data));
 
 			/* first element of the list is the label, so skip it and get the entry element */
-			tmp = tmp->next;
-
-			text = gtk_entry_get_text(GTK_ENTRY(tmp->data));
+			text = gtk_entry_get_text(GTK_ENTRY(list->next->data));
 
 			/* if the content of the entry is non-empty, add it to the result array */
 			if (text[0] != '\0')
@@ -366,7 +369,7 @@ static void cc_show_dialog_custom_commands(void)
 				result_list = g_slist_append(result_list, g_strdup(text));
 				len++;
 			}
-			children = children->next;
+			g_list_free(list);
 		}
 		/* create a new null-terminated array but only if there any commands defined */
 		if (len > 0)
@@ -400,7 +403,7 @@ static void cc_on_custom_command_menu_activate(GtkMenuItem *menuitem, gpointer u
 	GeanyDocument *doc = document_get_current();
 	gint i, len;
 	gboolean enable;
-	GList *children;
+	GList *children, *node;
 
 	g_return_if_fail(doc != NULL);
 
@@ -409,15 +412,15 @@ static void cc_on_custom_command_menu_activate(GtkMenuItem *menuitem, gpointer u
 	children = gtk_container_get_children(GTK_CONTAINER(user_data));
 	len = g_list_length(children);
 	i = 0;
-	while (children != NULL)
+	foreach_list(node, children)
 	{
 		if (i == (len - 2))
 			break; /* stop before the last two elements (the seperator and the set entry) */
 
-		gtk_widget_set_sensitive(GTK_WIDGET(children->data), enable);
-		children = children->next;
+		gtk_widget_set_sensitive(GTK_WIDGET(node->data), enable);
 		i++;
 	}
+	g_list_free(children);
 }
 
 
@@ -443,7 +446,7 @@ static void cc_on_custom_command_activate(GtkMenuItem *menuitem, gpointer user_d
 }
 
 
-static void cc_insert_custom_command_items(GtkMenu *me, GtkMenu *mp, gchar *label, gint idx)
+static void cc_insert_custom_command_items(GtkMenu *me, gchar *label, gint idx)
 {
 	GtkWidget *item;
 	gint key_idx = -1;
@@ -467,39 +470,21 @@ static void cc_insert_custom_command_items(GtkMenu *me, GtkMenu *mp, gchar *labe
 	gtk_widget_show(item);
 	g_signal_connect(item, "activate", G_CALLBACK(cc_on_custom_command_activate),
 		GINT_TO_POINTER(idx));
-
-	item = gtk_menu_item_new_with_label(label);
-	if (key_idx != -1)
-		gtk_widget_add_accelerator(item, "activate", gtk_accel_group_new(),
-			kb->key, kb->mods, GTK_ACCEL_VISIBLE);
-	gtk_container_add(GTK_CONTAINER(mp), item);
-	gtk_widget_show(item);
-	g_signal_connect(item, "activate", G_CALLBACK(cc_on_custom_command_activate),
-		GINT_TO_POINTER(idx));
 }
 
 
 void tools_create_insert_custom_command_menu_items(void)
 {
 	GtkMenu *menu_edit = GTK_MENU(ui_lookup_widget(main_widgets.window, "send_selection_to2_menu"));
-	GtkMenu *menu_popup = GTK_MENU(ui_lookup_widget(main_widgets.editor_menu, "send_selection_to1_menu"));
 	GtkWidget *item;
-	GList *me_children;
-	GList *mp_children;
+	GList *me_children, *node;
 	static gboolean signal_set = FALSE;
 
 	/* first clean the menus to be able to rebuild them */
 	me_children = gtk_container_get_children(GTK_CONTAINER(menu_edit));
-	mp_children = gtk_container_get_children(GTK_CONTAINER(menu_popup));
-	while (me_children != NULL)
-	{
-		gtk_widget_destroy(GTK_WIDGET(me_children->data));
-		gtk_widget_destroy(GTK_WIDGET(mp_children->data));
-
-		me_children = me_children->next;
-		mp_children = mp_children->next;
-	}
-
+	foreach_list(node, me_children)
+		gtk_widget_destroy(GTK_WIDGET(node->data));
+	g_list_free(me_children);
 
 	if (ui_prefs.custom_commands == NULL || g_strv_length(ui_prefs.custom_commands) == 0)
 	{
@@ -507,10 +492,6 @@ void tools_create_insert_custom_command_menu_items(void)
 		gtk_container_add(GTK_CONTAINER(menu_edit), item);
 		gtk_widget_set_sensitive(item, FALSE);
 		gtk_widget_show(item);
-		item = gtk_menu_item_new_with_label(_("No custom commands defined."));
-		gtk_container_add(GTK_CONTAINER(menu_popup), item);
-		gtk_widget_set_sensitive(item, FALSE);
-		gtk_widget_show(item);
 	}
 	else
 	{
@@ -521,7 +502,7 @@ void tools_create_insert_custom_command_menu_items(void)
 		{
 			if (ui_prefs.custom_commands[i][0] != '\0') /* skip empty fields */
 			{
-				cc_insert_custom_command_items(menu_edit, menu_popup, ui_prefs.custom_commands[i], idx);
+				cc_insert_custom_command_items(menu_edit, ui_prefs.custom_commands[i], idx);
 				idx++;
 			}
 		}
@@ -531,16 +512,11 @@ void tools_create_insert_custom_command_menu_items(void)
 	item = gtk_separator_menu_item_new();
 	gtk_container_add(GTK_CONTAINER(menu_edit), item);
 	gtk_widget_show(item);
-	item = gtk_separator_menu_item_new();
-	gtk_container_add(GTK_CONTAINER(menu_popup), item);
-	gtk_widget_show(item);
 
-	cc_insert_custom_command_items(menu_edit, menu_popup, _("Set Custom Commands"), -1);
+	cc_insert_custom_command_items(menu_edit, _("Set Custom Commands"), -1);
 
 	if (! signal_set)
 	{
-		g_signal_connect(ui_lookup_widget(main_widgets.editor_menu, "send_selection_to1"),
-					"activate", G_CALLBACK(cc_on_custom_command_menu_activate), menu_popup);
 		g_signal_connect(ui_lookup_widget(main_widgets.window, "send_selection_to2"),
 					"activate", G_CALLBACK(cc_on_custom_command_menu_activate), menu_edit);
 		signal_set = TRUE;
