@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: ui_utils.c 5149 2010-08-12 12:51:51Z ntrel $
+ * $Id: ui_utils.c 5383 2010-11-08 13:13:33Z ntrel $
  */
 
 /** @file ui_utils.h
@@ -51,6 +51,9 @@
 #include "symbols.h"
 #include "toolbar.h"
 #include "geanymenubuttonaction.h"
+#include "main.h"
+#include "stash.h"
+#include "keyfile.h"
 
 
 GeanyInterfacePrefs	interface_prefs;
@@ -72,8 +75,6 @@ static struct
 	GtkWidget	*undo_items[3];
 	GtkWidget	*save_buttons[4];
 	GtkWidget	*config_files_menu;
-	GtkWidget	*commands_menu;
-	GtkWidget	*format_menu;
 }
 widgets;
 
@@ -158,6 +159,117 @@ void ui_set_statusbar(gboolean log, const gchar *format, ...)
 }
 
 
+static gchar *statusbar_template = NULL;
+
+/* note: some comments below are for translators */
+static void add_statusbar_statistics(GString *stats_str,
+		GeanyDocument *doc, guint line, guint col)
+{
+	const gchar *cur_tag;
+	const gchar *fmt;
+	const gchar *expos;	/* % expansion position */
+	const gchar sp[] = "      ";
+
+	fmt = NZV(statusbar_template) ? statusbar_template :
+		/* Status bar statistics: col = column, sel = selection. */
+		_("line: %l / %L\t col: %c\t sel: %s\t %w      %t      %m"
+		"mode: %M      encoding: %e      filetype: %f      scope: %S");
+
+	g_string_assign(stats_str, "");
+	while ((expos = strchr(fmt, '%')) != NULL)
+	{
+		/* append leading text before % char */
+		g_string_append_len(stats_str, fmt, expos - fmt);
+
+		switch (*++expos)
+		{
+			case 'l':
+				g_string_append_printf(stats_str, "%d", line + 1);
+				break;
+			case 'L':
+				g_string_append_printf(stats_str, "%d",
+					sci_get_line_count(doc->editor->sci));
+				break;
+			case 'c':
+				g_string_append_printf(stats_str, "%d", col);
+				break;
+			case 'C':
+				g_string_append_printf(stats_str, "%d", col + 1);
+				break;
+			case 's':
+				g_string_append_printf(stats_str, "%d",
+					sci_get_selected_text_length(doc->editor->sci) - 1);
+				break;
+			case 'w':
+				/* RO = read-only */
+				g_string_append(stats_str, (doc->readonly) ? _("RO ") :
+					/* OVR = overwrite/overtype, INS = insert */
+					(sci_get_overtype(doc->editor->sci) ? _("OVR") : _("INS")));
+				break;
+			case 'r':
+				if (doc->readonly)
+				{
+					g_string_append(stats_str, _("RO "));	/* RO = read-only */
+					g_string_append(stats_str, sp + 1);
+				}
+				break;
+			case 't':
+			{
+				switch (editor_get_indent_prefs(doc->editor)->type)
+				{
+					case GEANY_INDENT_TYPE_TABS:
+						g_string_append(stats_str, _("TAB"));
+						break;
+					case GEANY_INDENT_TYPE_SPACES:	/* SP = space */
+						g_string_append(stats_str, _("SP"));
+						break;
+					case GEANY_INDENT_TYPE_BOTH:	/* T/S = tabs and spaces */
+						g_string_append(stats_str, _("T/S"));
+						break;
+				}
+				break;
+			}
+			case 'm':
+				if (doc->changed)
+				{
+					g_string_append(stats_str, _("MOD"));	/* MOD = modified */
+					g_string_append(stats_str, sp);
+				}
+				break;
+			case 'M':
+				g_string_append(stats_str, editor_get_eol_char_name(doc->editor));
+				break;
+			case 'e':
+				g_string_append(stats_str,
+					doc->encoding ? doc->encoding : _("unknown"));
+				if (encodings_is_unicode_charset(doc->encoding) && (doc->has_bom))
+				{
+					g_string_append_c(stats_str, ' ');
+					g_string_append(stats_str, _("(with BOM)"));	/* BOM = byte order mark */
+				}
+				break;
+			case 'f':
+				g_string_append(stats_str, filetypes_get_display_name(doc->file_type));
+				break;
+			case 'S':
+				symbols_get_current_function(doc, &cur_tag);
+				g_string_append(stats_str, cur_tag);
+				break;
+			default:
+				g_string_append_len(stats_str, expos, 1);
+		}
+
+		/* skip past %c chars */
+		if (*expos)
+			fmt = expos + 1;
+		else
+			break;
+	}
+	/* add any remaining text */
+	g_string_append(stats_str, fmt);
+}
+
+
 /* updates the status bar document statistics */
 void ui_update_statusbar(GeanyDocument *doc, gint pos)
 {
@@ -170,10 +282,7 @@ void ui_update_statusbar(GeanyDocument *doc, gint pos)
 	if (doc != NULL)
 	{
 		static GString *stats_str = NULL;
-		const gchar sp[] = "      ";
 		guint line, col;
-		const gchar *cur_tag;
-		gchar *filetype_name = doc->file_type->name;
 
 		if (G_UNLIKELY(stats_str == NULL))
 			stats_str = g_string_sized_new(120);
@@ -190,59 +299,17 @@ void ui_update_statusbar(GeanyDocument *doc, gint pos)
 		else
 			col = 0;
 
-		/* Status bar statistics: col = column, sel = selection. */
-		g_string_printf(stats_str, _("line: %d / %d\t col: %d\t sel: %d\t "),
-			(line + 1), sci_get_line_count(doc->editor->sci), col,
-			sci_get_selected_text_length(doc->editor->sci) - 1);
-
-		g_string_append(stats_str,
-			/* RO = read-only */
-			(doc->readonly) ? _("RO ") :
-				/* OVR = overwrite/overtype, INS = insert */
-				(sci_get_overtype(doc->editor->sci) ? _("OVR") : _("INS")));
-		g_string_append(stats_str, sp);
-
-		switch (editor_get_indent_prefs(doc->editor)->type)
-		{
-			case GEANY_INDENT_TYPE_TABS:
-				g_string_append(stats_str, _("TAB"));
-				break;
-			case GEANY_INDENT_TYPE_SPACES:
-				g_string_append(stats_str, _("SP"));	/* SP = space */
-				break;
-			case GEANY_INDENT_TYPE_BOTH:
-				g_string_append(stats_str, _("T/S"));	/* T/S = tabs and spaces */
-				break;
-		}
-		g_string_append(stats_str, sp);
-		g_string_append_printf(stats_str, _("mode: %s"),
-			editor_get_eol_char_name(doc->editor));
-		g_string_append(stats_str, sp);
-		g_string_append_printf(stats_str, _("encoding: %s %s"),
-			(doc->encoding) ? doc->encoding : _("unknown"),
-			(encodings_is_unicode_charset(doc->encoding)) ?
-				/* BOM = byte order mark */
-				((doc->has_bom) ? _("(with BOM)") : "") : "");
-		g_string_append(stats_str, sp);
-		g_string_append_printf(stats_str, _("filetype: %s"), filetype_name);
-		g_string_append(stats_str, sp);
-		if (doc->changed)
-		{
-			g_string_append(stats_str, _("MOD"));	/* MOD = modified */
-			g_string_append(stats_str, sp);
-		}
-
-		symbols_get_current_function(doc, &cur_tag);
-		g_string_append_printf(stats_str, _("scope: %s"),
-			cur_tag);
+		add_statusbar_statistics(stats_str, doc, line, col);
 
 #ifdef GEANY_DEBUG
+	{
+		const gchar sp[] = "      ";
 		g_string_append(stats_str, sp);
 		g_string_append_printf(stats_str, "pos: %d", pos);
 		g_string_append(stats_str, sp);
 		g_string_append_printf(stats_str, "style: %d", sci_get_style_at(doc->editor->sci, pos));
+	}
 #endif
-
 		/* can be overridden by status messages */
 		set_statusbar(stats_str->str, TRUE);
 	}
@@ -290,6 +357,10 @@ void ui_set_window_title(GeanyDocument *doc)
 		g_string_append(str, "] - ");
 	}
 	g_string_append(str, "Geany");
+	if (cl_options.new_instance)
+	{
+		g_string_append(str, _(" (new instance)"));
+	}
 	gtk_window_set_title(GTK_WINDOW(main_widgets.window), str->str);
 	g_string_free(str, TRUE);
 }
@@ -572,15 +643,13 @@ void ui_create_insert_date_menu_items(void)
 	gtk_container_add(GTK_CONTAINER(menu_edit), item);
 	gtk_widget_show(item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_menu_insert_date_activate), str);
-	g_object_set_data_full(G_OBJECT(main_widgets.window),
-		"insert_date_custom1", g_object_ref(item), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(main_widgets.window, item, "insert_date_custom1");
 
 	item = gtk_menu_item_new_with_mnemonic(str);
 	gtk_container_add(GTK_CONTAINER(menu_popup), item);
 	gtk_widget_show(item);
 	g_signal_connect(item, "activate", G_CALLBACK(on_insert_date_activate), str);
-	g_object_set_data_full(G_OBJECT(main_widgets.editor_menu),
-		"insert_date_custom2", g_object_ref(item), (GDestroyNotify)g_object_unref);
+	ui_hookup_widget(main_widgets.editor_menu, item, "insert_date_custom2");
 
 	insert_date_items(menu_edit, menu_popup, _("_Set Custom Date Format"));
 }
@@ -661,7 +730,11 @@ static void init_document_widgets(void)
 	add_doc_widget("replace1");
 	add_doc_widget("find_nextsel1");
 	add_doc_widget("find_prevsel1");
+	add_doc_widget("find_usage1");
+	add_doc_widget("find_document_usage1");
 	add_doc_widget("go_to_line1");
+	add_doc_widget("goto_tag_definition1");
+	add_doc_widget("goto_tag_declaration1");
 	add_doc_toolitem("Close");
 	add_doc_toolitem("CloseAll");
 	add_doc_toolitem("Search");
@@ -811,6 +884,16 @@ void ui_document_show_hide(GeanyDocument *doc)
 	item = ui_lookup_widget(main_widgets.window, widget_name);
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
 
+	if (iprefs->width >= 1 && iprefs->width <= 8)
+	{
+		gchar *name;
+
+		name = g_strdup_printf("indent_width_%d", iprefs->width);
+		item = ui_lookup_widget(main_widgets.window, name);
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), TRUE);
+		g_free(name);
+	}
+
 	gtk_check_menu_item_set_active(
 			GTK_CHECK_MENU_ITEM(ui_lookup_widget(main_widgets.window, "set_file_readonly1")),
 			doc->readonly);
@@ -1379,26 +1462,53 @@ void ui_hbutton_box_copy_layout(GtkButtonBox *master, GtkButtonBox *copy)
 }
 
 
-/* Prepends the active text to the drop down list, unless the first element in
- * the list is identical, ensuring there are <= history_len elements. */
-void ui_combo_box_add_to_history(GtkComboBox *combo, const gchar *text)
+static gboolean tree_model_find_text(GtkTreeModel *model,
+		GtkTreeIter *iter, gint column, const gchar *text)
 {
-	const gint history_len = 30;
+	gchar *combo_text;
+	gboolean found = FALSE;
+
+	if (gtk_tree_model_get_iter_first(model, iter))
+	{
+		do
+		{
+			gtk_tree_model_get(model, iter, 0, &combo_text, -1);
+			found = utils_str_equal(combo_text, text);
+			g_free(combo_text);
+
+			if (found)
+				return TRUE;
+		}
+		while (gtk_tree_model_iter_next(model, iter));
+	}
+	return FALSE;
+}
+
+
+/** Prepends @a text to the drop down list, removing a duplicate element in
+ * the list if found. Also ensures there are <= @a history_len elements.
+ * @param combo_entry .
+ * @param text Text to add, or @c NULL for current entry text.
+ * @param history_len Max number of items, or @c 0 for default. */
+void ui_combo_box_add_to_history(GtkComboBoxEntry *combo_entry,
+		const gchar *text, gint history_len)
+{
+	GtkComboBox *combo = GTK_COMBO_BOX(combo_entry);
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gchar *combo_text;
-	gboolean equal = FALSE;
 	GtkTreePath *path;
 
-	model = gtk_combo_box_get_model(combo);
-	if (gtk_tree_model_get_iter_first(model, &iter))
-	{
-		gtk_tree_model_get(model, &iter, 0, &combo_text, -1);
-		equal = utils_str_equal(combo_text, text);
-		g_free(combo_text);
-	}
-	if (equal) return;	/* don't prepend duplicate */
+	if (history_len <= 0)
+		history_len = 10;
+	if (!text)
+		text = gtk_entry_get_text(GTK_ENTRY(GTK_BIN(combo)->child));
 
+	model = gtk_combo_box_get_model(combo);
+
+	if (tree_model_find_text(model, &iter, 0, text))
+	{
+		gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
+	}
 	gtk_combo_box_prepend_text(combo, text);
 
 	/* limit history */
@@ -1417,21 +1527,9 @@ void ui_combo_box_prepend_text_once(GtkComboBox *combo, const gchar *text)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	gchar *combo_text;
-	gboolean found = FALSE;
 
 	model = gtk_combo_box_get_model(combo);
-	if (gtk_tree_model_get_iter_first(model, &iter))
-	{
-		do
-		{
-			gtk_tree_model_get(model, &iter, 0, &combo_text, -1);
-			found = utils_str_equal(combo_text, text);
-			g_free(combo_text);
-		}
-		while (!found && gtk_tree_model_iter_next(model, &iter));
-	}
-	if (found)
+	if (tree_model_find_text(model, &iter, 0, text))
 		return;	/* don't prepend duplicate */
 
 	gtk_combo_box_prepend_text(combo, text);
@@ -1595,8 +1693,7 @@ void ui_setup_open_button_callback(GtkWidget *open_btn, const gchar *title,
 		g_object_set_data_full(G_OBJECT(open_btn), "title", g_strdup(title),
 				(GDestroyNotify) g_free);
 	g_object_set_data(G_OBJECT(open_btn), "action", (gpointer) action);
-	g_object_set_data_full(G_OBJECT(open_btn), "entry",
-		g_object_ref(path_entry), (GDestroyNotify) g_object_unref);
+	ui_hookup_widget(open_btn, path_entry, "entry");
 	g_signal_connect(open_btn, "clicked", G_CALLBACK(ui_path_box_open_clicked), open_btn);
 }
 
@@ -1886,36 +1983,72 @@ static void ui_menu_move(GtkWidget *menu, GtkWidget *old, GtkWidget *new)
 }
 
 
-static void on_editor_menu_show(GtkWidget *item)
+typedef struct GeanySharedMenu
 {
-	GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, "commands1");
-	GtkWidget *bar = ui_lookup_widget(main_widgets.window, "commands2");
+	gchar *menu;
+	gchar *menubar_item;
+	gchar *popup_item;
+}
+GeanySharedMenu;
 
-	ui_menu_move(widgets.commands_menu, bar, popup);
+#define foreach_menu(item, array) \
+	for (item = array; item->menu; item++)
 
-	popup = ui_lookup_widget(main_widgets.editor_menu, "menu_format2");
-	bar = ui_lookup_widget(main_widgets.window, "menu_format1");
-	ui_menu_move(widgets.format_menu, bar, popup);
+static void on_editor_menu_show(GtkWidget *widget, GeanySharedMenu *items)
+{
+	GeanySharedMenu *item;
+
+	foreach_menu(item, items)
+	{
+		GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, item->popup_item);
+		GtkWidget *bar = ui_lookup_widget(main_widgets.window, item->menubar_item);
+		GtkWidget *menu = ui_lookup_widget(main_widgets.window, item->menu);
+
+		ui_menu_move(menu, bar, popup);
+	}
 }
 
 
-static void on_editor_menu_hide(GtkWidget *item)
+static void on_editor_menu_hide(GtkWidget *widget, GeanySharedMenu *items)
 {
-	GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, "commands1");
-	GtkWidget *bar = ui_lookup_widget(main_widgets.window, "commands2");
+	GeanySharedMenu *item;
 
-	ui_menu_move(widgets.commands_menu, popup, bar);
+	foreach_menu(item, items)
+	{
+		GtkWidget *popup = ui_lookup_widget(main_widgets.editor_menu, item->popup_item);
+		GtkWidget *bar = ui_lookup_widget(main_widgets.window, item->menubar_item);
+		GtkWidget *menu = ui_lookup_widget(main_widgets.window, item->menu);
 
-	popup = ui_lookup_widget(main_widgets.editor_menu, "menu_format2");
-	bar = ui_lookup_widget(main_widgets.window, "menu_format1");
-	ui_menu_move(widgets.format_menu, popup, bar);
+		ui_menu_move(menu, popup, bar);
+	}
+}
+
+
+/* Currently ui_init() is called before keyfile.c stash group code is initialized,
+ * so this is called after that's done. */
+void ui_init_prefs(void)
+{
+	StashGroup *group = stash_group_new(PACKAGE);
+
+	/* hidden prefs (don't overwrite them so users can edit them manually) */
+	configuration_add_pref_group(group, FALSE);
+	stash_group_set_write_once(group, TRUE);
+
+	stash_group_add_boolean(group, &interface_prefs.show_symbol_list_expanders,
+		"show_symbol_list_expanders", TRUE);
+	stash_group_add_boolean(group, &interface_prefs.compiler_tab_autoscroll,
+		"compiler_tab_autoscroll", TRUE);
+	stash_group_add_boolean(group, &ui_prefs.allow_always_save,
+		"allow_always_save", FALSE);
+	stash_group_add_string(group, &statusbar_template,
+		"statusbar_template", "");
+	stash_group_add_boolean(group, &ui_prefs.new_document_after_close,
+		"new_document_after_close", FALSE);
 }
 
 
 void ui_init(void)
 {
-	GtkWidget *item;
-
 	init_recent_files();
 
 	ui_widgets.statusbar = ui_lookup_widget(main_widgets.window, "statusbar");
@@ -1923,10 +2056,12 @@ void ui_init(void)
 
 	main_widgets.progressbar = progress_bar_create();
 
+	/* current word sensitive items */
 	widgets.popup_goto_items[0] = ui_lookup_widget(main_widgets.editor_menu, "goto_tag_definition1");
-	widgets.popup_goto_items[1] = ui_lookup_widget(main_widgets.editor_menu, "goto_tag_declaration1");
+	widgets.popup_goto_items[1] = ui_lookup_widget(main_widgets.editor_menu, "context_action1");
 	widgets.popup_goto_items[2] = ui_lookup_widget(main_widgets.editor_menu, "find_usage1");
 	widgets.popup_goto_items[3] = ui_lookup_widget(main_widgets.editor_menu, "find_document_usage1");
+
 	widgets.popup_copy_items[0] = ui_lookup_widget(main_widgets.editor_menu, "cut1");
 	widgets.popup_copy_items[1] = ui_lookup_widget(main_widgets.editor_menu, "copy1");
 	widgets.popup_copy_items[2] = ui_lookup_widget(main_widgets.editor_menu, "delete1");
@@ -1942,15 +2077,20 @@ void ui_init(void)
 	widgets.undo_items[0] = ui_lookup_widget(main_widgets.editor_menu, "undo1");
 	widgets.undo_items[1] = ui_lookup_widget(main_widgets.window, "menu_undo2");
 
-	item = ui_lookup_widget(main_widgets.window, "menu_format1");
-	widgets.format_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
-	item = ui_lookup_widget(main_widgets.window, "commands2");
-	widgets.commands_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
+	/* reparent context submenus as needed */
+	{
+		GeanySharedMenu arr[] = {
+			{"commands2_menu", "commands2", "commands1"},
+			{"menu_format1_menu", "menu_format1", "menu_format2"},
+			{"more1_menu", "more1", "search2"},
+			{NULL, NULL, NULL}
+		};
+		static GeanySharedMenu items[G_N_ELEMENTS(arr)];
 
-	/* reparent edit submenus as needed */
-	item = main_widgets.editor_menu;
-	g_signal_connect(item, "show", G_CALLBACK(on_editor_menu_show), NULL);
-	g_signal_connect(item, "hide", G_CALLBACK(on_editor_menu_hide), NULL);
+		memcpy(items, arr, sizeof(arr));
+		g_signal_connect(main_widgets.editor_menu, "show", G_CALLBACK(on_editor_menu_show), items);
+		g_signal_connect(main_widgets.editor_menu, "hide", G_CALLBACK(on_editor_menu_hide), items);
+	}
 
 	ui_init_toolbar_widgets();
 	init_document_widgets();
@@ -1958,6 +2098,12 @@ void ui_init(void)
 }
 
 
+void ui_finalize(void)
+{
+	g_free(statusbar_template);
+}
+
+
 static void auto_separator_update(GeanyAutoSeparator *autosep)
 {
 	g_return_if_fail(autosep->ref_count >= 0);
@@ -2181,8 +2327,7 @@ void ui_menu_sort_by_label(GtkMenu *menu)
 }
 
 
-/* return value is for macros */
-GtkWidget *ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
+void ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
 {
 	va_list a;
 	gchar *text;
@@ -2194,7 +2339,6 @@ GtkWidget *ui_label_set_markup(GtkLabel *label, const gchar *format, ...)
 	gtk_label_set_text(label, text);
 	gtk_label_set_use_markup(label, TRUE);
 	g_free(text);
-	return GTK_WIDGET(label);
 }
 
 
@@ -2204,7 +2348,8 @@ GtkWidget *ui_label_new_bold(const gchar *text)
 	gchar *label_text;
 
 	label_text = g_markup_escape_text(text, -1);
-	label = ui_label_set_markup(GTK_LABEL(gtk_label_new(NULL)), "<b>%s</b>", label_text);
+	label = gtk_label_new(NULL);
+	ui_label_set_markup(GTK_LABEL(label), "<b>%s</b>", label_text);
 	g_free(label_text);
 	return label;
 }
@@ -2219,7 +2364,7 @@ GtkWidget *ui_label_new_bold(const gchar *text)
  * @since 0.19 */
 void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback callback)
 {
-	GtkWidget *menu_item, *menu_item_label;
+	GtkWidget *menu_item, *menu_item_label, *image;
 	const GdkColor *color;
 	GeanyDocument *doc;
 	guint i, len;
@@ -2233,7 +2378,10 @@ void ui_menu_add_document_items(GtkMenu *menu, GeanyDocument *active, GCallback
 			continue;
 
 		base_name = g_path_get_basename(DOC_FILENAME(doc));
-		menu_item = gtk_menu_item_new_with_label(base_name);
+		menu_item = gtk_image_menu_item_new_with_label(base_name);
+		image = gtk_image_new_from_pixbuf(doc->file_type->icon);
+		gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menu_item), image);
+
 		gtk_widget_show(menu_item);
 		gtk_container_add(GTK_CONTAINER(menu), menu_item);
 		g_signal_connect(menu_item, "activate", callback, doc);
@@ -2291,20 +2439,72 @@ gint ui_get_gtk_settings_integer(const gchar *property_name, gint default_value)
 void ui_editable_insert_text_callback(GtkEditable *editable, gchar *new_text,
 									  gint new_text_len, gint *position, gpointer data)
 {
-	gboolean stop_signal = FALSE;
-	const gchar c = *new_text;
+	gboolean first = position != NULL && *position == 0;
+	gint i;
 
-	/* allow inserting '+' and '-' as the first character */
-	if (position != NULL && *position == 0)
+	if (new_text_len == -1)
+		new_text_len = strlen(new_text);
+
+	for (i = 0; i < new_text_len; i++, new_text++)
 	{
-		if (c != '+' && c != '-' && ! isdigit(c))
-			stop_signal = TRUE;
+		if ((!first || !strchr("+-", *new_text)) && !isdigit(*new_text))
+		{
+			g_signal_stop_emission_by_name(editable, "insert-text");
+			break;
+		}
+		first = FALSE;
 	}
-	/* don't insert any text when it is not a digit */
-	else if (! isdigit(c))
-		stop_signal = TRUE;
+}
 
-	if (stop_signal)
-		g_signal_stop_emission_by_name(editable, "insert-text");
+
+/* gets the icon that applies to a particular MIME type */
+GdkPixbuf *ui_get_mime_icon(const gchar *mime_type, GtkIconSize size)
+{
+	GdkPixbuf *icon = NULL;
+#if defined(HAVE_GIO) && GLIB_CHECK_VERSION(2, 18, 0)
+	gchar *ctype;
+	GIcon *gicon;
+	GtkIconInfo *info;
+	GtkIconTheme *theme;
+	gint real_size;
+
+	if (!gtk_icon_size_lookup(size, &real_size, NULL))
+	{
+		g_return_val_if_reached(NULL);
+	}
+	ctype = g_content_type_from_mime_type(mime_type);
+	if (ctype != NULL)
+	{
+		gicon = g_content_type_get_icon(ctype);
+		theme = gtk_icon_theme_get_default();
+		info = gtk_icon_theme_lookup_by_gicon(theme, gicon, real_size, 0);
+		g_object_unref(gicon);
+		g_free(ctype);
+
+		if (info != NULL)
+		{
+			icon = gtk_icon_info_load_icon(info, NULL);
+			gtk_icon_info_free(info);
+		}
+	}
+#endif
+	/* fallback for builds without GIO or if icon lookup failed, like it might happen on Windows */
+	if (icon == NULL)
+	{
+		const gchar *stock_id = GTK_STOCK_FILE;
+		GtkIconSet *icon_set;
+
+		if (strstr(mime_type, "directory"))
+			stock_id = GTK_STOCK_DIRECTORY;
+
+		icon_set = gtk_icon_factory_lookup_default(stock_id);
+		if (icon_set)
+		{
+			icon = gtk_icon_set_render_icon(icon_set, gtk_widget_get_default_style(),
+				gtk_widget_get_default_direction(),
+				GTK_STATE_NORMAL, size, NULL, NULL);
+		}
+	}
+	return icon;
 }
 
