@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: utils.c 4884 2010-05-08 13:10:33Z eht16 $
+ * $Id: utils.c 5454 2010-11-30 18:25:16Z ntrel $
  */
 
 /*
@@ -128,6 +128,9 @@ gint utils_get_line_endings(const gchar* buffer, glong size)
 	guint cr, lf, crlf, max_mode;
 	gint mode;
 
+	if (size == -1)
+		size = strlen(buffer);
+
 	cr = lf = crlf = 0;
 
 	for (i = 0; i < size ; i++)
@@ -214,15 +217,19 @@ gboolean utils_is_opening_brace(gchar c, gboolean include_angles)
 
 
 /**
- *  Writes the given @a text into a file with @a filename.
- *  If the file doesn't exist, it will be created.
- *  If it already exists, it will be overwritten.
+ * Writes @a text into a file named @a filename.
+ * If the file doesn't exist, it will be created.
+ * If it already exists, it will be overwritten.
  *
- *  @param filename The filename of the file to write, in locale encoding.
- *  @param text The text to write into the file.
+ * @warning You should use @c g_file_set_contents() instead if you don't need
+ * file permissions and other metadata to be preserved, as that always handles
+ * disk exhaustion safely.
  *
- *  @return 0 if the file was successfully written, otherwise the @c errno of the
- *          failed operation is returned.
+ * @param filename The filename of the file to write, in locale encoding.
+ * @param text The text to write into the file.
+ *
+ * @return 0 if the file was successfully written, otherwise the @c errno of the
+ *         failed operation is returned.
  **/
 gint utils_write_file(const gchar *filename, const gchar *text)
 {
@@ -243,43 +250,49 @@ gint utils_write_file(const gchar *filename, const gchar *text)
 	{
 		FILE *fp;
 		gint bytes_written, len;
+		gboolean fail = FALSE;
 
 		if (filename == NULL)
 			return ENOENT;
 
 		len = strlen(text);
+		errno = 0;
 		fp = g_fopen(filename, "w");
-		if (fp != NULL)
+		if (fp == NULL)
+			fail = TRUE;
+		else
 		{
-			bytes_written = fwrite(text, sizeof (gchar), len, fp);
-			fclose(fp);
+			bytes_written = fwrite(text, sizeof(gchar), len, fp);
 
 			if (len != bytes_written)
 			{
+				fail = TRUE;
 				geany_debug(
 					"utils_write_file(): written only %d bytes, had to write %d bytes to %s",
 					bytes_written, len, filename);
-				return EIO;
 			}
+			if (fclose(fp) != 0)
+				fail = TRUE;
 		}
-		else
+		if (fail)
 		{
 			geany_debug("utils_write_file(): could not write to file %s (%s)",
 				filename, g_strerror(errno));
-			return errno;
+			return NVL(errno, EIO);
 		}
 	}
 	return 0;
 }
 
 
-/*
- * (stolen from anjuta and modified)
- * Search backward through size bytes looking for a '<', then return the tag, if any.
- * @return The tag name
+/** Searches backward through @a size bytes looking for a '<', then returns the tag, if any.
+ * @param sel .
+ * @param size .
+ * @return The tag name.
  */
-gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
+gchar *utils_find_open_xml_tag(const gchar sel[], gint size)
 {
+	/* stolen from anjuta and modified */
 	const gchar *begin, *cur;
 
 	if (G_UNLIKELY(size < 3))
@@ -287,10 +300,7 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 		return NULL;
 	}
 	begin = &sel[0];
-	if (check_tag)
-		cur = &sel[size - 3];
-	else
-		cur = &sel[size - 1];
+	cur = &sel[size - 1];
 
 	/* Skip to the character before the closing brace */
 	while (cur > begin)
@@ -309,16 +319,20 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 	{
 		if (*cur == '<')
 			break;
-		else if (! check_tag && *cur == '>')
+		else if (*cur == '>')
 			break;
 		--cur;
 	}
 
 	if (*cur == '<')
 	{
-		GString *result = g_string_sized_new(64);
+		GString *result;
 
 		cur++;
+		if (*cur == '/')
+			return NULL; /* we found a closing tag */
+
+		result = g_string_sized_new(64);
 		while (strchr(":_-.", *cur) || isalnum(*cur))
 		{
 			g_string_append_c(result, *cur);
@@ -331,6 +345,22 @@ gchar *utils_find_open_xml_tag(const gchar sel[], gint size, gboolean check_tag)
 }
 
 
+/* Returns true if specified tag doesn't usually contain any content and can be left unclosed */
+gboolean utils_is_short_html_tag(const gchar *tag_name)
+{
+	return utils_str_equal(tag_name, "br")
+	 || utils_str_equal(tag_name, "hr")
+	 || utils_str_equal(tag_name, "img")
+	 || utils_str_equal(tag_name, "base")
+	 || utils_str_equal(tag_name, "basefont")	/* < or not < */
+	 || utils_str_equal(tag_name, "frame")
+	 || utils_str_equal(tag_name, "input")
+	 || utils_str_equal(tag_name, "link")
+	 || utils_str_equal(tag_name, "area")
+	 || utils_str_equal(tag_name, "meta");
+}
+
+
 const gchar *utils_get_eol_name(gint eol_mode)
 {
 	switch (eol_mode)
@@ -342,6 +372,34 @@ const gchar *utils_get_eol_name(gint eol_mode)
 }
 
 
+const gchar *utils_get_eol_char(gint eol_mode)
+{
+	switch (eol_mode)
+	{
+		case SC_EOL_CRLF: return "\r\n"; break;
+		case SC_EOL_CR: return "\r"; break;
+		default: return "\n"; break;
+	}
+}
+
+
+/* Converts line endings to @a target_eol_mode. */
+void utils_ensure_same_eol_characters(GString *string, gint target_eol_mode)
+{
+	const gchar *eol_str = utils_get_eol_char(target_eol_mode);
+
+	/* first convert data to LF only */
+	utils_string_replace_all(string, "\r\n", "\n");
+	utils_string_replace_all(string, "\r", "\n");
+
+	if (target_eol_mode == SC_EOL_LF)
+		return;
+
+	/* now convert to desired line endings */
+	utils_string_replace_all(string, "\n", eol_str);
+}
+
+
 gboolean utils_atob(const gchar *str)
 {
 	if (G_UNLIKELY(str == NULL))
@@ -1300,6 +1358,8 @@ void utils_free_pointers(gsize arg_count, ...)
 }
 
 
+/* currently unused */
+#if 0
 /* Creates a string array deep copy of a series of non-NULL strings.
  * The first argument is nothing special.
  * The list must be ended with NULL.
@@ -1333,6 +1393,7 @@ gchar **utils_strv_new(const gchar *first, ...)
 	strv[i] = NULL;
 	return strv;
 }
+#endif
 
 
 /**
@@ -1899,7 +1960,7 @@ gchar *utils_get_help_url(const gchar *suffix)
 	g_strdelimit(uri, "\\", '/'); /* replace '\\' by '/' */
 #else
 	skip = 7;
-	uri = g_strconcat("file://", app->docdir, "index.html", NULL);
+	uri = g_strconcat("file://", app->docdir, "/index.html", NULL);
 #endif
 
 	if (! g_file_test(uri + skip, G_FILE_TEST_IS_REGULAR))
@@ -2007,3 +2068,27 @@ gchar **utils_copy_environment(const gchar **exclude_vars, const gchar *first_va
 }
 
 
+/* Joins @a first and @a second into a new string vector, freeing the originals.
+ * The original contents are reused. */
+gchar **utils_strv_join(gchar **first, gchar **second)
+{
+	gchar **strv;
+	gchar **rptr, **wptr;
+
+	if (!first)
+		return second;
+	if (!second)
+		return first;
+
+	strv = g_new0(gchar*, g_strv_length(first) + g_strv_length(second) + 1);
+	wptr = strv;
+
+	foreach_strv(rptr, first)
+		*wptr++ = *rptr;
+	foreach_strv(rptr, second)
+		*wptr++ = *rptr;
+
+	g_free(first);
+	g_free(second);
+	return strv;
+}
