@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * $Id: vte.c 4789 2010-03-30 14:43:40Z ntrel $
+ * $Id: vte.c 5117 2010-08-05 21:45:58Z eht16 $
  */
 
 /*
@@ -114,6 +114,8 @@ struct VteFunctions
 	void (*vte_terminal_set_cursor_blink_mode) (VteTerminal *terminal,
 												VteTerminalCursorBlinkMode mode);
 	void (*vte_terminal_set_cursor_blinks) (VteTerminal *terminal, gboolean blink);
+	void (*vte_terminal_select_all) (VteTerminal *terminal);
+	void (*vte_terminal_set_audible_bell) (VteTerminal *terminal, gboolean is_audible);
 };
 
 
@@ -135,6 +137,7 @@ enum
 {
 	POPUP_COPY,
 	POPUP_PASTE,
+	POPUP_SELECTALL,
 	POPUP_CHANGEPATH,
 	POPUP_RESTARTTERMINAL,
 	POPUP_PREFERENCES,
@@ -429,6 +432,8 @@ static void vte_register_symbols(GModule *mod)
 		/* vte_terminal_set_cursor_blink_mode() is only available since 0.17.1, so if we don't find
 		 * this symbol, we are probably on an older version and use the old API instead */
 		g_module_symbol(mod, "vte_terminal_set_cursor_blinks", (void*)&vf->vte_terminal_set_cursor_blinks);
+	g_module_symbol(mod, "vte_terminal_select_all", (void*)&vf->vte_terminal_select_all);
+	g_module_symbol(mod, "vte_terminal_set_audible_bell", (void*)&vf->vte_terminal_set_audible_bell);
 }
 
 
@@ -445,6 +450,7 @@ void vte_apply_user_settings(void)
 	vf->vte_terminal_set_color_foreground(VTE_TERMINAL(vc->vte), vc->colour_fore);
 	vf->vte_terminal_set_color_bold(VTE_TERMINAL(vc->vte), vc->colour_fore);
 	vf->vte_terminal_set_color_background(VTE_TERMINAL(vc->vte), vc->colour_back);
+	vf->vte_terminal_set_audible_bell(VTE_TERMINAL(vc->vte), prefs.beep_on_errors);
 	vte_set_cursor_blink_mode();
 
 	override_menu_key();
@@ -466,6 +472,11 @@ static void vte_popup_menu_clicked(GtkMenuItem *menuitem, gpointer user_data)
 			vf->vte_terminal_paste_clipboard(VTE_TERMINAL(vc->vte));
 			break;
 		}
+		case POPUP_SELECTALL:
+		{
+			vte_select_all();
+			break;
+		}
 		case POPUP_CHANGEPATH:
 		{
 			GeanyDocument *doc = document_get_current();
@@ -513,6 +524,15 @@ static GtkWidget *vte_create_popup_menu(void)
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
 
+	item = gtk_image_menu_item_new_from_stock("gtk-select-all", NULL);
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+	g_signal_connect(item, "activate", G_CALLBACK(vte_popup_menu_clicked), GINT_TO_POINTER(POPUP_SELECTALL));
+
+	item = gtk_separator_menu_item_new();
+	gtk_widget_show(item);
+	gtk_container_add(GTK_CONTAINER(menu), item);
+
 	item = gtk_image_menu_item_new_with_mnemonic(_("_Set Path From Document"));
 	gtk_widget_show(item);
 	gtk_container_add(GTK_CONTAINER(menu), item);
@@ -854,32 +874,19 @@ void vte_append_preferences_tab(void)
 
 		gtk_box_pack_start(GTK_BOX(vbox), box, FALSE, FALSE, 0);
 
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "font_term",
-				g_object_ref(font_term),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "color_fore",
-				g_object_ref(color_fore),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "color_back",
-				g_object_ref(color_back),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "spin_scrollback",
-				g_object_ref(spin_scrollback),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "entry_shell",
-				g_object_ref(entry_shell),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_scroll_key",
-				g_object_ref(check_scroll_key),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_scroll_out",
-				g_object_ref(check_scroll_out),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_cursor_blinks",
-				g_object_ref(check_cursor_blinks),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_enable_bash_keys",
-				g_object_ref(check_enable_bash_keys),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_ignore_menu_key",
-				g_object_ref(check_ignore_menu_key),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_follow_path",
-				g_object_ref(check_follow_path),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_run_in_vte",
-				g_object_ref(check_run_in_vte),	(GDestroyNotify) g_object_unref);
-		g_object_set_data_full(G_OBJECT(ui_widgets.prefs_dialog), "check_skip_script",
-				g_object_ref(check_skip_script),	(GDestroyNotify) g_object_unref);
+		ui_hookup_widget(ui_widgets.prefs_dialog, font_term, "font_term");
+		ui_hookup_widget(ui_widgets.prefs_dialog, color_fore, "color_fore");
+		ui_hookup_widget(ui_widgets.prefs_dialog, color_back, "color_back");
+		ui_hookup_widget(ui_widgets.prefs_dialog, spin_scrollback, "spin_scrollback");
+		ui_hookup_widget(ui_widgets.prefs_dialog, entry_shell, "entry_shell");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_scroll_key, "check_scroll_key");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_scroll_out, "check_scroll_out");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_cursor_blinks, "check_cursor_blinks");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_enable_bash_keys, "check_enable_bash_keys");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_ignore_menu_key, "check_ignore_menu_key");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_follow_path, "check_follow_path");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_run_in_vte, "check_run_in_vte");
+		ui_hookup_widget(ui_widgets.prefs_dialog, check_skip_script, "check_skip_script");
 
 		gtk_widget_show_all(frame);
 
@@ -894,6 +901,13 @@ void vte_append_preferences_tab(void)
 }
 
 
+void vte_select_all(void)
+{
+	if (vf->vte_terminal_select_all != NULL)
+		vf->vte_terminal_select_all(VTE_TERMINAL(vc->vte));
+}
+
+
 void vte_send_selection_to_vte(void)
 {
 	GeanyDocument *doc;
