@@ -18,7 +18,7 @@
  *      along with this program; if not, write to the Free Software
  *      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: vte.h 4518 2010-01-01 22:55:18Z eht16 $
+ * $Id: vte.h 5470 2010-12-07 13:02:46Z ntrel $
  */
 
 
@@ -30,8 +30,8 @@
 
 typedef struct
 {
-	gboolean load_vte;
-	gboolean have_vte;
+	gboolean load_vte;	/* this is the preference, NOT the current instance VTE state */
+	gboolean have_vte;	/* use this field to check if the current instance has VTE */
 	gchar	*lib_vte;
 	gchar	*dir;
 } VteInfo;
@@ -79,6 +79,8 @@ void vte_append_preferences_tab(void);
 
 void vte_send_selection_to_vte(void);
 
+void vte_select_all(void);
+
 #endif
 
 #endif
