@@ -144,6 +144,7 @@ typedef enum eDeclaration
 	DECL_EVENT,
 	DECL_SIGNAL,
 	DECL_FUNCTION,
+	DECL_FUNCTION_TEMPLATE,
 	DECL_IGNORE,		/* non-taggable "declaration" */
 	DECL_INTERFACE,
 	DECL_MODULE,
@@ -414,13 +415,14 @@ static const keywordDesc KeywordTable [] = {
 	{ "delete",         KEYWORD_DELETE,         { 0, 1, 0, 0, 0, 1, 1 } },
 	{ "double",         KEYWORD_DOUBLE,         { 1, 1, 1, 1, 0, 1, 1 } },
 	{ "else",           KEYWORD_ELSE,           { 1, 1, 0, 1, 0, 1, 1 } },
+	{ "ensures",        KEYWORD_ATTRIBUTE,      { 0, 0, 0, 0, 0, 1, 0 } },	/* ignore */
 	{ "enum",           KEYWORD_ENUM,           { 1, 1, 1, 1, 1, 1, 1 } },
 	{ "errordomain",    KEYWORD_ENUM,           { 0, 0, 0, 0, 0, 1, 0 } }, /* errordomain behaves like enum */
 	{ "event",          KEYWORD_EVENT,          { 0, 0, 1, 0, 1, 0, 0 } },
 	{ "explicit",       KEYWORD_EXPLICIT,       { 0, 1, 1, 0, 0, 0, 1 } },
 	{ "extends",        KEYWORD_EXTENDS,        { 0, 0, 0, 1, 1, 0, 0 } },
 	{ "extern",         KEYWORD_EXTERN,         { 1, 1, 1, 0, 1, 1, 0 } },
-	{ "extern",         KEYWORD_NAMESPACE,      { 0, 0, 0, 0, 0, 0, 1 } },	/* hack to ignore extern */
+	{ "extern",         KEYWORD_NAMESPACE,      { 0, 0, 0, 0, 0, 0, 1 } },	/* parse block */
 	{ "final",          KEYWORD_FINAL,          { 0, 0, 0, 1, 0, 0, 1 } },
 	{ "finally",        KEYWORD_FINALLY,        { 0, 0, 0, 0, 0, 1, 1 } },
 	{ "float",          KEYWORD_FLOAT,          { 1, 1, 1, 1, 0, 1, 1 } },
@@ -466,6 +468,7 @@ static const keywordDesc KeywordTable [] = {
 	{ "public",         KEYWORD_PUBLIC,         { 0, 1, 1, 1, 1, 1, 1 } },
 	{ "ref",            KEYWORD_REF,            { 0, 0, 0, 0, 0, 1, 1 } },
 	{ "register",       KEYWORD_REGISTER,       { 1, 1, 0, 0, 0, 0, 0 } },
+	{ "requires",       KEYWORD_ATTRIBUTE,      { 0, 0, 0, 0, 0, 1, 0 } },	/* ignore */
 	{ "return",         KEYWORD_RETURN,         { 1, 1, 1, 1, 0, 1, 1 } },
 	{ "set",            KEYWORD_SET,            { 0, 0, 0, 0, 0, 1, 0 } },
 	{ "shadow",         KEYWORD_SHADOW,         { 0, 0, 0, 0, 1, 0, 0 } },
@@ -493,10 +496,11 @@ static const keywordDesc KeywordTable [] = {
 	{ "uint",           KEYWORD_UINT,           { 0, 0, 1, 0, 0, 1, 1 } },
 	{ "ulong",          KEYWORD_ULONG,          { 0, 0, 1, 0, 0, 1, 1 } },
 	{ "union",          KEYWORD_UNION,          { 1, 1, 0, 0, 0, 0, 1 } },
+	{ "unittest",       KEYWORD_BODY,           { 0, 0, 0, 0, 0, 0, 1 } },	/* ignore */
 	{ "unsigned",       KEYWORD_UNSIGNED,       { 1, 1, 1, 0, 0, 0, 1 } },
 	{ "ushort",         KEYWORD_USHORT,         { 0, 0, 1, 0, 0, 1, 1 } },
 	{ "using",          KEYWORD_USING,          { 0, 1, 1, 0, 0, 1, 0 } },
-	{ "version",        KEYWORD_NAMESPACE,      { 0, 0, 0, 0, 0, 0, 1 } },	/* hack to ignore version */
+	{ "version",        KEYWORD_NAMESPACE,      { 0, 0, 0, 0, 0, 0, 1 } },	/* parse block */
 	{ "virtual",        KEYWORD_VIRTUAL,        { 0, 1, 1, 0, 1, 1, 0 } },
 	{ "void",           KEYWORD_VOID,           { 1, 1, 1, 1, 1, 1, 1 } },
 	{ "volatile",       KEYWORD_VOLATILE,       { 1, 1, 1, 1, 0, 0, 1 } },
@@ -1155,6 +1159,7 @@ static tagType declToTagType (const declType declaration)
 		case DECL_CLASS:	type = TAG_CLASS;		break;
 		case DECL_ENUM:		type = TAG_ENUM;		break;
 		case DECL_FUNCTION:	type = TAG_FUNCTION;	break;
+		case DECL_FUNCTION_TEMPLATE: type = TAG_FUNCTION; break;
 		case DECL_INTERFACE:type = TAG_INTERFACE;	break;
 		case DECL_NAMESPACE:type = TAG_NAMESPACE;	break;
 		case DECL_STRUCT:	type = TAG_STRUCT;		break;
@@ -1741,7 +1746,7 @@ static void readIdentifier (tokenInfo *const token, const int firstChar)
 	vStringTerminate (name);
 	cppUngetc (c);		/* unget non-identifier character */
 
-	/* Vala supports '?' at end of a type (with or without whitspace before) for nullable types */
+	/* Vala supports '?' at end of a type (with or without whitespace before) for nullable types */
 	if (isLanguage (Lang_vala))
 	{
 		c = skipToNonWhite ();
@@ -2181,9 +2186,12 @@ static boolean skipPostArgumentStuff (statementInfo *const st,
 				{
 					readIdentifier (token, c);
 					if (isLanguage(Lang_d))
-					{	/* parse D contracts */
+					{
 						switch (token->keyword)
 						{
+							/* template constraint */
+							case KEYWORD_IF:
+							/* contracts */
 							case KEYWORD_IN:
 							case KEYWORD_OUT:
 							case KEYWORD_BODY:
@@ -2277,6 +2285,39 @@ static void skipJavaThrows (statementInfo *const st)
 	setToken (st, TOKEN_NONE);
 }
 
+static void skipValaPostParens (statementInfo *const st)
+{
+	tokenInfo *const token = activeToken (st);
+	int c = skipToNonWhite ();
+
+	while (isident1 (c))
+	{
+		readIdentifier (token, c);
+		if (token->keyword == KEYWORD_ATTRIBUTE)
+		{
+			/* parse contracts */
+			skipParens ();
+			c = skipToNonWhite ();
+		}
+		else if (token->keyword == KEYWORD_THROWS)
+		{
+			do
+			{
+				c = skipToNonWhite ();
+				if (isident1 (c))
+				{
+					readIdentifier (token, c);
+					c = skipToNonWhite ();
+				}
+			} while (c == '.'  ||  c == ',');
+		}
+		else
+			break;
+	}
+	cppUngetc (c);
+	setToken (st, TOKEN_NONE);
+}
+
 static void analyzePostParens (statementInfo *const st, parenInfo *const info)
 {
 	const unsigned long inputLineNumber = getInputLineNumber ();
@@ -2285,8 +2326,10 @@ static void analyzePostParens (statementInfo *const st, parenInfo *const info)
 	cppUngetc (c);
 	if (isOneOf (c, "{;,="))
 		;
-	else if (isLanguage (Lang_java) || isLanguage (Lang_vala))
+	else if (isLanguage (Lang_java))
 		skipJavaThrows (st);
+	else if (isLanguage (Lang_vala))
+		skipValaPostParens(st);
 	else
 	{
 		if (! skipPostArgumentStuff (st, info))
@@ -2498,6 +2541,10 @@ static void analyzeParens (statementInfo *const st)
 			token->type = TOKEN_NAME;
 			processName (st);
 			st->gotParenName = TRUE;
+			if (isLanguage(Lang_d) && c == '(' && isType (prev, TOKEN_NAME)) {
+				st->declaration = DECL_FUNCTION_TEMPLATE;
+				copyToken (st->blockName, prev);
+			}
 		}
 		else if (! st->gotArgs  &&  info.isParamList)
 		{
@@ -2853,7 +2900,9 @@ static void tagCheck (statementInfo *const st)
 		{
 			if (isType (prev, TOKEN_ARGS))
 			{
-				if (st->haveQualifyingName)
+				if (st->declaration == DECL_FUNCTION_TEMPLATE)
+					qualifyFunctionTag (st, st->blockName);
+				else if (st->haveQualifyingName)
 				{
 					st->declaration = DECL_FUNCTION;
 					if (isType (prev2, TOKEN_NAME))
