@@ -1,5 +1,5 @@
 /*
-*   $Id: get.c 5136 2010-08-11 14:19:57Z ntrel $
+*   $Id: get.c 5295 2010-10-15 12:17:22Z ntrel $
 *
 *   Copyright (c) 1996-2002, Darren Hiebert
 *
@@ -33,7 +33,7 @@
 /*
 *   DATA DECLARATIONS
 */
-typedef enum { COMMENT_NONE, COMMENT_C, COMMENT_CPLUS } Comment;
+typedef enum { COMMENT_NONE, COMMENT_C, COMMENT_CPLUS, COMMENT_D } Comment;
 
 enum eCppLimits {
 	MaxCppNestingLevel = 20,
@@ -444,6 +444,8 @@ static Comment isComment (void)
 		comment = COMMENT_C;
 	else if (next == '/')
 		comment = COMMENT_CPLUS;
+	else if (next == '+')
+		comment = COMMENT_D;
 	else
 	{
 		fileUngetc (next);
@@ -495,6 +497,33 @@ static int skipOverCplusComment (void)
 	return c;
 }
 
+/* Skips over a D style comment.
+ * Really we should match nested /+ comments. At least they're less common.
+ */
+int skipOverDComment (void)
+{
+	int c = fileGetc ();
+
+	while (c != EOF)
+	{
+		if (c != '+')
+			c = fileGetc ();
+		else
+		{
+			const int next = fileGetc ();
+
+			if (next != '/')
+				c = next;
+			else
+			{
+				c = SPACE;  /* replace comment with space */
+				break;
+			}
+		}
+	}
+	return c;
+}
+
 /*  Skips to the end of a string, returning a special character to
  *  symbolically represent a generic string.
  */
@@ -614,6 +643,8 @@ process:
 					if (c == NEWLINE)
 						fileUngetc (c);
 				}
+				else if (comment == COMMENT_D)
+					c = skipOverDComment ();
 				else
 					Cpp.directive.accept = FALSE;
 				break;
