@@ -1,5 +1,5 @@
 /*
- *   $Id: latex.c 2287 2008-02-27 13:17:29Z eht16 $
+ *   $Id: latex.c 4437 2009-11-22 23:51:59Z eht16 $
  *
  *   Copyright (c) 2000-2001, Jérôme Plût
  *   Copyright (c) 2006, Enrico Tröger
@@ -80,8 +80,15 @@ static void createTag(int flags, TeXKind kind, const char * l)
  	l++;
     if (flags & (TEX_BRACES | TEX_LABEL))
     {
- 	if ((*(l++)) != '{')
+	if (*l == '[')
+	{
+	    while (*l != ']')
+		l++;
+	    l++; /* skip the closing square bracket */
+	}   
+	if (*l != '{')
  	    goto no_tag;
+	l++;
      }
      if (flags & TEX_BSLASH)
      {
