@@ -1,5 +1,5 @@
 /*
-*   $Id: python.c 3831 2009-06-01 15:41:24Z ntrel $
+*   $Id: python.c 5224 2010-09-13 15:03:18Z ntrel $
 *
 *   Copyright (c) 2000-2003, Darren Hiebert
 *
@@ -57,6 +57,30 @@ static boolean isIdentifierCharacter (int c)
 	return (boolean) (isalnum (c) || c == '_');
 }
 
+static const char *get_class_name_from_parent (const char *parent)
+{
+	const char *result;
+
+	if (parent == NULL)
+		return NULL;
+
+	result = strrchr (parent, '.');
+	if (result != NULL)
+	{
+		result++;
+		parent = result;
+	}
+
+	result = strrchr (parent, '/');
+	if (result != NULL)
+		result++;
+	else
+		result = parent;
+
+	return result;
+}
+
+
 /* Given a string with the contents of a line directly after the "def" keyword,
  * extract all relevant information and create a tag.
  */
@@ -69,6 +93,13 @@ static void makeFunctionTag (vString *const function,
 	tag.kindName = "function";
 	tag.kind = 'f';
 	tag.extensionFields.arglist = arglist;
+	/* add argument list of __init__() methods to the class tag */
+	if (strcmp (vStringValue (function), "__init__") == 0 && parent != NULL)
+	{
+		const char *parent_tag_name = get_class_name_from_parent (vStringValue (parent));
+		if (parent_tag_name != NULL)
+			setTagArglistByName (parent_tag_name, arglist);
+	}
 
 	if (vStringLength (parent) > 0)
 	{
@@ -166,7 +197,7 @@ static const char *skipEverything (const char *cp)
 {
 	for (; *cp; cp++)
 	{
-		if (*cp == '"' || *cp == '\'')
+		if (*cp == '"' || *cp == '\'' || *cp == '#')
 		{
 			cp = skipString(cp);
 			if (!*cp) break;
@@ -324,7 +355,8 @@ static void parseFunction (const char *cp, vString *const def,
 	cp = parseIdentifier (cp, def);
 	arglist = parseArglist (cp);
 	makeFunctionTag (def, parent, is_class_parent, arglist);
-	eFree (arglist);
+	if (arglist != NULL)
+		eFree (arglist);
 }
 
 /* Get the combined name of a nested symbol. Classes are separated with ".",
@@ -379,9 +411,9 @@ static void checkParent(NestingLevels *nls, int indent, vString *parent)
 	{
 		n = nls->levels + i;
 		/* is there a better way to compare two vStrings? */
-		if (strcmp(vStringValue(parent), vStringValue(n->name)) == 0)
+		if (n && strcmp(vStringValue(parent), vStringValue(n->name)) == 0)
 		{
-			if (n && indent <= n->indentation)
+			if (indent <= n->indentation)
 			{
 				/* remove this level by clearing its name */
 				vStringClear(n->name);
@@ -534,6 +566,49 @@ static const char *skipTypeDecl (const char *cp, boolean *is_class)
 	return NULL;
 }
 
+/* checks if there is a lambda at position of cp, and return its argument list
+ * if so.
+ * We don't return the lambda name since it is useless for now since we already
+ * know it when we call this function, and it would be a little slower. */
+static boolean varIsLambda (const char *cp, char **arglist)
+{
+	boolean is_lambda = FALSE;
+
+	cp = skipSpace (cp);
+	cp = skipIdentifier (cp); /* skip the lambda's name */
+	cp = skipSpace (cp);
+	if (*cp == '=')
+	{
+		cp++;
+		cp = skipSpace (cp);
+		if (strncmp (cp, "lambda", 6) == 0)
+		{
+			const char *tmp;
+
+			cp += 6; /* skip the lambda */
+			tmp = skipSpace (cp);
+			/* check if there is a space after lambda to detect assignations
+			 * starting with 'lambdaXXX' */
+			if (tmp != cp)
+			{
+				vString *args = vStringNew ();
+
+				cp = tmp;
+				vStringPut (args, '(');
+				for (; *cp != 0 && *cp != ':'; cp++)
+					vStringPut (args, *cp);
+				vStringPut (args, ')');
+				vStringTerminate (args);
+				if (arglist)
+					*arglist = strdup (vStringValue (args));
+				vStringDelete (args);
+				is_lambda = TRUE;
+			}
+		}
+	}
+	return is_lambda;
+}
+
 static void findPythonTags (void)
 {
 	vString *const continuation = vStringNew ();
@@ -656,6 +731,7 @@ static void findPythonTags (void)
 		if (variable)
 		{
 			const char *start = variable;
+			char *arglist;
 			boolean parent_is_class;
 
 			vStringClear (name);
@@ -667,11 +743,22 @@ static void findPythonTags (void)
 			vStringTerminate (name);
 
 			parent_is_class = constructParentString(nesting_levels, indent, parent);
-			/* skip variables in methods */
-			if (! parent_is_class && vStringLength(parent) > 0)
-				continue;
+			if (varIsLambda (variable, &arglist))
+			{
+				/* show class members or top-level script lambdas only */
+				if (parent_is_class || vStringLength(parent) == 0)
+					makeFunctionTag (name, parent, parent_is_class, arglist);
+				if (arglist != NULL)
+					eFree (arglist);
+			}
+			else
+			{
+				/* skip variables in methods */
+				if (! parent_is_class && vStringLength(parent) > 0)
+					continue;
 
-			makeVariableTag (name, parent);
+				makeVariableTag (name, parent);
+			}
 		}
 		/* Find and parse imports */
 		parseImports(line);
