@@ -1,5 +1,5 @@
 /*
-*   $Id: tcl.c 2380 2008-03-21 16:47:17Z eht16 $
+*   $Id: tcl.c 3900 2009-06-28 10:42:34Z eht16 $
 *
 *   Copyright (c) 2000-2003, Darren Hiebert
 *
@@ -24,13 +24,14 @@
 *   DATA DEFINITIONS
 */
 typedef enum {
-	K_CLASS, K_METHOD, K_PROCEDURE
+	K_CLASS, K_METHOD, K_PROCEDURE, K_MODULE
 } tclKind;
 
 static kindOption TclKinds [] = {
 	{ TRUE, 'c', "class",     "classes" },
-	{ TRUE, 'm', "member",    "methods" },
-	{ TRUE, 'p', "function",  "procedures" }
+	{ TRUE, 'f', "member",    "methods" },
+	{ TRUE, 'p', "function",  "procedures" },
+	{ TRUE, 'm', "namespace", "modules" }
 };
 
 /*
@@ -55,7 +56,16 @@ static const unsigned char *makeTclTag (
 
 static boolean match (const unsigned char *line, const char *word)
 {
-	return (boolean) (strncmp ((const char*) line, word, strlen (word)) == 0);
+	size_t len = strlen (word);
+	boolean matched = (strncmp ((const char*) line, word, len) == 0);
+
+	if (matched)
+	{
+		/* check that the word is followed by a space to avoid detecting something
+		 * like "proc_new ..." */
+		matched = isspace (*(line + len));
+	}
+	return matched;
 }
 
 static void findTclTags (void)
@@ -98,6 +108,29 @@ static void findTclTags (void)
 				cp = makeTclTag (cp, name, K_METHOD);
 			}
 		}
+		else if (match (line, "method"))
+		{
+			cp = makeTclTag (cp, name, K_METHOD);
+		}
+		else if (match (line, "oo::class") ) {
+			if (match (cp, "create"))
+			{
+				cp += 6;
+				while (isspace ((int) *cp))
+					++cp;
+				cp = makeTclTag (cp, name, K_CLASS);
+			}
+		}
+		else if (match (line, "namespace") ) {
+			if (match (cp, "eval"))
+			{
+				cp += 4;
+				while (isspace ((int) *cp))
+					++cp;
+				cp = makeTclTag (cp, name, K_MODULE);
+			}
+		}
+
 	}
 	vStringDelete (name);
 }
